@@ -63,7 +63,10 @@ namespace ApplePictParserInternal
 //! internal: low level class to store a region
 struct Region {
   //! constructor
-  Region() : m_bdBox(), m_points(), m_extra("")
+  Region()
+    : m_bdBox()
+    , m_points()
+    , m_extra("")
   {
   }
   //! operator<<
@@ -72,8 +75,8 @@ struct Region {
     o << rgn.m_bdBox << ",";
     if (!rgn.m_points.empty()) {
       o << "points=[";
-      for (size_t c = 0; c < rgn.m_points.size(); c++)
-        o << rgn.m_points[c] << ",";
+      for (auto const &pt : rgn.m_points)
+        o << pt << ",";
       o << "],";
     }
     o << rgn.m_extra;
@@ -89,8 +92,14 @@ struct Region {
 
 //!  Internal and low level: a class used to read pack/unpack black-white bitmap
 struct Bitmap {
-  Bitmap() : m_rowBytes(), m_rect(), m_src(), m_dst(), m_region(),
-    m_bitmap(), m_mode(0) {}
+  Bitmap()
+    : m_rowBytes()
+    , m_rect()
+    , m_src()
+    , m_dst()
+    , m_region()
+    , m_bitmap()
+    , m_mode(0) {}
 
   //! operator<< for Bitmap
   friend std::ostream &operator<< (std::ostream &o, Bitmap const &f)
@@ -109,7 +118,7 @@ struct Bitmap {
 
     while (rPos < sz) {
       if (rPos+2 > sz) return false;
-      signed char n = static_cast<signed char>(pData[rPos++]);
+      auto n = static_cast<signed char>(pData[rPos++]);
       if (n < 0) {
         int nCount = 1-n;
         if (wPos+size_t(nCount) > wNPos) return false;
@@ -155,7 +164,7 @@ struct Bitmap {
           m_bitmap[pos++]=data[j];
       }
       else {
-        int numB = static_cast<int>(input.readULong(szRowSize));
+        auto numB = static_cast<int>(input.readULong(szRowSize));
         if (numB < 0 || numB > 2*m_rowBytes) {
           MWAW_DEBUG_MSG(("ApplePictParserInternal::Bitmap::readBitmapData: odd numB:%d in row: %d/%d\n", numB, i, numRows));
           return false;
@@ -183,7 +192,7 @@ struct Bitmap {
     if (!bitmap.valid()) return false;
 
     for (int i = 0; i < nRows; i++)
-      bitmap.setRowPacked(i, &m_bitmap[size_t(i*m_rowBytes)]);
+      bitmap.setRowPacked(i, &m_bitmap[size_t(i*m_rowBytes)], &m_bitmap[0] + m_bitmap.size());
 
     return bitmap.getBinary(picture);
   }
@@ -192,7 +201,7 @@ struct Bitmap {
   int m_rowBytes;
   MWAWBox2i m_rect /** the bitmap rectangle */, m_src/** the initial dimension */, /** the final dimension */ m_dst  ;
   //! the region
-  shared_ptr<Region> m_region;
+  std::shared_ptr<Region> m_region;
   //! the bitmap
   std::vector<unsigned char> m_bitmap;
   //! the encoding mode ?
@@ -202,7 +211,9 @@ struct Bitmap {
 //! Internal and low level: a class used to read a color map in a Apple Pict
 struct ColorTable {
   //! constructor
-  ColorTable() : m_flags(0), m_colors() {}
+  ColorTable()
+    : m_flags(0)
+    , m_colors() {}
 
   //! tries to read a colortable
   bool read(MWAWInputStream &input)
@@ -216,8 +227,8 @@ struct ColorTable {
     for (size_t i = 0; i < size_t(n); i++) {
       input.readULong(2); // indexId: ignored
       unsigned char col[3];
-      for (int c = 0 ; c < 3; c++) {
-        col[c] = static_cast<unsigned char>(input.readULong(1));
+      for (unsigned char &c : col) {
+        c = static_cast<unsigned char>(input.readULong(1));
         input.readULong(1);
       }
       m_colors[i] = MWAWColor(col[0], col[1], col[2]);
@@ -247,10 +258,24 @@ struct ColorTable {
 
 //!  Internal and low level: a class used to read pack/unpack color pixmap (version 2)
 struct Pixmap {
-  Pixmap() : m_rowBytes(0), m_rect(), m_version(-1), m_packType(0),
-    m_packSize(0), m_pixelType(0), m_pixelSize(0), m_compCount(0),
-    m_compSize(0), m_planeBytes(0), m_colorTable(), m_src(), m_dst(),
-    m_region(), m_indices(), m_colors(), m_mode(0)
+  Pixmap()
+    : m_rowBytes(0)
+    , m_rect()
+    , m_version(-1)
+    , m_packType(0)
+    , m_packSize(0)
+    , m_pixelType(0)
+    , m_pixelSize(0)
+    , m_compCount(0)
+    , m_compSize(0)
+    , m_planeBytes(0)
+    , m_colorTable()
+    , m_src()
+    , m_dst()
+    , m_region()
+    , m_indices()
+    , m_colors()
+    , m_mode(0)
   {
     m_resolution[0] = m_resolution[1] = 0;
   }
@@ -275,7 +300,7 @@ struct Pixmap {
     int rPos = 0, wPos = 0, maxW = m_rowBytes+24;
     while (rPos < sz) {
       if (rPos+2 > sz) return false;
-      signed char n = static_cast<signed char>(pData[rPos++]);
+      auto n = static_cast<signed char>(pData[rPos++]);
       if (n < 0) {
         int nCount = 1-n;
         if (rPos+byteSz > sz || wPos+byteSz *nCount >= maxW) return false;
@@ -298,21 +323,50 @@ struct Pixmap {
     return wPos+8 >= nSize;
   }
 
+  MWAWColor extractColor(const std::vector<unsigned char> &data, size_t rIdx, size_t gIdx, size_t bIdx)
+  {
+    unsigned char r = rIdx < data.size() ? data[rIdx] : 0;
+    unsigned char g = gIdx < data.size() ? data[gIdx] : 0;
+    unsigned char b = bIdx < data.size() ? data[bIdx] : 0;
+    return MWAWColor(r, g, b);
+  }
+
+  //! computes the height based on available data
+  int computeHeight(MWAWInputStream &input, const int height, const int width, const bool packed, const int szRowSize) const
+  {
+    if (packed) {
+      const long pos = input.tell();
+      int h = 0;
+      for (; h < height && !input.isEnd(); ++h) {
+        auto len = long(input.readULong(szRowSize));
+        input.seek(len, librevenge::RVNG_SEEK_CUR);
+      }
+      input.seek(pos, librevenge::RVNG_SEEK_SET);
+      return h;
+    }
+    else {
+      const long remaining = input.size() - input.tell();
+      const long maxHeight = remaining / width + int(remaining % width > 1);
+      return std::min(height, int(maxHeight));
+    }
+  }
+
   //! parses the pixmap data zone
   bool readPixmapData(MWAWInputStream &input)
   {
-    int W = m_rect.size().x(), H = m_rect.size().y();
+    int W = m_rect.size().x();
 
     int szRowSize=1;
     if (m_rowBytes > 250) szRowSize = 2;
 
+    bool packed = !(m_rowBytes < 8 || m_packType == 1);
+    int H = computeHeight(input, m_rect.size().y(), W, packed, szRowSize);
+
     int nPlanes = 1, nBytes = 3, rowBytes = m_rowBytes;
     int numValuesByInt = 1;
-    int maxValues = (1 << m_pixelSize)-1;
     int numColors = m_colorTable.get() ? int(m_colorTable->m_colors.size()) : 0;
     int maxColorsIndex = -1;
 
-    bool packed = !(m_rowBytes < 8 || m_packType == 1);
     switch (m_pixelSize) {
     case 1:
     case 2:
@@ -356,13 +410,14 @@ struct Pixmap {
       MWAW_DEBUG_MSG(("ApplePictParserInternal::Pixmap::readPixmapData: do not known how to read pixelsize=%d \n", m_pixelSize));
       return false;
     }
+    const size_t dataSize = size_t(H) * size_t(W);
     if (m_pixelSize <= 8)
-      m_indices.resize(size_t(H*W));
+      m_indices.resize(dataSize);
     else {
       if (rowBytes != W * nBytes * nPlanes) {
         MWAW_DEBUG_MSG(("ApplePictParserInternal::Pixmap::readPixmapData: find W=%d pixelsize=%d, rowSize=%d\n", W, m_pixelSize, m_rowBytes));
       }
-      m_colors.resize(size_t(H*W));
+      m_colors.resize(dataSize);
     }
 
     std::vector<unsigned char> values;
@@ -380,7 +435,7 @@ struct Pixmap {
           values[j]=data[j];
       }
       else {   // ok, packed
-        int numB = static_cast<int>(input.readULong(szRowSize));
+        auto numB = static_cast<int>(input.readULong(szRowSize));
         if (numB < 0 || numB > 2*m_rowBytes) {
           MWAW_DEBUG_MSG(("ApplePictParserInternal::Pixmap::readPixmapData: odd numB:%d in row: %d/%d\n", numB, y, H));
           return false;
@@ -402,6 +457,7 @@ struct Pixmap {
       //
       int wPos = y*W;
       if (m_pixelSize <= 8) { // indexed
+        int maxValues = (1 << m_pixelSize)-1;
         for (int x = 0, rPos = 0; x < W;) {
           unsigned char val = values[size_t(rPos++)];
           for (int v = numValuesByInt-1; v >=0; v--) {
@@ -414,7 +470,9 @@ struct Pixmap {
       }
       else if (m_pixelSize == 16) {
         for (int x = 0, rPos = 0; x < W; x++) {
-          unsigned int val = 256*static_cast<unsigned int>(values[size_t(rPos)])+static_cast<unsigned int>(values[size_t(rPos+1)]);
+          unsigned int c1 = size_t(rPos) < values.size() ? values[size_t(rPos)] : 0;
+          unsigned int c2 = size_t(rPos+1) < values.size() ? values[size_t(rPos+1)] : 0;
+          unsigned int val = 256*c1 + c2;
           rPos+=2;
           m_colors[size_t(wPos++)]=MWAWColor((val>>7)& 0xF8, (val>>2) & 0xF8, static_cast<unsigned char>(val << 3));
         }
@@ -422,13 +480,13 @@ struct Pixmap {
       else if (nPlanes==1) {
         for (int x = 0, rPos = 0; x < W; x++) {
           if (nBytes==4) rPos++;
-          m_colors[size_t(wPos++)]=MWAWColor(values[size_t(rPos)], values[size_t(rPos+1)],  values[size_t(rPos+2)]);
+          m_colors[size_t(wPos++)]=extractColor(values, size_t(rPos), size_t(rPos+1),  size_t(rPos+2));
           rPos+=3;
         }
       }
       else {
         for (int x = 0, rPos = (nPlanes==4) ? W:0; x < W; x++) {
-          m_colors[size_t(wPos++)]=MWAWColor(values[size_t(rPos)], values[size_t(rPos+W)],  values[size_t(rPos+2*W)]);
+          m_colors[size_t(wPos++)]=extractColor(values, size_t(rPos), size_t(rPos+W),  size_t(rPos+2*W));
           rPos+=1;
         }
       }
@@ -502,11 +560,11 @@ struct Pixmap {
   int m_compCount /** logical components per pixels */;
   int m_compSize /** logical bits by components */;
   long m_planeBytes /** offset to the next plane */;
-  shared_ptr<ColorTable> m_colorTable /** the color table */;
+  std::shared_ptr<ColorTable> m_colorTable /** the color table */;
 
   MWAWBox2i m_src/** the initial dimension */, /** another final dimension */ m_dst  ;
   //! the region
-  shared_ptr<Region> m_region;
+  std::shared_ptr<Region> m_region;
   //! the pixmap indices
   std::vector<int> m_indices;
   //! the colors
@@ -519,12 +577,31 @@ struct Pixmap {
 //! Internal: the state of a ApplePictParser
 struct State {
   //! constructor
-  State() : m_version(0), m_bdBox(), m_origin(0,0), m_penPosition(0,0), m_textPosition(0,0),
-    m_penSize(1,1), m_ovalSize(1,1), m_penMode(0), m_textMode(0), m_isHiliteMode(false), m_font(3,12),
-    m_foreColor(MWAWColor::black()), m_backColor(MWAWColor::white()), m_hiliteColor(MWAWColor::black()), m_opColor(MWAWColor::black()),
-    m_penPattern(), m_backgroundPattern(), m_fillPattern(),
-    m_rectangle(), m_roundRectangle(), m_circle(), m_pie(), m_points(),
-    m_afterQuicktime(false)
+  State()
+    : m_version(0)
+    , m_bdBox()
+    , m_origin(0,0)
+    , m_penPosition(0,0)
+    , m_textPosition(0,0)
+    , m_penSize(1,1)
+    , m_ovalSize(1,1)
+    , m_penMode(0)
+    , m_textMode(0)
+    , m_isHiliteMode(false)
+    , m_font(3,12)
+    , m_foreColor(MWAWColor::black())
+    , m_backColor(MWAWColor::white())
+    , m_hiliteColor(MWAWColor::black())
+    , m_opColor(MWAWColor::black())
+    , m_penPattern()
+    , m_backgroundPattern()
+    , m_fillPattern()
+    , m_rectangle()
+    , m_roundRectangle()
+    , m_circle()
+    , m_pie()
+    , m_points()
+    , m_afterQuicktime(false)
   {
   }
   //! init the patterns list
@@ -652,35 +729,30 @@ struct State {
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a ApplePictParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(ApplePictParser &pars, MWAWInputStreamPtr input, MWAWEntry const &entry) : MWAWSubDocument(&pars, input, entry) {}
+  SubDocument(ApplePictParser &pars, MWAWInputStreamPtr const &input, MWAWEntry const &entry)
+    : MWAWSubDocument(&pars, input, entry) {}
 
   //! destructor
-  virtual ~SubDocument() {}
+  ~SubDocument() final {}
 
   //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const
+  bool operator!=(MWAWSubDocument const &doc) const final
   {
     if (MWAWSubDocument::operator!=(doc)) return true;
-    SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+    auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
     if (!sDoc) return true;
     return false;
   }
 
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
-  {
-    return !operator!=(doc);
-  }
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
-protected:
 private:
-  SubDocument(SubDocument const &orig);
-  SubDocument &operator=(SubDocument const &orig);
+  SubDocument(SubDocument const &orig) = delete;
+  SubDocument &operator=(SubDocument const &orig) = delete;
 };
 
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
@@ -689,7 +761,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
     MWAW_DEBUG_MSG(("ApplePictParserInternal::SubDocument::parse: no listener\n"));
     return;
   }
-  ApplePictParser *parser=dynamic_cast<ApplePictParser *>(m_parser);
+  auto *parser=dynamic_cast<ApplePictParser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("ApplePictParserInternal::SubDocument::parse: no parser\n"));
     return;
@@ -705,8 +777,9 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-ApplePictParser::ApplePictParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWGraphicParser(input, rsrcParser, header), m_state()
+ApplePictParser::ApplePictParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWGraphicParser(input, rsrcParser, header)
+  , m_state()
 {
   init();
 }
@@ -730,13 +803,13 @@ void ApplePictParser::init()
 ////////////////////////////////////////////////////////////
 void ApplePictParser::parse(librevenge::RVNGDrawingInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
-    checkHeader(0L);
+    checkHeader(nullptr);
     createDocument(docInterface);
     ok = createZones();
     ascii().reset();
@@ -809,7 +882,7 @@ bool ApplePictParser::readZone()
   int const opLength=version()>=2 ? 2 : 1;
   if (!input->checkPosition(pos+opLength))
     return false;
-  int opCode=static_cast<int>(input->readULong(opLength));
+  auto opCode=static_cast<int>(input->readULong(opLength));
   DrawingMethod drawingMethod = D_UNDEFINED;
   /* in general a basic bitmap follow a Quicktime picture/movie,
      so let store the state to ignore some uneeded bitmap/pixmap
@@ -865,7 +938,7 @@ bool ApplePictParser::readZone()
     if (!input->checkPosition(actPos+1))
       return false;
     f << "Entries(TextFace):";
-    int flag=static_cast<int>(input->readULong(1));
+    auto flag=static_cast<int>(input->readULong(1));
     uint32_t flags=0;
     if (flag&0x1) {
       flags |= MWAWFont::boldBit;
@@ -906,7 +979,7 @@ bool ApplePictParser::readZone()
   case 8: {
     if (!input->checkPosition(actPos+2))
       return false;
-    int mode=static_cast<int>(input->readULong(2));
+    auto mode=static_cast<int>(input->readULong(2));
     if (opCode==5) {
       f << "Entries(TextMode):";
       m_state->m_textMode=mode;
@@ -1190,9 +1263,9 @@ bool ApplePictParser::readZone()
     f << "dSz=" << dSz << ",";
     if (!input->checkPosition(2+dSz+actPos))
       return false;
-    int id=static_cast<int>(input->readULong(2));
+    auto id=static_cast<int>(input->readULong(2));
     f << "id=" << id << ",";
-    int sSz=static_cast<int>(input->readULong(1));
+    auto sSz=static_cast<int>(input->readULong(1));
     if (sSz>dSz+3) {
       MWAW_DEBUG_MSG(("ApplePictParser::readZone: font name size seems bad\n"));
       sSz=int(dSz-3);
@@ -1310,7 +1383,7 @@ bool ApplePictParser::readZone()
       return false;
     f << "Entries(Pie)[" << getDrawingName(drawingMethod) << "]:same";
     int angles[2];
-    for (int i=0; i<2; ++i) angles[i]=static_cast<int>(input->readLong(2));
+    for (int &angle : angles) angle=static_cast<int>(input->readLong(2));
     drawPie(drawingMethod, angles[0], angles[1]);
     break;
   }
@@ -1545,7 +1618,7 @@ bool ApplePictParser::readAndDrawRectangle(ApplePictParser::DrawingMethod method
   libmwaw::DebugStream f;
   f << "Entries(Rectangle)[" << getDrawingName(method) << "]:";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   m_state->m_rectangle=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   f << m_state->m_rectangle;
   drawRectangle(method);
@@ -1563,7 +1636,7 @@ bool ApplePictParser::readAndDrawRoundRectangle(ApplePictParser::DrawingMethod m
   libmwaw::DebugStream f;
   f << "Entries(RoundRect)[" << getDrawingName(method) << "]:";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   m_state->m_roundRectangle=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   f << m_state->m_roundRectangle;
   drawRoundRectangle(method);
@@ -1581,7 +1654,7 @@ bool ApplePictParser::readAndDrawCircle(ApplePictParser::DrawingMethod method)
   libmwaw::DebugStream f;
   f << "Entries(Circle)[" << getDrawingName(method) << "]:";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   m_state->m_circle=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   f << m_state->m_circle;
   drawCircle(method);
@@ -1599,11 +1672,11 @@ bool ApplePictParser::readAndDrawPie(ApplePictParser::DrawingMethod method)
   libmwaw::DebugStream f;
   f << "Entries(Pie)[" << getDrawingName(method) << "]:";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   m_state->m_pie=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   f << m_state->m_pie << ",";
   int angles[2];
-  for (int i=0; i<2; ++i) angles[i]=static_cast<int>(input->readLong(2));
+  for (int &angle : angles) angle=static_cast<int>(input->readLong(2));
   f << "angl=" << angles[0] << "x" << angles[0]+angles[1] << ",";
   drawPie(method, angles[0], angles[1]);
   ascii().addPos(pos-(version()==1 ? 1 : 2));
@@ -1615,22 +1688,22 @@ bool ApplePictParser::readAndDrawPolygon(ApplePictParser::DrawingMethod method)
 {
   MWAWInputStreamPtr input = getInput();
   long pos=input->tell();
-  int dSz=static_cast<int>(input->readULong(2));
+  auto dSz=static_cast<int>(input->readULong(2));
   if (dSz<10 || (dSz%4)!=2 || !input->checkPosition(pos+dSz))
     return false;
   libmwaw::DebugStream f;
   f << "Entries(Polygon)[" << getDrawingName(method) << "]:";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   f << MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2])) << ",";
   int N=(dSz-10)/4;
   f << "pts=[";
   m_state->m_points.clear();
   for (int i=0; i<N; ++i) {
-    float coord[2];
-    for (int j=0; j<2; ++j) coord[j]=float(input->readLong(2));
-    m_state->m_points.push_back(MWAWVec2f(coord[1],coord[0]));
-    f << MWAWVec2f(coord[1],coord[0]) << ",";
+    int coord[2];
+    for (int &j : coord) j=int(input->readLong(2));
+    m_state->m_points.push_back(MWAWVec2i(coord[1],coord[0]));
+    f << MWAWVec2i(coord[1],coord[0]) << ",";
   }
   f << "],";
   drawPolygon(method);
@@ -1646,7 +1719,7 @@ bool ApplePictParser::readRGBColor(MWAWColor &color)
   if (!input->checkPosition(pos+6))
     return false;
   uint8_t col[3];
-  for (int i=0; i<3; ++i) col[i]=uint8_t(input->readULong(2)>>8);
+  for (uint8_t &i : col) i=uint8_t(input->readULong(2)>>8);
   color=MWAWColor(col[0],col[1],col[2]);
   return true;
 }
@@ -1673,7 +1746,7 @@ bool ApplePictParser::readColorPattern(MWAWGraphicStyle::Pattern &pat)
   long pos=input->tell();
   if (!input->checkPosition(pos+10))
     return false;
-  int type = static_cast<int>(input->readULong(2));
+  auto type = static_cast<int>(input->readULong(2));
   if (type !=1 && type != 2) {
     MWAW_DEBUG_MSG(("ApplePictParser::readColorPattern: unknown type=%d... \n", type));
     return false;
@@ -1704,7 +1777,7 @@ bool ApplePictParser::readAndDrawText(std::string &text)
   long pos=input->tell();
   if (!input->checkPosition(pos+1))
     return false;
-  int dSz=static_cast<int>(input->readULong(1));
+  auto dSz=static_cast<int>(input->readULong(1));
   if (!input->checkPosition(pos+1+dSz))
     return false;
   text="";
@@ -1719,10 +1792,10 @@ bool ApplePictParser::readAndDrawText(std::string &text)
     MWAW_DEBUG_MSG(("ApplePictParser::readAndDrawText: can not find the listener\n"));
     return true;
   }
-  shared_ptr<MWAWSubDocument> doc(new ApplePictParserInternal::SubDocument(*this, getInput(), entry));
+  std::shared_ptr<MWAWSubDocument> doc(new ApplePictParserInternal::SubDocument(*this, getInput(), entry));
   MWAWGraphicStyle style;
   m_state->updateStyle(D_TEXT, style);
-  MWAWVec2f orig=m_state->m_textPosition;
+  MWAWVec2f orig(m_state->m_textPosition);
   orig[1]=orig[1]-m_state->m_font.size();
   MWAWPosition position;
   m_state->updatePosition(orig, position);
@@ -1751,7 +1824,7 @@ bool ApplePictParser::readBitmap(ApplePictParserInternal::Bitmap &bitmap, bool p
   // and the two general rectangle src, dst
   for (int c = 0; c < 3; c++) {
     int val[4];
-    for (int d = 0; d < 4; d++) val[d] = static_cast<int>(input->readLong(2));
+    for (int &d : val) d = static_cast<int>(input->readLong(2));
     MWAWBox2i box(MWAWVec2i(val[1],val[0]), MWAWVec2i(val[3],val[2]));
     if (box.size().x() <= 0 || box.size().y() <= 0) {
       MWAW_DEBUG_MSG(("ApplePictParser::readBitmap: find odd rectangle %d... \n", c));
@@ -1773,7 +1846,7 @@ bool ApplePictParser::readBitmap(ApplePictParserInternal::Bitmap &bitmap, bool p
   }
 
   if (hasRegion) { // CHECKME...
-    shared_ptr<ApplePictParserInternal::Region>rgn(new ApplePictParserInternal::Region);
+    std::shared_ptr<ApplePictParserInternal::Region>rgn(new ApplePictParserInternal::Region);
     if (!readRegion(*rgn)) return false;
     bitmap.m_region = rgn;
   }
@@ -1803,7 +1876,7 @@ bool ApplePictParser::readPixmap(ApplePictParserInternal::Pixmap &pixmap, bool p
 
   // read the rectangle: bound
   int val[4];
-  for (int d = 0; d < 4; d++) val[d] = static_cast<int>(input->readLong(2));
+  for (int &d : val) d = static_cast<int>(input->readLong(2));
   pixmap.m_rect = MWAWBox2i(MWAWVec2i(val[1],val[0]), MWAWVec2i(val[3],val[2]));
   if (pixmap.m_rect.size().x() <= 0 || pixmap.m_rect.size().y() <= 0) {
     MWAW_DEBUG_MSG(("ApplePictParser::readPixmap: find odd bound rectangle ... \n"));
@@ -1812,8 +1885,8 @@ bool ApplePictParser::readPixmap(ApplePictParserInternal::Pixmap &pixmap, bool p
   pixmap.m_version = static_cast<int>(input->readLong(2));
   pixmap.m_packType = static_cast<int>(input->readLong(2));
   pixmap.m_packSize = static_cast<int>(input->readLong(4));
-  for (int c = 0; c < 2; c++) {
-    pixmap.m_resolution[c] = static_cast<int>(input->readLong(2));
+  for (int &c : pixmap.m_resolution) {
+    c = static_cast<int>(input->readLong(2));
     input->readLong(2);
   }
   pixmap.m_pixelType = static_cast<int>(input->readLong(2));
@@ -1840,7 +1913,7 @@ bool ApplePictParser::readPixmap(ApplePictParserInternal::Pixmap &pixmap, bool p
   if (hasRectsMode) {
     for (int c = 0; c < 2; c++) {
       int dim[4];
-      for (int d = 0; d < 4; d++) dim[d] = static_cast<int>(input->readLong(2));
+      for (int &d : dim) d = static_cast<int>(input->readLong(2));
       MWAWBox2i box(MWAWVec2i(dim[1],dim[0]), MWAWVec2i(dim[3],dim[2]));
       if (box.size().x() <= 0 || box.size().y() <= 0) {
         MWAW_DEBUG_MSG(("ApplePictParser::readPixmap: find odd rectangle %d... \n", c));
@@ -1854,7 +1927,7 @@ bool ApplePictParser::readPixmap(ApplePictParserInternal::Pixmap &pixmap, bool p
   }
 
   if (hasRegion) { // CHECKME...
-    shared_ptr<ApplePictParserInternal::Region> rgn(new ApplePictParserInternal::Region);
+    std::shared_ptr<ApplePictParserInternal::Region> rgn(new ApplePictParserInternal::Region);
     if (!readRegion(*rgn)) return false;
     pixmap.m_region = rgn;
   }
@@ -1874,13 +1947,13 @@ bool ApplePictParser::readQuicktime(MWAWEmbeddedObject &object, MWAWBox2f &bdBox
   long pos=input->tell();
   if (!input->checkPosition(pos+4))
     return false;
-  long dSz=static_cast<long>(input->readULong(4));
+  auto dSz=static_cast<long>(input->readULong(4));
   long endPos=pos+4+dSz;
   if (dSz<68 || !input->checkPosition(endPos))
     return false;
   libmwaw::DebugStream f;
   f << "Entries(Quicktime):";
-  int val=static_cast<int>(input->readULong(2));
+  auto val=static_cast<int>(input->readULong(2));
   if (val) f << "vers=" << val << ",";
   float matrix[9];
   f << "mat=[";
@@ -1900,17 +1973,17 @@ bool ApplePictParser::readQuicktime(MWAWEmbeddedObject &object, MWAWBox2f &bdBox
     f << "###w,";
     matrix[8]=1;
   }
-  long matteSize=static_cast<long>(input->readULong(4));
+  auto matteSize=static_cast<long>(input->readULong(4));
   if (matteSize)
     f << "matteSize=" << std::hex << matteSize << std::dec << ",";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   if (dim[2]!=dim[0] || dim[3]!=dim[2])
     f << "matteRect=" << MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2])) << ",";
 
-  int mode=static_cast<int>(input->readULong(2));
+  auto mode=static_cast<int>(input->readULong(2));
   if (mode) f << "mode=" << mode << ",";
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   f << "srcRec=" << MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2])) << ",";
   // assume that the matrix contains only some scaling and/or translation
   if (matrix[1]<0 || matrix[1]>0  || matrix[2]<0 || matrix[2]>0 ||
@@ -1929,7 +2002,7 @@ bool ApplePictParser::readQuicktime(MWAWEmbeddedObject &object, MWAWBox2f &bdBox
   bdBox=MWAWBox2f(MWAWVec2f(dimF[1],dimF[0]),MWAWVec2f(dimF[3],dimF[2]));
   val=static_cast<int>(input->readULong(4));
   if (val) f << "accuracy=" << val << ",";
-  long maskSize=static_cast<long>(input->readULong(4));
+  auto maskSize=static_cast<long>(input->readULong(4));
   if (maskSize)
     f << "maskSize=" << std::hex << maskSize << std::dec << ",";
   ascii().addPos(pos-2);
@@ -2007,10 +2080,10 @@ bool ApplePictParser::readQuicktime(MWAWEmbeddedObject &object, MWAWBox2f &bdBox
   for (int i=0; i<2; ++i) dim[i]=static_cast<int>(input->readLong(2));
   f << "src[sz]=" << MWAWVec2i(dim[0],dim[1]) << ",";
   f << "res=" << double(input->readLong(4))/65536. << "x" << double(input->readLong(4))/65536. << ",";
-  long dataSize=static_cast<long>(input->readULong(4));
+  auto dataSize=static_cast<long>(input->readULong(4));
   f << "dataSize=" << std::hex << dataSize << std::dec << ",";
   f << "frame[count]=" << input->readULong(2) << ",";
-  int sSz=static_cast<int>(input->readULong(1));
+  auto sSz=static_cast<int>(input->readULong(1));
   if (sSz>31 || input->tell()+sSz>pos+dSz) {
     MWAW_DEBUG_MSG(("ApplePictParser::readQuicktime: can not read the compression name\n"));
     f << "###";
@@ -2070,16 +2143,16 @@ bool ApplePictParser::readRegion(ApplePictParserInternal::Region &region)
   long pos=input->tell();
   if (!input->checkPosition(pos+10))
     return false;
-  int dSz=static_cast<int>(input->readULong(2));
+  auto dSz=static_cast<int>(input->readULong(2));
   if (dSz<10||!input->checkPosition(pos+dSz))
     return false;
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   region.m_bdBox=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   int remain=(dSz-10)/2;
   // une liste de point dans la box: x1, y1, .. yn 0x7fff, x2, ... 0x7fff
   while (remain > 0) {
-    int y = static_cast<int>(input->readLong(2));
+    auto y = static_cast<int>(input->readLong(2));
     --remain;
     if (y == 0x7fff) break;
     if (y < region.m_bdBox[0].y() || y > region.m_bdBox[1].y()) {
@@ -2088,7 +2161,7 @@ bool ApplePictParser::readRegion(ApplePictParserInternal::Region &region)
     }
     bool endF = false;
     while (remain > 0) {
-      int x = static_cast<int>(input->readLong(2));
+      auto x = static_cast<int>(input->readLong(2));
       --remain;
       if (x == 0x7fff) {
         endF = true;
@@ -2203,19 +2276,20 @@ bool ApplePictParser::checkHeader(MWAWHeader *header, bool strict)
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     libmwaw::DebugStream f;
     f << "FileHeader:";
-    int dSz=static_cast<int>(input->readULong(2));
+    auto dSz=static_cast<int>(input->readULong(2));
     if (dSz)
       f << "dSz=" << dSz << ",";
     int dim[4];
-    for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+    for (int &i : dim) i=static_cast<int>(input->readLong(2));
     if (dim[0]>dim[2]||dim[1]>dim[3])
       continue;
-    m_state->m_bdBox=MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
+    m_state->m_bdBox=MWAWBox2f(MWAWVec2f(float(dim[1]),float(dim[0])),MWAWVec2f(float(dim[3]),float(dim[2])));
     f << "dim=" << m_state->m_bdBox << ",";
     long lastFlag = input->readLong(2);
     switch (lastFlag) {
     case 0x1101:
-      if (!input->checkPosition(pos+dSz))
+      // some MacDraw Pro pict stores a bigger size 512(file header?)+10?
+      if (!input->checkPosition(pos+dSz) && (st!=1 && input->size()!=dSz+512+2))
         break;
       f << "pict1,";
       vers = 1;
@@ -2230,7 +2304,7 @@ bool ApplePictParser::checkHeader(MWAWHeader *header, bool strict)
       switch (fileVersion) {
       case 1:
         f << "pict2[1:" << subvers << "],";
-        for (int i=0; i<4; ++i) dim2[i]=float(input->readLong(4))/65536.f;
+        for (float &i : dim2) i=float(input->readLong(4))/65536.f;
         if (strict && (dim2[0]>dim2[2]||dim2[1]>dim2[3]))
           break;
         m_state->m_bdBox=MWAWBox2f(MWAWVec2f(static_cast<float>(dim2[0]),static_cast<float>(dim2[1])),MWAWVec2f(static_cast<float>(dim2[2]),static_cast<float>(dim2[3])));
@@ -2242,7 +2316,7 @@ bool ApplePictParser::checkHeader(MWAWHeader *header, bool strict)
         for (int i=0; i<2; ++i) dim2[i]=float(input->readLong(4))/65536.f;
         if (strict && (dim2[0]<0 || dim2[1]<=0)) break;
         f << "res=" << MWAWVec2f(dim2[1],dim2[0]) << ",";
-        for (int i=0; i<4; ++i) dim[i]=static_cast<int>(input->readLong(2));
+        for (int &i : dim) i=static_cast<int>(input->readLong(2));
         if (dim[0]>dim[2]||dim[1]>dim[3])
           break;
         MWAWBox2f bdbox(MWAWVec2f(static_cast<float>(dim[1]),static_cast<float>(dim[0])),MWAWVec2f(static_cast<float>(dim[3]),static_cast<float>(dim[2])));
@@ -2293,14 +2367,14 @@ bool ApplePictParser::checkHeader(MWAWHeader *header, bool strict)
 ////////////////////////////////////////////////////////////
 void ApplePictParser::drawLine(MWAWVec2i const &pt)
 {
-  MWAWVec2f orig=m_state->m_penPosition;
+  MWAWVec2f orig(m_state->m_penPosition);
   m_state->m_penPosition=pt;
   MWAWListenerPtr listener=getGraphicListener();
   if (m_state->isInvisible(D_FRAME) || !listener)
     return;
   MWAWGraphicStyle style;
   m_state->updateStyle(D_FRAME, style);
-  MWAWGraphicShape shape=MWAWGraphicShape::line(orig, pt);
+  auto shape=MWAWGraphicShape::line(orig, MWAWVec2f(pt));
   MWAWPosition pos;
   m_state->updatePosition(shape.getBdBox(), pos);
   listener->insertShape(pos,shape, style);
@@ -2313,10 +2387,10 @@ void ApplePictParser::drawRectangle(ApplePictParser::DrawingMethod method)
     return;
   MWAWGraphicStyle style;
   m_state->updateStyle(method, style);
-  MWAWBox2f rect=m_state->m_rectangle;
+  MWAWBox2f rect(m_state->m_rectangle);
   if (method==D_ERASE) // the rectangle can be very large
     rect=rect.getIntersection(m_state->m_bdBox);
-  MWAWGraphicShape shape=MWAWGraphicShape::rectangle(rect);
+  auto shape=MWAWGraphicShape::rectangle(rect);
   MWAWPosition pos;
   m_state->updatePosition(shape.getBdBox(), pos);
   listener->insertShape(pos,shape, style);
@@ -2329,7 +2403,7 @@ void ApplePictParser::drawRoundRectangle(ApplePictParser::DrawingMethod method)
     return;
   MWAWGraphicStyle style;
   m_state->updateStyle(D_FRAME, style);
-  MWAWGraphicShape shape=MWAWGraphicShape::rectangle(m_state->m_roundRectangle, m_state->m_ovalSize);
+  auto shape=MWAWGraphicShape::rectangle(MWAWBox2f(m_state->m_roundRectangle), MWAWVec2f(m_state->m_ovalSize));
   MWAWPosition pos;
   m_state->updatePosition(shape.getBdBox(), pos);
   listener->insertShape(pos,shape, style);
@@ -2342,7 +2416,7 @@ void ApplePictParser::drawCircle(ApplePictParser::DrawingMethod method)
     return;
   MWAWGraphicStyle style;
   m_state->updateStyle(method, style);
-  MWAWGraphicShape shape=MWAWGraphicShape::circle(m_state->m_circle);
+  auto shape=MWAWGraphicShape::circle(MWAWBox2f(m_state->m_circle));
   MWAWPosition pos;
   m_state->updatePosition(shape.getBdBox(), pos);
   listener->insertShape(pos,shape, style);
@@ -2396,12 +2470,12 @@ void ApplePictParser::drawPie(DrawingMethod method, int startAngle, int dAngle)
     if (actVal[1] < minVal[1]) minVal[1] = actVal[1];
     else if (actVal[1] > maxVal[1]) maxVal[1] = actVal[1];
   }
-  MWAWVec2f center = m_state->m_pie.center();
+  MWAWVec2f center(m_state->m_pie.center());
   MWAWBox2f realBox(MWAWVec2f(center[0]+minVal[0],center[1]+minVal[1]),
                     MWAWVec2f(center[0]+maxVal[0],center[1]+maxVal[1]));
-  MWAWGraphicShape shape = method==D_FRAME ?
-                           MWAWGraphicShape::arc(realBox, m_state->m_pie, MWAWVec2f(float(angle[0]),float(angle[1]))) :
-                           MWAWGraphicShape::pie(realBox, m_state->m_pie, MWAWVec2f(float(angle[0]),float(angle[1])));
+  auto shape = method==D_FRAME ?
+               MWAWGraphicShape::arc(realBox, MWAWBox2f(m_state->m_pie), MWAWVec2f(float(angle[0]),float(angle[1]))) :
+               MWAWGraphicShape::pie(realBox, MWAWBox2f(m_state->m_pie), MWAWVec2f(float(angle[0]),float(angle[1])));
   MWAWPosition pos;
   m_state->updatePosition(shape.getBdBox(), pos);
   listener->insertShape(pos,shape, style);
@@ -2419,11 +2493,11 @@ void ApplePictParser::drawPolygon(ApplePictParser::DrawingMethod method)
   // first compute the bdbox
   MWAWGraphicShape shape;
   shape.m_type=MWAWGraphicShape::Polygon;
-  MWAWBox2f box(m_state->m_points[0],m_state->m_points[0]);
-  shape.m_vertices.push_back(m_state->m_points[0]);
+  MWAWBox2f box(MWAWVec2f(m_state->m_points[0]),MWAWVec2f(m_state->m_points[0]));
+  shape.m_vertices.push_back(MWAWVec2f(m_state->m_points[0]));
   for (size_t i=1; i<m_state->m_points.size(); ++i) {
-    box=box.getUnion(MWAWBox2f(m_state->m_points[i],m_state->m_points[i]));
-    shape.m_vertices.push_back(m_state->m_points[i]);
+    box=box.getUnion(MWAWBox2f(MWAWVec2f(m_state->m_points[i]),MWAWVec2f(m_state->m_points[i])));
+    shape.m_vertices.push_back(MWAWVec2f(m_state->m_points[i]));
   }
   shape.m_bdBox=box;
   MWAWGraphicStyle style;
@@ -2452,7 +2526,7 @@ void ApplePictParser::drawText(MWAWEntry const &entry)
   input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
   long endPos=entry.end();
   while (!input->isEnd() && input->tell()<endPos) {
-    char c = static_cast<char>(input->readULong(1));
+    auto c = static_cast<char>(input->readULong(1));
     if (c==0) {
       MWAW_DEBUG_MSG(("ApplePictParser::drawText: find char 0\n"));
       continue;
@@ -2483,9 +2557,9 @@ void ApplePictParser::drawBitmap(ApplePictParserInternal::Bitmap const &bitmap)
   MWAWGraphicStyle style;
   MWAWPosition pos;
   if (bitmap.m_dst.size()[0]>0 && bitmap.m_dst.size()[1]>0)
-    m_state->updatePosition(bitmap.m_dst, pos);
+    m_state->updatePosition(MWAWBox2f(bitmap.m_dst), pos);
   else
-    m_state->updatePosition(bitmap.m_rect, pos);
+    m_state->updatePosition(MWAWBox2f(bitmap.m_rect), pos);
   listener->insertPicture(pos,picture);
 }
 
@@ -2499,7 +2573,7 @@ void ApplePictParser::drawPixmap(ApplePictParserInternal::Pixmap const &pixmap)
     return;
   MWAWGraphicStyle style;
   MWAWPosition pos;
-  m_state->updatePosition(pixmap.m_dst, pos);
+  m_state->updatePosition(MWAWBox2f(pixmap.m_dst), pos);
   listener->insertPicture(pos,picture);
 }
 
