@@ -59,7 +59,10 @@ namespace BeagleWksBMParserInternal
 //! Internal: a bitmap of a BeagleWksBMParser
 struct Bitmap {
   //! the constructor
-  Bitmap() : m_colorBytes(1), m_dim(0,0), m_colorList()
+  Bitmap()
+    : m_colorBytes(1)
+    , m_dim(0,0)
+    , m_colorList()
   {
   }
   //! the number of color bytes (1: means black/white, 8: means color)
@@ -74,8 +77,15 @@ struct Bitmap {
 //! Internal: the state of a BeagleWksBMParser
 struct State {
   //! constructor
-  State() :  m_graphicBegin(-1), m_bitmap(), m_pict(), m_typeEntryMap(),
-    m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0)
+  State()
+    :  m_graphicBegin(-1)
+    , m_bitmap()
+    , m_pict()
+    , m_typeEntryMap()
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_headerHeight(0)
+    , m_footerHeight(0)
   {
   }
 
@@ -84,7 +94,7 @@ struct State {
   /** the bitmap data */
   Bitmap m_bitmap;
   /// the bitmap
-  shared_ptr<MWAWPict> m_pict;
+  std::shared_ptr<MWAWPict> m_pict;
   /** the type entry map */
   std::multimap<std::string, MWAWEntry> m_typeEntryMap;
 
@@ -100,8 +110,10 @@ struct State {
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-BeagleWksBMParser::BeagleWksBMParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWGraphicParser(input, rsrcParser, header), m_state(), m_structureManager()
+BeagleWksBMParser::BeagleWksBMParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWGraphicParser(input, rsrcParser, header)
+  , m_state()
+  , m_structureManager()
 {
   init();
 }
@@ -162,14 +174,14 @@ void BeagleWksBMParser::newPage(int number)
 ////////////////////////////////////////////////////////////
 void BeagleWksBMParser::parse(librevenge::RVNGDrawingInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
 
-    checkHeader(0L);
+    checkHeader(nullptr);
     ok = createZones();
     if (ok) {
       createDocument(docInterface);
@@ -241,7 +253,7 @@ bool BeagleWksBMParser::createZones()
     MWAWEntry entry;
     entry.setBegin(input->readLong(4));
     entry.setLength(input->readLong(4));
-    entry.setId((int) input->readLong(2));
+    entry.setId(static_cast<int>(input->readLong(2)));
     if (entry.length()==0) continue;
     entry.setType(i==1?"Frame":"Unknown");
     f << entry.type() << "[" << entry.id() << "]="
@@ -266,8 +278,7 @@ bool BeagleWksBMParser::createZones()
   ascii().addNote(f.str().c_str());
 
   // now parse the different zones
-  std::multimap<std::string, MWAWEntry>::iterator it;
-  it=m_state->m_typeEntryMap.find("FontNames");
+  auto it=m_state->m_typeEntryMap.find("FontNames");
   if (it!=m_state->m_typeEntryMap.end())
     m_structureManager->readFontNames(it->second);
   it=m_state->m_typeEntryMap.find("Frame");
@@ -294,6 +305,10 @@ bool BeagleWksBMParser::createZones()
     return false;
   }
 
+  pos = input->tell();
+  if (!readBitmap(true))
+    return false;
+  input->seek(pos, librevenge::RVNG_SEEK_SET);
   if (!readBitmap() || !m_state->m_pict) return false;
   if (!input->isEnd()) {
     MWAW_DEBUG_MSG(("BeagleWksBMParser::createZones: find some extra data\n"));
@@ -309,12 +324,11 @@ bool BeagleWksBMParser::readRSRCZones()
   if (!rsrcParser)
     return true;
 
-  std::multimap<std::string, MWAWEntry> &entryMap = rsrcParser->getEntriesMap();
-  std::multimap<std::string, MWAWEntry>::iterator it;
+  auto &entryMap = rsrcParser->getEntriesMap();
   // the 1 zone
-  char const *(zNames[]) = {"wPos", "DMPF" };
+  char const *zNames[] = {"wPos", "DMPF" };
   for (int z = 0; z < 2; ++z) {
-    it = entryMap.lower_bound(zNames[z]);
+    auto it = entryMap.lower_bound(zNames[z]);
     while (it != entryMap.end()) {
       if (it->first != zNames[z])
         break;
@@ -342,14 +356,19 @@ bool BeagleWksBMParser::readRSRCZones()
 ////////////////////////////////////////////////////////////
 // read the bitmap data
 ////////////////////////////////////////////////////////////
-bool BeagleWksBMParser::readBitmap()
+bool BeagleWksBMParser::readBitmap(bool onlyCheck)
 {
   MWAWInputStreamPtr input = getInput();
-  BeagleWksBMParserInternal::Bitmap &bitmap=m_state->m_bitmap;
+  auto &bitmap=m_state->m_bitmap;
   bool isColor=bitmap.m_colorBytes==8;
 
+  if (bitmap.m_dim[0] < 0 || bitmap.m_dim[1] < 0) {
+    MWAW_DEBUG_MSG(("BeagleWksBMParser::readBitmap: invalid bitmap dimensions\n"));
+    return false;
+  }
+
   long pos = input->tell();
-  long dSz=(long) input->readULong(2);
+  auto dSz=long(input->readULong(2));
   long endPos=pos+dSz;
   if (dSz<38 || !input->checkPosition(endPos)) {
     ascii().addPos(pos);
@@ -360,43 +379,43 @@ bool BeagleWksBMParser::readBitmap()
   libmwaw::DebugStream f;
   f << "Entries(Bitmap):";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=(int)input->readULong(2);
+  for (int &i : dim) i=static_cast<int>(input->readULong(2));
   if (dim[0]||dim[1]||dim[2]!=bitmap.m_dim[1]||dim[3]!=bitmap.m_dim[0])
     f << "dim=" << dim[1] << "x" << dim[0] << "<->" << dim[3] << "x" << dim[2] << ",";
   int val;
   for (int i=0; i<8; ++i) {
-    int const(expected[])= {0x11,0x2ff, 0xc00, -1, -1, 0, 0, 0 };
-    val=(int) input->readLong(2);
+    int const expected[] = {0x11,0x2ff, 0xc00, -1, -1, 0, 0, 0 };
+    val=static_cast<int>(input->readLong(2));
     if (val!=expected[i])
       f << "f" << i << "=" << val << ",";
   }
-  for (int i=0; i<4; ++i) dim[i]=(int)input->readULong(2);
+  for (int &i : dim) i=static_cast<int>(input->readULong(2));
   if (dim[0]||dim[2]||dim[1]!=bitmap.m_dim[0]||dim[3]!=bitmap.m_dim[1])
     f << "dim1=" << dim[0] << "x" << dim[2] << "<->" << dim[1] << "x" << dim[3] << ",";
   for (int i=0; i<5; ++i) {
-    int const(expected[])= {0, 0, 0, 0xa0, 0x82 };
-    val=(int) input->readLong(2);
+    int const expected[] = {0, 0, 0, 0xa0, 0x82 };
+    val=static_cast<int>(input->readLong(2));
     if (val!=expected[i])
       f << "g" << i << "=" << val << ",";
   }
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val==0x1e) {
     f << "f1e,";
-    val=(int) input->readULong(2);
+    val=static_cast<int>(input->readULong(2));
   }
   if (val!=1) {
     MWAW_DEBUG_MSG(("BeagleWksBMParser::readBitmap: we may have a problem\n"));
     f << "h0=" << val << ",";
   }
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val != 0xa) f << "h1=" << val << ",";
-  for (int i=0; i<4; ++i) dim[i]=(int)input->readULong(2);
+  for (int &i : dim) i=static_cast<int>(input->readULong(2));
   if (dim[0]||dim[1]||dim[2]!=bitmap.m_dim[1]||dim[3]!=bitmap.m_dim[0])
     f << "dim2=" << dim[1] << "x" << dim[0] << "<->" << dim[3] << "x" << dim[2] << ",";
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val != 0x98) f << "h2=" << val << ",";
   f << "fl?=" << std::hex << input->readULong(2) << std::dec << ","; // 48[BW]|8240[Color]
-  for (int i=0; i<4; ++i) dim[i]=(int)input->readULong(2);
+  for (int &i : dim) i=static_cast<int>(input->readULong(2));
   if (dim[0]||dim[1]||dim[2]!=bitmap.m_dim[1]||dim[3]!=bitmap.m_dim[0])
     f << "dim3=" << dim[1] << "x" << dim[0] << "<->" << dim[3] << "x" << dim[2] << ",";
   ascii().addPos(pos);
@@ -418,29 +437,32 @@ bool BeagleWksBMParser::readBitmap()
   f.str("");
   f << "Bitmap-A:";
   for (int j=0; j < 2; ++j) {
-    for (int i=0; i<4; ++i) dim[i]=(int)input->readULong(2);
+    for (int &i : dim) i=static_cast<int>(input->readULong(2));
     if (dim[0]||dim[1]||dim[2]!=bitmap.m_dim[1]||dim[3]!=bitmap.m_dim[0])
       f << "dim" << j << "=" << dim[1] << "x" << dim[0] << "<->" << dim[3] << "x" << dim[2] << ",";
   }
-  val=(int) input->readLong(2);
+  val=static_cast<int>(input->readLong(2));
   if (val) f << "f0=" << val << ",";
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
 
   int sSz=isColor ? 2 : 1;
-  int numColors=(int) bitmap.m_colorList.size();
-  shared_ptr<MWAWPictBitmapIndexed> pict(new MWAWPictBitmapIndexed(bitmap.m_dim));
-  if (isColor)
-    pict->setColors(bitmap.m_colorList);
-  else {
-    std::vector<MWAWColor> colors(2);
-    colors[0]=MWAWColor::white();
-    colors[1]=MWAWColor::black();
-    pict->setColors(colors);
+  auto numColors=static_cast<int>(bitmap.m_colorList.size());
+  std::shared_ptr<MWAWPictBitmapIndexed> pict;
+  if (!onlyCheck) {
+    pict = std::make_shared<MWAWPictBitmapIndexed>(bitmap.m_dim);
+    if (isColor)
+      pict->setColors(bitmap.m_colorList);
+    else {
+      std::vector<MWAWColor> colors(2);
+      colors[0]=MWAWColor::white();
+      colors[1]=MWAWColor::black();
+      pict->setColors(colors);
+    }
   }
   for (int r=0; r<bitmap.m_dim[1]; ++r) {
     pos=input->tell();
-    int sz=(int) input->readULong(sSz);
+    auto sz=static_cast<int>(input->readULong(sSz));
     f.str("");
     f << "Bitmap-R" << r << ":";
     long rEndPos=pos+sSz+sz;
@@ -452,36 +474,48 @@ bool BeagleWksBMParser::readBitmap()
       return false;
     }
     int col=0;
-    while (input->tell()<rEndPos) {
-      int wh=(int) input->readULong(1);
+    while (input->tell()<rEndPos) { // UnpackBits
+      auto wh=static_cast<int>(input->readULong(1));
       if (wh>=0x81) {
-        int color=(int) input->readULong(1);
+        auto color=static_cast<int>(input->readULong(1));
         for (int j=0; j < 0x101-wh; ++j) {
           if (col>=bitmap.m_dim[0]) break;
           if (isColor) {
             if (color>=numColors) break;
-            pict->set(col++, r, color);
+            if (onlyCheck)
+              ++col;
+            else
+              pict->set(col++, r, color);
           }
           else {
             for (int b=7; b>=0; --b) {
               if (col>=bitmap.m_dim[0]) break;
-              pict->set(col++, r, (color>>b)&1);
+              if (onlyCheck)
+                ++col;
+              else
+                pict->set(col++, r, (color>>b)&1);
             }
           }
         }
       }
-      else {
+      else { // checkme normally 0x80 is reserved and almost nobody used it (for ending the compression)
         for (int j=0; j < wh+1; ++j) {
-          int color=(int) input->readULong(1);
+          auto color=static_cast<int>(input->readULong(1));
           if (col>=bitmap.m_dim[0]) continue;
           if (isColor) {
             if (color>=numColors) break;
-            pict->set(col++, r, color);
+            if (onlyCheck)
+              ++col;
+            else
+              pict->set(col++, r, color);
           }
           else {
             for (int b=7; b>=0; --b) {
               if (col>=bitmap.m_dim[0]) break;
-              pict->set(col++, r, (color>>b)&1);
+              if (onlyCheck)
+                ++col;
+              else
+                pict->set(col++, r, (color>>b)&1);
             }
           }
         }
@@ -505,12 +539,13 @@ bool BeagleWksBMParser::readBitmap()
     ascii().addNote(f.str().c_str());
     input->seek(rEndPos, librevenge::RVNG_SEEK_SET);
   }
-  m_state->m_pict=pict;
+  if (!onlyCheck)
+    m_state->m_pict=pict;
   pos=input->tell();
   f.str("");
   f << "Bitmap-Data[end]:";
   if (pos+7==endPos) { // no sure when it exists
-    val=(int)input->readLong(1);
+    val=static_cast<int>(input->readLong(1));
     if (val) f << "unkn=" << val << ",";
   }
   if (input->tell()+6!=endPos) {
@@ -537,21 +572,21 @@ bool BeagleWksBMParser::readPrefColorMap()
     ascii().addNote("Entries(ColorMap)[pref]:###");
     return false;
   }
-  BeagleWksBMParserInternal::Bitmap &bitmap=m_state->m_bitmap;
+  auto &bitmap=m_state->m_bitmap;
   libmwaw::DebugStream f;
   f << "Entries(ColorMap)[pref,header]:";
-  int val=(int) input->readLong(2);
+  auto val=static_cast<int>(input->readLong(2));
   if (val) f << "f0=" << val << ",";
   f << "sz[bitmap]=" << std::hex << input->readULong(2) << std::dec << ",";
   for (int i=0; i<4; ++i) { // 0,0,8,8 ? f2-f3=colorMap size?
-    val=(int) input->readLong(1);
+    val=static_cast<int>(input->readLong(1));
     if (val) f << "f" << i+1 << "=" << val << ",";
   }
   int dim[2];
-  for (int i=0; i<2; ++i) dim[i]=(int) input->readLong(2);
+  for (int &i : dim) i=static_cast<int>(input->readLong(2));
   bitmap.m_dim=MWAWVec2i(dim[1], dim[0]);
   f << "dim=" << bitmap.m_dim << ",";
-  bitmap.m_colorBytes=(int) input->readLong(2);
+  bitmap.m_colorBytes=static_cast<int>(input->readLong(2));
   switch (bitmap.m_colorBytes) {
   case 1:
     f << "BW,";
@@ -565,10 +600,10 @@ bool BeagleWksBMParser::readPrefColorMap()
     break;
   }
   for (int i=0; i<11; ++i) {
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val) f << "g" << i << "=" << val << ",";
   }
-  int maxColor=(int) input->readULong(2);
+  auto maxColor=static_cast<int>(input->readULong(2));
   f << "N="<< maxColor+1 << ",";
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
@@ -581,10 +616,10 @@ bool BeagleWksBMParser::readPrefColorMap()
     f.str("");
     f << "ColorMap-P" << i << ":";
     pos = input->tell();
-    int id=(int) input->readLong(2);
+    auto id=static_cast<int>(input->readLong(2));
     if (id!=i) f << "#id=" << id << ",";
     unsigned char col[3];
-    for (int c = 0; c < 3; c++) col[c] = (unsigned char)(input->readULong(2)>>8);
+    for (unsigned char &c : col) c = static_cast<unsigned char>(input->readULong(2)>>8);
     MWAWColor color(col[0], col[1], col[2]);
     f << color << ",";
     ascii().addPos(pos);
@@ -607,15 +642,15 @@ bool BeagleWksBMParser::readColorMap()
   f << "ColorMap[header]:";
   int val;
   for (int i=0; i<14; ++i) { // always f4=72,f6=72,f9=8,f10=1,f11=8 ?
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   f << "id=" << std::hex << input->readULong(4) << std::dec << ",";
   for (int i=0; i<5; ++i) { // 0,0,0,sz?, 0
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val) f << "g" << i << "=" << std::hex << val << std::dec << ",";
   }
-  int maxColor=(int) input->readULong(2);
+  auto maxColor=static_cast<int>(input->readULong(2));
   f << "N="<< maxColor+1 << ",";
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
@@ -624,16 +659,16 @@ bool BeagleWksBMParser::readColorMap()
   if (!input->checkPosition(pos+(1+maxColor)*8))
     return false;
 
-  BeagleWksBMParserInternal::Bitmap &bitmap=m_state->m_bitmap;
+  auto &bitmap=m_state->m_bitmap;
   bitmap.m_colorList.resize(size_t(maxColor+1));
   for (int i=0; i<= maxColor; ++i) {
     f.str("");
     f << "ColorMap-" << i << ":";
     pos = input->tell();
-    int id=(int) input->readLong(2);
+    auto id=static_cast<int>(input->readLong(2));
     if (id!=i) f << "#id=" << id << ",";
     unsigned char col[3];
-    for (int c = 0; c < 3; c++) col[c] = (unsigned char)(input->readULong(2)>>8);
+    for (unsigned char &c : col) c = static_cast<unsigned char>(input->readULong(2)>>8);
     MWAWColor color(col[0], col[1], col[2]);
     bitmap.m_colorList[size_t(i)]=color;
     f << color << ",";
@@ -714,8 +749,8 @@ bool BeagleWksBMParser::sendBitmap()
   if (!m_state->m_pict || !m_state->m_pict->getBinary(picture)) return false;
 
   MWAWPageSpan const &page=getPageSpan();
-  MWAWPosition pos(MWAWVec2f((float)page.getMarginLeft(),(float)page.getMarginRight()),
-                   MWAWVec2f((float)page.getPageWidth(),(float)page.getPageLength()), librevenge::RVNG_INCH);
+  MWAWPosition pos(MWAWVec2f(float(page.getMarginLeft()),float(page.getMarginRight())),
+                   MWAWVec2f(float(page.getPageWidth()),float(page.getPageLength())), librevenge::RVNG_INCH);
   pos.setRelativePosition(MWAWPosition::Page);
   pos.m_wrapping = MWAWPosition::WNone;
   listener->insertPicture(pos, picture);
@@ -797,7 +832,7 @@ bool BeagleWksBMParser::checkHeader(MWAWHeader *header, bool strict)
     return false;
   }
   for (int i=0; i < 9; ++i) { // f2=f6=1 other 0
-    int val=(int) input->readLong(2);
+    auto val=static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   setVersion(1);
@@ -824,7 +859,7 @@ bool BeagleWksBMParser::checkHeader(MWAWHeader *header, bool strict)
   MWAWEntry entry;
   entry.setBegin(input->readLong(4));
   entry.setLength(input->readLong(4));
-  entry.setId((int) input->readLong(2)); // in fact nFonts
+  entry.setId(static_cast<int>(input->readLong(2))); // in fact nFonts
   entry.setType("FontNames");
   f << "fontNames[ptr]=" << std::hex << entry.begin() << "<->" << entry.end()
     << std::dec << ",nFonts=" << entry.id() << ",";
