@@ -59,16 +59,24 @@
 namespace BeagleWksDBParserInternal
 {
 //! Internal: the cell of a BeagleWksDBParser
-struct Cell : public MWAWCell {
+struct Cell final : public MWAWCell {
   //! the cell type
   enum Type { Text, Number, Date, Time, Picture, Formula, Memo, Unknown };
   //! constructor
-  explicit Cell(MWAWVec2i pos=MWAWVec2i(0,0)) : MWAWCell(), m_type(Unknown), m_name(""), m_content(), m_formula(-1), m_pictureId(-1), m_isEmpty(false)
+  explicit Cell(MWAWVec2i pos=MWAWVec2i(0,0))
+    : MWAWCell()
+    , m_type(Unknown)
+    , m_name("")
+    , m_content()
+    , m_formula(-1)
+    , m_pictureId(-1)
+    , m_isEmpty(false)
   {
     setPosition(pos);
   }
+  Cell(Cell const &)=default;
   //! destructor
-  virtual ~Cell();
+  ~Cell() final;
   //! returns true if the field has no content
   bool empty() const
   {
@@ -95,7 +103,12 @@ Cell::~Cell()
 //! Internal: the spreadsheet of a BeagleWksDBParser
 struct Database {
   //! constructor
-  Database() : m_numFields(0), m_fields(), m_records(), m_memos(), m_name("Sheet0")
+  Database()
+    : m_numFields(0)
+    , m_fields()
+    , m_records()
+    , m_memos()
+    , m_name("Sheet0")
   {
   }
   //! convert the m_widthCols, m_heightRows in a vector of of point size
@@ -139,7 +152,7 @@ void Database::updateWithContent(Cell &cell, MWAWVec2i const &pos, MWAWCellConte
     break;
   case Cell::Memo: {
     if (!content.isValueSet()) break;
-    int id=int(0.1+content.m_value);
+    auto id=int(0.1+content.m_value);
     if (id<1 || id>static_cast<int>(m_memos.size())) {
       MWAW_DEBUG_MSG(("BeagleWksDBParserInternal::Database::updateWithContent: can not retrieve the memo content\n"));
       break;
@@ -157,14 +170,13 @@ void Database::updateWithContent(Cell &cell, MWAWVec2i const &pos, MWAWCellConte
   case Cell::Unknown:
     break;
   }
-  MWAWCell::Format const &format=cell.getFormat();
+  auto const &format=cell.getFormat();
   // change the reference date from 1/1/1904 to 1/1/1900
   if (format.m_format==MWAWCell::F_DATE && cell.m_content.isValueSet())
     cell.m_content.setValue(cell.m_content.m_value+1460.);
   // and try to update the 1D formula in 2D
-  std::vector<MWAWCellContent::FormulaInstruction> &formula=cell.m_content.m_formula;
-  for (size_t i=0; i<formula.size(); ++i) {
-    MWAWCellContent::FormulaInstruction &instr=formula[i];
+  auto &formula=cell.m_content.m_formula;
+  for (auto &instr : formula) {
     if (instr.m_type==MWAWCellContent::FormulaInstruction::F_Cell) {
       --instr.m_position[0][0];
       instr.m_position[0][1]=pos[1];
@@ -181,8 +193,14 @@ void Database::updateWithContent(Cell &cell, MWAWVec2i const &pos, MWAWCellConte
 //! Internal: the state of a BeagleWksDBParser
 struct State {
   //! constructor
-  State() :  m_databaseBegin(-1), m_database(), m_typeEntryMap(),
-    m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0)
+  State()
+    : m_databaseBegin(-1)
+    , m_database()
+    , m_typeEntryMap()
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_headerHeight(0)
+    , m_footerHeight(0)
   {
   }
   /** the database begin position */
@@ -199,35 +217,30 @@ struct State {
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a BeagleWksDBParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(BeagleWksDBParser &pars, MWAWInputStreamPtr input, MWAWEntry const &entry) :
-    MWAWSubDocument(&pars, input, entry)
+  SubDocument(BeagleWksDBParser &pars, MWAWInputStreamPtr const &input, MWAWEntry const &entry)
+    : MWAWSubDocument(&pars, input, entry)
   {
   }
 
   //! destructor
-  virtual ~SubDocument() {}
+  ~SubDocument() final {}
 
   //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const
+  bool operator!=(MWAWSubDocument const &doc) const final
   {
     return MWAWSubDocument::operator!=(doc);
   }
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
-  {
-    return !operator!=(doc);
-  }
 
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
 private:
-  SubDocument(SubDocument const &orig);
-  SubDocument &operator=(SubDocument const &orig);
+  SubDocument(SubDocument const &orig) = delete;
+  SubDocument &operator=(SubDocument const &orig) = delete;
 };
 
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*type*/)
@@ -236,7 +249,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     MWAW_DEBUG_MSG(("BeagleWksDBParserInternal::SubDocument::parse: no listener\n"));
     return;
   }
-  BeagleWksDBParser *parser=dynamic_cast<BeagleWksDBParser *>(m_parser);
+  auto *parser=dynamic_cast<BeagleWksDBParser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("BeagleWksDBParserInternal::SubDocument::parse: can not find the parser\n"));
     return;
@@ -253,8 +266,10 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-BeagleWksDBParser::BeagleWksDBParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWSpreadsheetParser(input, rsrcParser, header), m_state(), m_structureManager()
+BeagleWksDBParser::BeagleWksDBParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWSpreadsheetParser(input, rsrcParser, header)
+  , m_state()
+  , m_structureManager()
 {
   init();
 }
@@ -299,14 +314,14 @@ MWAWVec2f BeagleWksDBParser::getPageLeftTop() const
 ////////////////////////////////////////////////////////////
 void BeagleWksDBParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
 
-    checkHeader(0L);
+    checkHeader(nullptr);
     ok = createZones();
     if (ok) {
       createDocument(docInterface);
@@ -346,14 +361,14 @@ void BeagleWksDBParser::createDocument(librevenge::RVNGSpreadsheetInterface *doc
   std::vector<MWAWPageSpan> pageList;
   MWAWPageSpan ps(getPageSpan());
   if (header.valid()) {
-    shared_ptr<BeagleWksDBParserInternal::SubDocument> subDoc
+    std::shared_ptr<BeagleWksDBParserInternal::SubDocument> subDoc
     (new BeagleWksDBParserInternal::SubDocument(*this, getInput(), header));
     MWAWHeaderFooter hf(MWAWHeaderFooter::HEADER, MWAWHeaderFooter::ALL);
     hf.m_subDocument=subDoc;
     ps.setHeaderFooter(hf);
   }
   if (footer.valid()) {
-    shared_ptr<BeagleWksDBParserInternal::SubDocument> subDoc
+    std::shared_ptr<BeagleWksDBParserInternal::SubDocument> subDoc
     (new BeagleWksDBParserInternal::SubDocument(*this, getInput(), footer));
     MWAWHeaderFooter hf(MWAWHeaderFooter::FOOTER, MWAWHeaderFooter::ALL);
     hf.m_subDocument=subDoc;
@@ -416,8 +431,7 @@ bool BeagleWksDBParser::createZones()
   ascii().addNote(f.str().c_str());
 
   // now parse the different zones
-  std::multimap<std::string, MWAWEntry>::iterator it;
-  it=m_state->m_typeEntryMap.find("FontNames");
+  auto it=m_state->m_typeEntryMap.find("FontNames");
   if (it!=m_state->m_typeEntryMap.end())
     m_structureManager->readFontNames(it->second);
   it=m_state->m_typeEntryMap.find("Frame");
@@ -425,8 +439,8 @@ bool BeagleWksDBParser::createZones()
     m_structureManager->readFrame(it->second);
 
   // now parse the different zones
-  for (it=m_state->m_typeEntryMap.begin(); it!=m_state->m_typeEntryMap.end(); ++it) {
-    MWAWEntry const &entry=it->second;
+  for (auto eIt : m_state->m_typeEntryMap) {
+    MWAWEntry const &entry=eIt.second;
     if (entry.isParsed())
       continue;
     f.str("");
@@ -447,7 +461,7 @@ bool BeagleWksDBParser::createZones()
   if (!readDatabase())
     return m_state->m_database.m_records.size()!=0;
   pos=input->tell();
-  int N=static_cast<int>(input->readULong(2));
+  auto N=static_cast<int>(input->readULong(2));
   if (N==0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -492,7 +506,7 @@ bool BeagleWksDBParser::createZones()
   pos=input->tell();
   f.str("");
   f << "Entries(Memo):";
-  long dSz=static_cast<long>(input->readULong(2));
+  auto dSz=static_cast<long>(input->readULong(2));
   long endPos=pos+2+dSz;
   N=static_cast<int>(input->readULong(2));
   if (dSz<2+2*long(N) || !input->checkPosition(endPos)) {
@@ -508,9 +522,9 @@ bool BeagleWksDBParser::createZones()
     pos=input->tell();
     f.str("");
     f << "Memo-"<< i << ":";
-    int val=static_cast<int>(input->readLong(1));
+    auto val=static_cast<int>(input->readLong(1));
     if (val) f << "f0=" << val << ",";
-    int sSz=static_cast<int>(input->readULong(1));
+    auto sSz=static_cast<int>(input->readULong(1));
     if (pos+2+sSz>endPos) {
       MWAW_DEBUG_MSG(("BeagleWksDBParser::createZones: can not read a memo\n"));
       f << "###";
@@ -535,7 +549,7 @@ bool BeagleWksDBParser::createZones()
   f << "Entries(UnknZone2):";
   dSz=long(input->readULong(2));
   endPos=pos+6+dSz;
-  int val=static_cast<int>(input->readULong(2));
+  auto val=static_cast<int>(input->readULong(2));
   if ((dSz%2) || val!=0xeb || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::createZones: can not read zone2\n"));
     f << "###";
@@ -558,16 +572,17 @@ bool BeagleWksDBParser::createZones()
   ascii().addNote(f.str().c_str());
 
   /* now the formula zone */
-  std::vector<BeagleWksDBParserInternal::Cell> &fields=m_state->m_database.m_fields;
-  for (size_t i=0; i<fields.size(); ++i) {
-    BeagleWksDBParserInternal::Cell &field=fields[i];
+  auto &fields=m_state->m_database.m_fields;
+  int n=0;
+  for (auto &field : fields) {
+    ++n;
     if (field.m_type!=BeagleWksDBParserInternal::Cell::Formula)
       continue;
     pos=input->tell();
     f.str("");
     f << "Entries(Formula):";
-    int id=static_cast<int>(input->readLong(2));
-    if (id!=static_cast<int>(i)) f << "#id=" << id << ",";
+    auto id=static_cast<int>(input->readLong(2));
+    if (id!=n-1) f << "#id=" << id << ",";
     dSz=static_cast<int>(input->readULong(2));
     endPos=pos+4+dSz;
     if (dSz==0 || !input->checkPosition(pos+4+dSz)) {
@@ -585,8 +600,8 @@ bool BeagleWksDBParser::createZones()
     }
     else
       f << "###";
-    for (size_t l=0; l < formula.size(); ++l)
-      f << formula[l];
+    for (auto const &fo : formula)
+      f << fo;
     input->seek(endPos, librevenge::RVNG_SEEK_SET);
     ascii().addPos(pos);
     ascii().addNote(f.str().c_str());
@@ -607,12 +622,11 @@ bool BeagleWksDBParser::readRSRCZones()
   if (!rsrcParser)
     return true;
 
-  std::multimap<std::string, MWAWEntry> &entryMap = rsrcParser->getEntriesMap();
-  std::multimap<std::string, MWAWEntry>::iterator it;
+  auto &entryMap = rsrcParser->getEntriesMap();
   // the 1 zone
-  char const *(zNames[]) = {"wPos", "DMPF" };
+  char const *zNames[] = {"wPos", "DMPF" };
   for (int z = 0; z < 2; ++z) {
-    it = entryMap.lower_bound(zNames[z]);
+    auto it = entryMap.lower_bound(zNames[z]);
     while (it != entryMap.end()) {
       if (it->first != zNames[z])
         break;
@@ -715,7 +729,7 @@ bool BeagleWksDBParser::readDatabase()
   }
   val =static_cast<int>(input->readLong(2));
   if (val!=7) f << "f2=" << val << ",";
-  int N=static_cast<int>(input->readLong(2));
+  auto N=static_cast<int>(input->readLong(2));
   f << "N=" << N << ",";
   val =static_cast<int>(input->readLong(2));
   if (val) f << "f3=" << val << ",";
@@ -735,9 +749,9 @@ bool BeagleWksDBParser::readDatabase()
   f << "N=" << N << ",";
   val =static_cast<int>(input->readLong(2));
   if (val) f << "f1=" << val << ",";
-  int dSz =static_cast<int>(input->readULong(2));
+  auto dSz =static_cast<int>(input->readULong(2));
   f << "dSz=" << dSz << ",";
-  if (dSz<14 || !input->checkPosition(pos+8+(N+1)*dSz)) {
+  if (dSz<14 || dSz>(input->size()-pos-8)/(N+1)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readDatabase: can not find the database field format\n"));
     f << "###";
     ascii().addPos(pos);
@@ -747,7 +761,7 @@ bool BeagleWksDBParser::readDatabase()
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
 
-  BeagleWksDBParserInternal::Database &database=m_state->m_database;
+  auto &database=m_state->m_database;
   size_t numFields=database.m_fields.size();
   if (N+1>static_cast<int>(numFields)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readDatabase: the number of field seems too small, must increase them\n"));
@@ -763,7 +777,7 @@ bool BeagleWksDBParser::readDatabase()
     val=static_cast<int>(input->readLong(2));
     if (val!=0x4b) f << "f1=" << val << ",";
     // checkme: maybe we must use the read id here...
-    BeagleWksDBParserInternal::Cell &field=database.m_fields[size_t(i)];
+    auto &field=database.m_fields[size_t(i)];
     if (readFormat(field, extra)) {
       f << extra << ",";
       if (dSz>14) { // find also dSz=18 which add c0000034 here
@@ -796,7 +810,7 @@ bool BeagleWksDBParser::readFormat(MWAWCell &cell, std::string &extra)
   libmwaw::DebugStream f;
   MWAWFont font;
   MWAWCell::Format format;
-  int val=static_cast<int>(input->readULong(1));
+  auto val=static_cast<int>(input->readULong(1));
   if (val) f << "f0=" << std::hex << val << std::dec << ",";
   val=static_cast<int>(input->readLong(1));
   if (val>0)
@@ -804,7 +818,7 @@ bool BeagleWksDBParser::readFormat(MWAWCell &cell, std::string &extra)
   val=static_cast<int>(input->readLong(2));
   if (val>=0)
     font.setId(val);
-  int flag=static_cast<int>(input->readULong(1));
+  auto flag=static_cast<int>(input->readULong(1));
   uint32_t flags=0;
   if (flag&0x8) flags |= MWAWFont::boldBit;
   if (flag&0x10) flags |= MWAWFont::italicBit;
@@ -815,7 +829,7 @@ bool BeagleWksDBParser::readFormat(MWAWCell &cell, std::string &extra)
   f << font.getDebugString(getParserState()->m_fontConverter);
   if (flag&7) f << "flags=" << (flag&7) << ",";
 
-  int form=static_cast<int>(input->readULong(1));
+  auto form=static_cast<int>(input->readULong(1));
   if (form) {
     if (form & 0x10)
       format.m_thousandHasSeparator=true;
@@ -840,7 +854,7 @@ bool BeagleWksDBParser::readFormat(MWAWCell &cell, std::string &extra)
       break;
     case 5:
       format.m_format=MWAWCell::F_DATE;
-      format.m_DTFormat="%m/%d/%y:";
+      format.m_DTFormat="%m/%d/%y";
       break;
     case 6:
       switch (form & 0x7) {
@@ -938,11 +952,11 @@ bool BeagleWksDBParser::readRow()
   long pos=input->tell();
   libmwaw::DebugStream f;
 
-  int id=static_cast<int>(input->readLong(2));
+  auto id=static_cast<int>(input->readLong(2));
   f << "DbRow" << id << ":";
-  int val=static_cast<int>(input->readLong(2));
+  auto val=static_cast<int>(input->readLong(2));
   if (val) f << "f0=" << val << ",";
-  long dSz=long(input->readULong(2));
+  auto dSz=long(input->readULong(2));
   long endPos=pos+6+dSz;
   if (dSz<18 || !input->checkPosition(pos+6+dSz)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readRow: can not find the database row %d\n", id));
@@ -971,16 +985,16 @@ bool BeagleWksDBParser::readRow()
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
 
-  BeagleWksDBParserInternal::Database &database=m_state->m_database;
-  size_t numFields=database.m_fields.size();
+  auto &database=m_state->m_database;
   database.m_records.resize(database.m_records.size()+1);
-  std::vector<MWAWCellContent> &records=database.m_records.back();
-  for (size_t fd=0; fd<numFields; ++fd) {
+  auto &records=database.m_records.back();
+  int fd=0;
+  for (auto const &field : database.m_fields) {
     pos=input->tell();
     if (pos>=endPos) break;
     f.str("");
-    f << "DbRow" << id << "-" << fd << ":";
-    int fSz=static_cast<int>(input->readULong(1));
+    f << "DbRow" << id << "-" << fd++ << ":";
+    auto fSz=static_cast<int>(input->readULong(1));
     if (fSz==0xFF) {
       ascii().addPos(pos);
       ascii().addNote("_");
@@ -993,7 +1007,6 @@ bool BeagleWksDBParser::readRow()
     }
     val=static_cast<int>(input->readULong(1));
     if (val!=0x20) f << "fl=" << std::hex << val << std::dec << ",";
-    BeagleWksDBParserInternal::Cell const &field=database.m_fields[fd];
     MWAWCellContent content;
     if (fSz && fSz<8) {
       MWAW_DEBUG_MSG(("BeagleWksDBParser::readRow: find some very short field\n"));
@@ -1009,6 +1022,7 @@ bool BeagleWksDBParser::readRow()
         if (fSz>20) break;
         val=static_cast<int>(input->readULong(2));
         if (val) f << "g0=" << std::hex << val << std::dec << ",";
+        MWAW_FALLTHROUGH;
       case BeagleWksDBParserInternal::Cell::Memo: // memoId
       case BeagleWksDBParserInternal::Cell::Picture: // pictId
       case BeagleWksDBParserInternal::Cell::Number:
@@ -1071,7 +1085,7 @@ bool BeagleWksDBParser::readRow()
 bool BeagleWksDBParser::readFields()
 {
   MWAWInputStreamPtr &input= getInput();
-  BeagleWksDBParserInternal::Database &database=m_state->m_database;
+  auto &database=m_state->m_database;
   long pos=input->tell();
   if (!input->checkPosition(pos+6)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readFields: can not find the field zone\n"));
@@ -1079,7 +1093,7 @@ bool BeagleWksDBParser::readFields()
   }
   libmwaw::DebugStream f;
   f << "Entries(Field):";
-  int val=static_cast<int>(input->readLong(2));
+  auto val=static_cast<int>(input->readLong(2));
   if (val) f << "f0=" << val << ",";
   val=static_cast<int>(input->readLong(2));
   if (val!=0x2c) // may a type
@@ -1101,7 +1115,7 @@ bool BeagleWksDBParser::readFields()
     f.str("");
     f << "Field-" << fld << ":";
     BeagleWksDBParserInternal::Cell field;
-    long dSz=long(input->readULong(2));
+    auto dSz=long(input->readULong(2));
     long endPos=pos+4+dSz;
     if (dSz<0x3c || !input->checkPosition(endPos)) {
       MWAW_DEBUG_MSG(("BeagleWksDBParser::readFields: can not read a field\n"));
@@ -1110,9 +1124,9 @@ bool BeagleWksDBParser::readFields()
       ascii().addNote(f.str().c_str());
       return false;
     }
-    int id=static_cast<int>(input->readLong(2));
+    auto id=static_cast<int>(input->readLong(2));
     if (id) f << "id=" << id << ",";
-    int sSz=static_cast<int>(input->readULong(1));
+    auto sSz=static_cast<int>(input->readULong(1));
     if (sSz+1>dSz) {
       MWAW_DEBUG_MSG(("BeagleWksDBParser::readFields: can not read a field\n"));
       f << "###";
@@ -1127,8 +1141,8 @@ bool BeagleWksDBParser::readFields()
     ascii().addDelimiter(input->tell(),'|');
 
     MWAWCell::Format format;
-    MWAWCellContent &content=field.m_content;
-    int type=static_cast<int>(input->readLong(1));
+    auto &content=field.m_content;
+    auto type=static_cast<int>(input->readLong(1));
     switch (type) {
     case 0:
       field.m_type = BeagleWksDBParserInternal::Cell::Text;
@@ -1203,12 +1217,12 @@ bool BeagleWksDBParser::readLayouts()
   }
   libmwaw::DebugStream f;
   f << "Entries(Layout):";
-  int val=static_cast<int>(input->readLong(2));
+  auto val=static_cast<int>(input->readLong(2));
   if (val) f << "f0=" << val << ",";
   val=static_cast<int>(input->readLong(2));
   if (val!=0x29) // may a type
     f << "f1=" << val << ",";
-  int numLayouts=static_cast<int>(input->readULong(2));
+  auto numLayouts=static_cast<int>(input->readULong(2));
   f << "num[layout]=" << numLayouts << ",";
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
@@ -1226,8 +1240,8 @@ bool BeagleWksDBParser::readLayout(int id)
   f << "Layout-" << id << "[A]:";
 
   long pos=input->tell();
-  int readId=static_cast<int>(input->readULong(1));
-  long dSz=long(input->readULong(2));
+  auto readId=static_cast<int>(input->readULong(1));
+  auto dSz=long(input->readULong(2));
   long endPos=pos+1+dSz;
   if (dSz<100 || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readLayouts: can find a layout\n"));
@@ -1237,11 +1251,11 @@ bool BeagleWksDBParser::readLayout(int id)
     return false;
   }
   if (readId!=id) f << "#id=" << readId << ",";
-  int val=static_cast<int>(input->readLong(2));
+  auto val=static_cast<int>(input->readLong(2));
   if (val) f << "f0=" << val << ",";
   val=static_cast<int>(input->readULong(1));
   if (val!=readId) f << "#id1=" << val << ",";
-  int sSz=static_cast<int>(input->readULong(1));
+  auto sSz=static_cast<int>(input->readULong(1));
   if (sSz>30) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readLayouts: can find layout string\n"));
     f << "###sSz=" << sSz << ",";
@@ -1282,9 +1296,9 @@ bool BeagleWksDBParser::readLayout(int id)
   f << "Layout-" << id << "[B]:";
   double margins[4];
   f << "margins=[";
-  for (int i=0; i < 4; ++i) {
-    margins[i]=double(input->readLong(4))/72.;
-    f << margins[i] << ",";
+  for (double &margin : margins) {
+    margin=double(input->readLong(4))/72.;
+    f << margin << ",";
   }
   f << "],";
   for (int i=0; i<3; ++i) {
@@ -1346,7 +1360,7 @@ bool BeagleWksDBParser::readLayout(int id)
   pos=input->tell();
   f.str("");
   f << "Layout-" << id << "[D]:";
-  int N=static_cast<int>(input->readULong(2));
+  auto N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   if (!input->checkPosition(pos+2+2*N)) {
     MWAW_DEBUG_MSG(("BeagleWksDBParser::readLayouts: can find zone D\n"));
@@ -1367,7 +1381,7 @@ bool BeagleWksDBParser::readLayout(int id)
   f.str("");
   f << "Layout-" << id << "[fields]:";
   val=static_cast<int>(input->readULong(2));
-  int type=static_cast<int>(input->readULong(2));
+  auto type=static_cast<int>(input->readULong(2));
   N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   if (type!=0x5a || !input->checkPosition(pos+6+36*N)) {
@@ -1398,7 +1412,7 @@ bool BeagleWksDBParser::readLayout(int id)
     }
     for (int j=0; j<2; ++j) {
       int dim[4];
-      for (int k=0; k<4; ++k) dim[k]=static_cast<int>(input->readLong(2));
+      for (auto &k : dim) k=static_cast<int>(input->readLong(2));
       f << "box" << j << "=" << dim[1] << "x" << dim[0] << "<->" << dim[3] << "x" << dim[2] << ",";
     }
     ascii().addPos(pos);
@@ -1436,10 +1450,9 @@ bool BeagleWksDBParser::readLayout(int id)
 ////////////////////////////////////////////////////////////
 bool BeagleWksDBParser::sendPageFrames()
 {
-  std::map<int, BeagleWksStructManager::Frame> const &frameMap = m_structureManager->getIdFrameMap();
-  std::map<int, BeagleWksStructManager::Frame>::const_iterator it;
-  for (it=frameMap.begin(); it!=frameMap.end(); ++it)
-    sendFrame(it->second);
+  auto const &frameMap = m_structureManager->getIdFrameMap();
+  for (auto it : frameMap)
+    sendFrame(it.second);
   return true;
 }
 
@@ -1451,7 +1464,7 @@ bool BeagleWksDBParser::sendFrame(BeagleWksStructManager::Frame const &frame)
   fPos.setRelativePosition(MWAWPosition::Page);
   fPos.m_wrapping = frame.m_wrap==0 ? MWAWPosition::WNone : MWAWPosition::WDynamic;
 
-  MWAWGraphicStyle style=MWAWGraphicStyle::emptyStyle();
+  auto style=MWAWGraphicStyle::emptyStyle();
   style.setBorders(frame.m_bordersSet, frame.m_border);
   return sendPicture(frame.m_pictId, fPos, true, style);
 }
@@ -1465,7 +1478,7 @@ bool BeagleWksDBParser::sendPicture
     MWAW_DEBUG_MSG(("BeagleWksDBParser::sendPicture: can not find the listener\n"));
     return false;
   }
-  MWAWRSRCParserPtr rsrcParser = getRSRCParser();
+  auto rsrcParser = getRSRCParser();
   if (!rsrcParser) {
     static bool first=true;
     if (first) {
@@ -1483,7 +1496,7 @@ bool BeagleWksDBParser::sendPicture
   return true;
 }
 
-bool BeagleWksDBParser::sendText(MWAWEntry entry, bool /*headerFooter*/)
+bool BeagleWksDBParser::sendText(MWAWEntry const &entry, bool /*headerFooter*/)
 {
   MWAWSpreadsheetListenerPtr listener=getSpreadsheetListener();
   if (!listener) {
@@ -1501,7 +1514,7 @@ bool BeagleWksDBParser::sendText(MWAWEntry entry, bool /*headerFooter*/)
   while (!input->isEnd()) {
     long pos=input->tell();
     if (pos>=endPos) break;
-    unsigned char c = static_cast<unsigned char>(input->readULong(1));
+    auto c = static_cast<unsigned char>(input->readULong(1));
     switch (c) {
     case 0x9:
       listener->insertTab();
@@ -1527,10 +1540,10 @@ bool BeagleWksDBParser::sendDatabase()
     return false;
   }
   MWAWInputStreamPtr input=getInput();
-  BeagleWksDBParserInternal::Database const &database=m_state->m_database;
-  std::vector<BeagleWksDBParserInternal::Cell> const &fields = database.m_fields;
+  auto const &database=m_state->m_database;
+  auto const &fields = database.m_fields;
   size_t numFields=fields.size();
-  std::vector<std::vector<MWAWCellContent> > const &records=database.m_records;
+  auto const &records=database.m_records;
   size_t numRecords=records.size();
   // fixme: use first layout colWidth here
   listener->openSheet(std::vector<float>(1,76), librevenge::RVNG_POINT, std::vector<int>(1,static_cast<int>(numRecords)), "Sheet0");
@@ -1539,17 +1552,17 @@ bool BeagleWksDBParser::sendDatabase()
     listener->openSheetRow(12, librevenge::RVNG_POINT);
     for (size_t c=0; c<row.size(); ++c) {
       if (c>=numFields) break;
-      BeagleWksDBParserInternal::Cell field=fields[c];
+      auto field=fields[c];
       database.updateWithContent(field, MWAWVec2i(int(c),int(r)), row[c]);
       if (field.empty()) continue;
 
-      MWAWCellContent const &content=field.m_content;
+      auto const &content=field.m_content;
       listener->openSheetCell(field, content);
       if (content.m_contentType==MWAWCellContent::C_TEXT && content.m_textEntry.valid()) {
         listener->setFont(field.getFont());
         input->seek(content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
         while (!input->isEnd() && input->tell()<content.m_textEntry.end()) {
-          unsigned char ch=static_cast<unsigned char>(input->readULong(1));
+          auto ch=static_cast<unsigned char>(input->readULong(1));
           if (ch==0xd)
             listener->insertEOL();
           else if (ch<30) {
@@ -1601,7 +1614,7 @@ bool BeagleWksDBParser::checkHeader(MWAWHeader *header, bool strict)
     return false;
   }
   for (int i=0; i < 9; ++i) { // f2=f6=1 other 0
-    int val=static_cast<int>(input->readLong(2));
+    auto val=static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   setVersion(1);
