@@ -61,7 +61,11 @@ namespace GreatWksDocumentInternal
 //! Internal: the state of a GreatWksDocument
 struct State {
   //! constructor
-  State() : m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0)
+  State()
+    : m_actPage(0)
+    , m_numPages(0)
+    , m_headerHeight(0)
+    , m_footerHeight(0)
   {
   }
 
@@ -76,10 +80,14 @@ struct State {
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-GreatWksDocument::GreatWksDocument(MWAWParser &parser) :
-  m_state(), m_parserState(parser.getParserState()),
-  m_parser(&parser), m_graphParser(), m_textParser(),
-  m_getMainSection(0), m_newPage(0)
+GreatWksDocument::GreatWksDocument(MWAWParser &parser)
+  : m_state()
+  , m_parserState(parser.getParserState())
+  , m_parser(&parser)
+  , m_graphParser()
+  , m_textParser()
+  , m_getMainSection(nullptr)
+  , m_newPage(nullptr)
 {
   m_state.reset(new GreatWksDocumentInternal::State);
 
@@ -167,12 +175,11 @@ bool GreatWksDocument::readRSRCZones()
   if (!rsrcParser)
     return true;
 
-  std::multimap<std::string, MWAWEntry> &entryMap = rsrcParser->getEntriesMap();
-  std::multimap<std::string, MWAWEntry>::iterator it;
+  auto &entryMap = rsrcParser->getEntriesMap();
   // the 1 zone
-  char const *(zNames[]) = {"PRNT", "PAT#", "WPSN", "PlTT", "ARRs", "DaHS", "GrDS", "NxEd" };
+  char const *zNames[] = {"PRNT", "PAT#", "WPSN", "PlTT", "ARRs", "DaHS", "GrDS", "NxEd" };
   for (int z = 0; z < 8; ++z) {
-    it = entryMap.lower_bound(zNames[z]);
+    auto it = entryMap.lower_bound(zNames[z]);
     while (it != entryMap.end()) {
       if (it->first != zNames[z])
         break;
@@ -235,7 +242,7 @@ bool GreatWksDocument::readWPSN(MWAWEntry const &entry)
 
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(Windows):";
-  int N=(int) input->readLong(2);
+  auto N=static_cast<int>(input->readLong(2));
   f << "N=" << N << ",";
   if (2+24*N!=int(entry.length())) {
     f << "###";
@@ -251,17 +258,14 @@ bool GreatWksDocument::readWPSN(MWAWEntry const &entry)
     f.str("");
     f << "Windows-" << i << ":";
     int width[2];
-    for (int j=0; j < 2; ++j)
-      width[j]=(int) input->readLong(2);
+    for (auto &w : width) w=static_cast<int>(input->readLong(2));
     f << "w=" << width[1] << "x" << width[0] << ",";
     int LT[2];
-    for (int j=0; j < 2; ++j)
-      LT[j]=(int) input->readLong(2);
+    for (auto &orig : LT) orig=static_cast<int>(input->readLong(2));
     f << "LT=" << LT[1] << "x" << LT[0] << ",";
     for (int st=0; st < 2; ++st) {
       int dim[4];
-      for (int j=0; j < 4; ++j)
-        dim[j]=(int) input->readLong(2);
+      for (auto &d : dim) d=static_cast<int>(input->readLong(2));
       if (dim[0]!=LT[0] || dim[1]!=LT[1] || dim[2]!=LT[0]+width[0])
         f << "dim" << st << "=" << dim[1] << "x" << dim[0] << "<->"
           << dim[3] << "x" << dim[2] << ",";
@@ -345,7 +349,7 @@ bool GreatWksDocument::readARRs(MWAWEntry const &entry)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   ascFile.addPos(pos-4);
   ascFile.addNote("Entries(ARRs)");
-  int N=int(entry.length()/32);
+  auto N=int(entry.length()/32);
   for (int i=0; i < N; ++i) {
     pos = input->tell();
     f.str("");
@@ -372,10 +376,10 @@ bool GreatWksDocument::readDaHS(MWAWEntry const &entry)
 
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(DaHS):";
-  int val=(int) input->readLong(2);
+  auto val=static_cast<int>(input->readLong(2));
   if (val!=2)
     f << "#f0=" << val << ",";
-  val=(int) input->readLong(2);
+  val=static_cast<int>(input->readLong(2));
   if (val!=9)
     f << "#f1=" << val << ",";
   ascFile.addDelimiter(input->tell(), '|');
@@ -413,24 +417,23 @@ bool GreatWksDocument::readGrDS(MWAWEntry const &entry)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   ascFile.addPos(pos-4);
   ascFile.addNote("Entries(GrDS)");
-  int N=int(entry.length()/16);
+  auto N=int(entry.length()/16);
   for (int i=0; i < N; ++i) {
     pos = input->tell();
     f.str("");
     f << "GrDS-" << i << ":";
-    int val=(int)input->readLong(2); // 1,2,3
+    auto val=static_cast<int>(input->readLong(2)); // 1,2,3
     f << "unkn=" << val << ",";
     for (int st=0; st < 2; ++st) {
       unsigned char col[3];
-      for (int j=0; j < 3; ++j)
-        col[j]=(unsigned char)(input->readULong(2)>>8);
+      for (auto &c : col) c=static_cast<unsigned char>(input->readULong(2)>>8);
       MWAWColor color(col[0], col[1], col[2]);
       if (st==0) {
         if (!color.isWhite()) f << "backColor=" << color << ",";
       }
       else if (!color.isBlack()) f << "frontColor=" << color << ",";
     }
-    val = (int) input->readULong(2);
+    val = static_cast<int>(input->readULong(2));
     if (val) f << "ptr?=" << std::hex << val << std::dec << ",";
     input->seek(pos+16, librevenge::RVNG_SEEK_SET);
     ascFile.addPos(i==0?pos-4:pos);
@@ -459,7 +462,7 @@ bool GreatWksDocument::readNxEd(MWAWEntry const &entry)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   f << "Entries(NxED):";
   for (int i = 0; i < 2; ++i) { // always 0
-    int val=(int) input->readLong(2);
+    auto val=static_cast<int>(input->readLong(2));
     if (val)
       f << "f" << i << "=" << val << ",";
   }
@@ -483,7 +486,7 @@ bool GreatWksDocument::checkHeader(MWAWHeader *header, bool strict)
   f << "FileHeader:";
 
   input->seek(0,librevenge::RVNG_SEEK_SET);
-  int vers=(int) input->readLong(1);
+  auto vers=static_cast<int>(input->readLong(1));
   if (vers < 1 || vers > 2)
     return false;
   if (input->readLong(1))
@@ -491,7 +494,7 @@ bool GreatWksDocument::checkHeader(MWAWHeader *header, bool strict)
   m_parserState->m_version=vers;
   std::string type("");
   for (int i=0; i < 4; ++i)
-    type+=(char) input->readLong(1);
+    type+=char(input->readLong(1));
   if (type=="ZOBJ") {
     m_parserState->m_kind=MWAWDocument::MWAW_K_DRAW;
     MWAW_DEBUG_MSG(("GreatWksDocument::checkHeader: find a draw file\n"));
@@ -544,7 +547,7 @@ bool GreatWksDocument::readCellInFormula(MWAWVec2i const &pos, MWAWCellContent::
   bool absolute[2] = { true, true};
   int cPos[2];
   for (int i=0; i<2; ++i) {
-    int val = (int) input->readULong(2);
+    auto val = static_cast<int>(input->readULong(2));
     if (val & 0x8000) {
       absolute[i]=false;
       if (val&0x4000)
@@ -570,13 +573,13 @@ bool GreatWksDocument::readString(long endPos, std::string &res)
   res="";
   MWAWInputStreamPtr input=m_parserState->m_input;
   long pos=input->tell();
-  int fSz=(int) input->readULong(1);
+  auto fSz=static_cast<int>(input->readULong(1));
   if (pos+1+fSz>endPos) {
     MWAW_DEBUG_MSG(("GreatWksDocument::readString: can not read string size\n"));
     return false;
   }
   for (int i=0; i<fSz; ++i)
-    res += (char) input->readULong(1);
+    res += char(input->readULong(1));
   return true;
 }
 
@@ -632,7 +635,7 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
     long pos=input->tell();
     if (pos >= endPos)
       break;
-    int arity=0, val, type=(int) input->readULong(1);
+    int arity=0, val, type=static_cast<int>(input->readULong(1));
     MWAWCellContent::FormulaInstruction instr;
     switch (type) {
     case 2:
@@ -640,7 +643,7 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
         ok = false;
         break;
       }
-      val = (int) input->readLong(2);
+      val = static_cast<int>(input->readLong(2));
       instr.m_type=MWAWCellContent::FormulaInstruction::F_Long;
       instr.m_longValue=val;
       break;
@@ -666,7 +669,7 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
         ok = false;
         break;
       }
-      val = (int) input->readLong(1);
+      val = static_cast<int>(input->readLong(1));
       instr.m_type=MWAWCellContent::FormulaInstruction::F_Long;
       instr.m_longValue=val;
       break;
@@ -696,11 +699,11 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
         ok = false;
         break;
       }
-      val = (int) input->readULong(1);
-      arity= (int) input->readULong(1);
+      val = static_cast<int>(input->readULong(1));
+      arity= static_cast<int>(input->readULong(1));
       instr.m_type=MWAWCellContent::FormulaInstruction::F_Function;
 
-      static char const* (s_functions[]) = {
+      static char const *s_functions[] = {
         "", "Abs", "Exp", "Fact", "Int", "Ln", "Log", "Log10",
         "Mod", "Pi", "Product", "Rand", "Round", "Sign", "Sqrt", "Trunc",
 
@@ -745,7 +748,7 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
       instr.m_content=GreatWksDocumentInternal::s_listFunctions[type].m_name;
       ok=!instr.m_content.empty();
       arity = GreatWksDocumentInternal::s_listFunctions[type].m_arity;
-      if (arity == -1) arity = (int) input->readLong(1);
+      if (arity == -1) arity = static_cast<int>(input->readLong(1));
       if (arity<0) ok=false;
       break;
     }
@@ -757,7 +760,7 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
       continue;
     }
     size_t numElt = stack.size();
-    if ((int) numElt < arity) {
+    if (static_cast<int>(numElt) < arity) {
       f.str("");
       f << instr.m_content << "[##" << arity << "]";
       ok = false;
@@ -775,15 +778,14 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
           instr.m_content=";";
           child.push_back(instr);
         }
-        std::vector<MWAWCellContent::FormulaInstruction> const &node=
-          stack[size_t((int)numElt-arity+i)];
+        auto const &node= stack[size_t(static_cast<int>(numElt)-arity+i)];
         child.insert(child.end(), node.begin(), node.end());
       }
       instr.m_content=")";
       child.push_back(instr);
 
-      stack.resize(size_t((int) numElt-arity+1));
-      stack[size_t((int)numElt-arity)] = child;
+      stack.resize(size_t(static_cast<int>(numElt)-arity+1));
+      stack[size_t(static_cast<int>(numElt)-arity)] = child;
       continue;
     }
     if (arity==1) {
@@ -801,7 +803,7 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
         break;
       }
       instr=stack[numElt-2][0];
-      MWAWCellContent::FormulaInstruction instr2=stack[numElt-1][0];
+      auto instr2=stack[numElt-1][0];
       instr.m_type=MWAWCellContent::FormulaInstruction::F_CellList;
       instr.m_position[1]=instr2.m_position[0];
       instr.m_positionRelative[1]=instr2.m_positionRelative[0];
@@ -838,9 +840,9 @@ bool GreatWksDocument::readFormula(MWAWVec2i const &cPos, long endPos, std::vect
 
   error = f.str();
   f.str("");
-  for (size_t i = 0; i < stack.size(); ++i) {
-    for (size_t j=0; j < stack[i].size(); ++j)
-      f << stack[i][j] << ",";
+  for (auto const &i : stack) {
+    for (auto const &j : i)
+      f << j << ",";
   }
   f << error;
   error = f.str();
