@@ -65,7 +65,9 @@ class Style
 {
 public:
   /** constructor */
-  Style() : m_font(3,10), m_backgroundColor(MWAWColor::white())
+  Style()
+    : m_font(3,10)
+    , m_backgroundColor(MWAWColor::white())
   {
   }
   /** the font style */
@@ -75,11 +77,16 @@ public:
 };
 
 /** a cell of a GreatWksSSParser */
-class Cell : public MWAWCell
+class Cell final : public MWAWCell
 {
 public:
   /// constructor
   Cell() : m_content(), m_style(-1) { }
+  Cell(Cell const &)=default;
+  Cell &operator=(Cell &&)=default;
+  Cell &operator=(Cell const &)=default;
+  /// destructor
+  ~Cell() final;
   //! returns true if the cell do contain any content
   bool isEmpty() const
   {
@@ -92,19 +99,28 @@ public:
   int m_style;
 };
 
+Cell::~Cell()
+{
+}
+
 /** the spreadsheet of a of a MsWksSSParser */
 class Spreadsheet
 {
 public:
   //! constructor
-  Spreadsheet() : m_widthDefault(75), m_widthCols(), m_heightDefault(13), m_heightRows(),
-    m_cells(), m_name("Sheet0")
+  Spreadsheet()
+    : m_widthDefault(75)
+    , m_widthCols()
+    , m_heightDefault(13)
+    , m_heightRows()
+    , m_cells()
+    , m_name("Sheet0")
   {
   }
   //! returns the row size in point
   int getRowHeight(int row) const
   {
-    if (row>=0&&row<(int) m_heightRows.size())
+    if (row>=0&&row<static_cast<int>(m_heightRows.size()))
       return m_heightRows[size_t(row)];
     return m_heightDefault;
   }
@@ -113,10 +129,10 @@ public:
   {
     double res=getRowHeight(row);
     numRepeated=1;
-    if (row<0 || row>=(int) m_heightRows.size())
+    if (row<0 || row>=static_cast<int>(m_heightRows.size()))
       numRepeated=1000;
     else {
-      for (size_t r=size_t(row+1); r<m_heightRows.size(); ++r) {
+      for (auto r=size_t(row+1); r<m_heightRows.size(); ++r) {
         double nextH=getRowHeight(row+1);
         if (nextH<res || nextH>res)
           break;
@@ -128,7 +144,7 @@ public:
   //! convert the m_widthCols in a vector of of point size
   std::vector<float> convertInPoint(std::vector<int> const &list) const
   {
-    size_t numCols=size_t(getRightBottomPosition()[0]+1);
+    auto numCols=size_t(getRightBottomPosition()[0]+1);
     std::vector<float> res;
     res.resize(numCols);
     for (size_t i = 0; i < numCols; i++) {
@@ -154,9 +170,8 @@ protected:
   MWAWVec2i getRightBottomPosition() const
   {
     int maxX = 0, maxY = 0;
-    size_t numCell = m_cells.size();
-    for (size_t i = 0; i < numCell; i++) {
-      MWAWVec2i const &p = m_cells[i].position();
+    for (auto const &cell : m_cells) {
+      MWAWVec2i const &p = cell.position();
       if (p[0] > maxX) maxX = p[0];
       if (p[1] > maxY) maxY = p[1];
     }
@@ -168,8 +183,17 @@ protected:
 //! Internal: the state of a GreatWksSSParser
 struct State {
   //! constructor
-  State() : m_spreadsheet(), m_styleList(), m_actPage(0), m_numPages(0),
-    m_headerEntry(), m_footerEntry(), m_headerPrint(false), m_footerPrint(false), m_headerHeight(0), m_footerHeight(0)
+  State()
+    : m_spreadsheet()
+    , m_styleList()
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_headerEntry()
+    , m_footerEntry()
+    , m_headerPrint(false)
+    , m_footerPrint(false)
+    , m_headerHeight(0)
+    , m_footerHeight(0)
   {
   }
   //! returns the style corresponding to an id
@@ -195,25 +219,20 @@ struct State {
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a GreatWksSSParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(GreatWksSSParser &pars, MWAWInputStreamPtr input, int zoneId) :
+  SubDocument(GreatWksSSParser &pars, MWAWInputStreamPtr const &input, int zoneId) :
     MWAWSubDocument(&pars, input, MWAWEntry()), m_id(zoneId) {}
 
   //! destructor
-  virtual ~SubDocument() {}
+  ~SubDocument() final {}
 
   //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const;
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
-  {
-    return !operator!=(doc);
-  }
+  bool operator!=(MWAWSubDocument const &doc) const final;
 
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
   //! the subdocument id
@@ -230,7 +249,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type
     MWAW_DEBUG_MSG(("GreatWksSSParserInternal::SubDocument::parse: unknown type\n"));
     return;
   }
-  GreatWksSSParser *parser=dynamic_cast<GreatWksSSParser *>(m_parser);
+  auto *parser=dynamic_cast<GreatWksSSParser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("GreatWksSSParserInternal::SubDocument::parse: no parser\n"));
     return;
@@ -243,7 +262,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type
 bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 {
   if (MWAWSubDocument::operator!=(doc)) return true;
-  SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+  auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
   if (!sDoc) return true;
   if (m_id != sDoc->m_id) return true;
   return false;
@@ -253,8 +272,10 @@ bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-GreatWksSSParser::GreatWksSSParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWSpreadsheetParser(input, rsrcParser, header), m_state(), m_document()
+GreatWksSSParser::GreatWksSSParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWSpreadsheetParser(input, rsrcParser, header)
+  , m_state()
+  , m_document()
 {
   init();
 }
@@ -289,14 +310,14 @@ bool GreatWksSSParser::sendHF(int id)
 ////////////////////////////////////////////////////////////
 void GreatWksSSParser::parse(librevenge::RVNGSpreadsheetInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
 
-    checkHeader(0L);
+    checkHeader(nullptr);
     ok = createZones();
     if (ok) {
       createDocument(docInterface);
@@ -400,7 +421,7 @@ bool GreatWksSSParser::readSpreadsheet()
     bool ok=true, printDone=false;
     long pos = input->tell();
     f.str("");
-    int type=(int) input->readLong(2);
+    auto type=static_cast<int>(input->readLong(2));
     if (type==0x10) {
       if (vers==1)
         f << "Entries(DocInfo):";
@@ -408,7 +429,7 @@ bool GreatWksSSParser::readSpreadsheet()
         f << "Entries(Zone10):";
     }
     else if (type>=0 && type < 0x1a) {
-      static char const *(wh[0x1a]) = {
+      static char const *wh[0x1a] = {
         "_", "FontNames", "Style", "Column", "Column", "Row", "Zone6", "Row",
         "Column", "Zone9", "Zonea", "Zoneb", "Row", "Screen", "DocOptions", "Selection",
         "DocInfo", "CalcMode", "Zone12", "Zone13", "Zone14", "GridOptions", "DocInfo", "Header",
@@ -440,19 +461,19 @@ bool GreatWksSSParser::readSpreadsheet()
       break;
     case 3: // column
     case 5: { // row :
-      long sz=(long) input->readULong(2);
-      endPos=pos+4+(long) sz;
+      auto sz=long(input->readULong(2));
+      endPos=pos+4+long(sz);
       ok = (sz>=4 && input->checkPosition(endPos));
       if (!ok) break;
-      int id=(int) input->readLong(2);
-      int length=(int) input->readLong(2);
+      auto id=static_cast<int>(input->readLong(2));
+      auto length=static_cast<int>(input->readLong(2));
       if (id<1 || length < 0) {
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: the row/col definition seems bad\n"));
         f << "###";
       }
       else {
         std::vector<int> &lengths=type==3 ? m_state->m_spreadsheet.m_widthCols : m_state->m_spreadsheet.m_heightRows;
-        if (id-1 >= (int) lengths.size())
+        if (id-1 >= static_cast<int>(lengths.size()))
           lengths.resize(size_t(id), -1);
         lengths[size_t(id-1)]=length;
       }
@@ -472,7 +493,7 @@ bool GreatWksSSParser::readSpreadsheet()
         break;
       }
       for (int i=0; i < 4; ++i) {
-        int val=(int) input->readLong(2);
+        auto val=static_cast<int>(input->readLong(2));
         static int const expected[]= {9,0,2,0x2e};
         if (val!=expected[i])
           f << "f" << i << "=" << val << ",";
@@ -486,7 +507,7 @@ bool GreatWksSSParser::readSpreadsheet()
     case 0xc: // d
     case 0x11: // 0
     case 0x1a: { // 1
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -497,7 +518,7 @@ bool GreatWksSSParser::readSpreadsheet()
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: size of zone %d seems odd\n", type));
         break;
       }
-      int val=(int)input->readULong(2);
+      auto val=static_cast<int>(input->readULong(2));
       switch (type) {
       case 7:
         m_state->m_spreadsheet.m_heightDefault=val;
@@ -521,7 +542,7 @@ bool GreatWksSSParser::readSpreadsheet()
       break;
     }
     case 0xd: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -533,18 +554,16 @@ bool GreatWksSSParser::readSpreadsheet()
         break;
       }
       int dim[4];
-      for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+      for (auto &d : dim) d=static_cast<int>(input->readLong(2));
       f << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
       if (vers==1) break;
-      for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+      for (auto &d : dim) d=static_cast<int>(input->readLong(2));
       f << "screen2?=" << dim[0] << "x" << dim[1] << "<->" << dim[1] << "x" << dim[2] << ",";
       // now find 00000003cccc3333
       break;
     }
     case 0xe: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -557,10 +576,10 @@ bool GreatWksSSParser::readSpreadsheet()
       }
       int val;
       for (int i=0; i<9; ++i) {
-        val=(int)input->readULong(1);
+        val=static_cast<int>(input->readULong(1));
         if (!val) continue;
         // checkme: same value in v2 ?
-        static char const *(what[])= {"grid[display]", "headerRC[display]", "zero[doc]", "formula[doc]", "grid[print]", "headerRC[print]", "header[print]", "footer[print]", "protected" };
+        static char const *what[]= {"grid[display]", "headerRC[display]", "zero[doc]", "formula[doc]", "grid[print]", "headerRC[print]", "header[print]", "footer[print]", "protected" };
         if (val==1) {
           if (i==6)
             m_state->m_headerPrint=true;
@@ -571,14 +590,14 @@ bool GreatWksSSParser::readSpreadsheet()
         else
           f << "###" << what[i] << "=" << val << ",";
       }
-      val=(int)input->readULong(1);
+      val=static_cast<int>(input->readULong(1));
       if (val!=1) f << "first[page]=" << val << ",";
       break;
     }
     case 0xf: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
-      if (!input->checkPosition(endPos)) {
+      if (sz<0 || !input->checkPosition(endPos)) {
         ok=false;
         break;
       }
@@ -587,16 +606,18 @@ bool GreatWksSSParser::readSpreadsheet()
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: size of selection zone seems odd\n"));
         break;
       }
-      int val=(int)input->readULong(2);
+      auto val=static_cast<int>(input->readULong(2));
       if (val != 4) f << "f0=" << val << ",";
       f << "cells=[";
-      for (int i=0; i<3; ++i) // probably selected cell, followed by a cell range
-        f << input->readLong(2) << "x" << input->readLong(2) << ",";
+      for (int i=0; i<3; ++i) { // probably selected cell, followed by a cell range
+        f << input->readLong(2);
+        f << "x" << input->readLong(2) << ",";
+      }
       f << "],";
       break;
     }
     case 0x10: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -609,13 +630,11 @@ bool GreatWksSSParser::readSpreadsheet()
         break;
       }
       int dim[4];
-      for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+      for (auto &d : dim) d=static_cast<int>(input->readLong(2));
       f << "page=" << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
       if (input->readLong(1)!=1) break;
       input->seek(1,librevenge::RVNG_SEEK_CUR);
-      for (int i=0; i<4; ++i)
-        dim[i]=(int)input->readLong(2);
+      for (auto &d : dim) d=static_cast<int>(input->readLong(2));
       f << "select=" << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ",";
       break;
     }
@@ -636,7 +655,7 @@ bool GreatWksSSParser::readSpreadsheet()
       ascii().addNote("_");
       return true;
     case 0x15: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -647,14 +666,14 @@ bool GreatWksSSParser::readSpreadsheet()
         MWAW_DEBUG_MSG(("GreatWksSSParser::readSpreadsheet: size of grid option zone seems odd\n"));
         break;
       }
-      int val=(int)input->readLong(2);
+      auto val=static_cast<int>(input->readLong(2));
       if (val!=1) f << "unit=" << val << ",";
-      val=(int)input->readLong(2);
+      val=static_cast<int>(input->readLong(2));
       if (val!=5) f << "subdiv=" << val << ",";
       break;
     }
     case 0x16: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -673,7 +692,7 @@ bool GreatWksSSParser::readSpreadsheet()
         // two times the same values: related to header/footer ?
         f << "unkn" << c << "=[";
         for (int j=0; j<8; ++j) {
-          int val=(int)input->readLong(2);
+          auto val=static_cast<int>(input->readLong(2));
           if (val)
             f << val << ",";
           else
@@ -685,7 +704,7 @@ bool GreatWksSSParser::readSpreadsheet()
     }
     case 0x17:
     case 0x18: {
-      long sz=(long) input->readULong(4);
+      auto sz=long(input->readULong(4));
       endPos=pos+6+sz;
       if (!input->checkPosition(endPos)) {
         ok=false;
@@ -718,7 +737,7 @@ bool GreatWksSSParser::readSpreadsheet()
       endPos=input->tell();
       break;
     default: {
-      endPos=pos+6+(long) input->readULong(4);
+      endPos=pos+6+long(input->readULong(4));
       if (type <= 0 || !input->checkPosition(endPos)) {
         ok=false;
         break;
@@ -729,7 +748,7 @@ bool GreatWksSSParser::readSpreadsheet()
     if (!ok && type>=7) {
       // try to use the default value
       input->seek(pos+2, librevenge::RVNG_SEEK_SET);
-      long sz= (long) input->readULong(4);
+      auto sz= long(input->readULong(4));
       endPos=pos+6+sz;
       ok=input->checkPosition(endPos);
     }
@@ -758,15 +777,13 @@ bool GreatWksSSParser::sendSpreadsheet()
     MWAW_DEBUG_MSG(("GreatWksSSParser::sendSpreadsheet: I can not find the listener\n"));
     return false;
   }
-  GreatWksSSParserInternal::Spreadsheet &sheet = m_state->m_spreadsheet;
-  size_t numCell = sheet.m_cells.size();
+  auto &sheet = m_state->m_spreadsheet;
 
   listener->openSheet(sheet.convertInPoint(sheet.m_widthCols), librevenge::RVNG_POINT, std::vector<int>(), sheet.m_name);
   m_document->getGraphParser()->sendPageGraphics();
 
   int prevRow = -1;
-  for (size_t i = 0; i < numCell; i++) {
-    GreatWksSSParserInternal::Cell cell= sheet.m_cells[i];
+  for (auto cell : sheet.m_cells) {
     if (cell.position()[1]>prevRow+1) {
       while (cell.position()[1] > prevRow+1) {
         if (prevRow != -1) listener->closeSheetRow();
@@ -774,16 +791,16 @@ bool GreatWksSSParser::sendSpreadsheet()
         double h=sheet.getRowHeight(prevRow+1, numRepeat);
         if (cell.position()[1]<prevRow+1+numRepeat)
           numRepeat=cell.position()[1]-1-prevRow;
-        listener->openSheetRow((float) h, librevenge::RVNG_POINT, numRepeat);
+        listener->openSheetRow(float(h), librevenge::RVNG_POINT, numRepeat);
         prevRow+=numRepeat;
       }
     }
     if (cell.position()[1] > prevRow) {
       if (prevRow != -1) listener->closeSheetRow();
-      listener->openSheetRow((float) sheet.getRowHeight(++prevRow), librevenge::RVNG_POINT);
+      listener->openSheetRow(float(sheet.getRowHeight(++prevRow)), librevenge::RVNG_POINT);
     }
 
-    GreatWksSSParserInternal::Style style=m_state->getStyle(cell.m_style);
+    auto style=m_state->getStyle(cell.m_style);
     cell.setFont(style.m_font);
     if (!style.m_backgroundColor.isWhite())
       cell.setBackgroundColor(style.m_backgroundColor);
@@ -792,7 +809,7 @@ bool GreatWksSSParser::sendSpreadsheet()
       listener->setFont(style.m_font);
       input->seek(cell.m_content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
       while (!input->isEnd() && input->tell()<cell.m_content.m_textEntry.end()) {
-        unsigned char c=(unsigned char) input->readULong(1);
+        auto c=static_cast<unsigned char>(input->readULong(1));
         if (c==0xd)
           listener->insertEOL();
         else
@@ -816,7 +833,7 @@ bool GreatWksSSParser::readChart()
   libmwaw::DebugStream f;
 
   f << "Entries(Chart):";
-  long sz=(long) input->readULong(4);
+  auto sz=long(input->readULong(4));
   long endPos=pos+4+sz;
   if (sz < 4 || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("GreatWksSSParser::readChart: can not find the chart zone size\n"));
@@ -825,8 +842,8 @@ bool GreatWksSSParser::readChart()
     ascii().addNote(f.str().c_str());
     return false;
   }
-  int N=(int) input->readULong(2);
-  int fSz=(int) input->readULong(2);
+  auto N=static_cast<int>(input->readULong(2));
+  auto fSz=static_cast<int>(input->readULong(2));
   if (N) f << "N=" << N << ",";
   if ((N && fSz!=0x14) || long(4+N*fSz)!=sz) {
     MWAW_DEBUG_MSG(("GreatWksSSParser::readChart: can not find the number of chart\n"));
@@ -862,7 +879,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   libmwaw::DebugStream f;
 
   f << "Entries(Cell):";
-  long sz=(long) input->readULong(4);
+  auto sz=long(input->readULong(4));
   long endPos=pos+4+sz;
   if (sz < 0x12 || !input->checkPosition(endPos)) {
     MWAW_DEBUG_MSG(("GreatWksSSParser::readCell: can not find a cell\n"));
@@ -872,14 +889,13 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
     return false;
   }
   int cPos[2];
-  for (int i=0; i<2; ++i)
-    cPos[i]=(int) input->readLong(2);
+  for (auto &c : cPos) c=static_cast<int>(input->readLong(2));
   MWAWVec2i cellPos(cPos[0]-1,cPos[1]-1);
   cell.setPosition(cellPos);
   f << "cell=" << cellPos << ",";
-  cell.m_style=(int) input->readLong(2);
+  cell.m_style=static_cast<int>(input->readLong(2));
   f << "style" << cell.m_style << ",";
-  int val=(int) input->readULong(2);
+  auto val=static_cast<int>(input->readULong(2));
   int form=(val>>8)&0x1f;
   MWAWCell::Format format;
   format.m_format=MWAWCell::F_TEXT;
@@ -920,7 +936,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   case 16:
   case 17:
   case 18: {
-    static char const *(wh[]) = {
+    static char const *wh[] = {
       "%m/%d/%y", "%b %d, %Y", "%b %Y", "%b %d", "%B %d, %Y", "%B %Y", "%B %d", "%a, %b %d, %Y", "%A, %B %d, %Y"
     };
     format.m_format=MWAWCell::F_DATE;
@@ -932,7 +948,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   case 21:
   case 22:
   case 23: {
-    static char const *(wh[]) = {
+    static char const *wh[] = {
       "%H:%M", "%H:%M:%S", "%I:%M %p", "%I:%M:%S %p"
     };
     format.m_format=MWAWCell::F_TIME;
@@ -952,7 +968,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   val &=0x20F0;
   if (val)
     f << "fl0=" << std::hex << val << std::dec << ",";
-  val=(int) input->readULong(2);
+  val=static_cast<int>(input->readULong(2));
   if (val&0xf) {
     f << "bord=";
     int wh=0;
@@ -1003,10 +1019,10 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   val &= 0x78F0;
   if (val)
     f << "fl1=" << std::hex << val << std::dec << ",";
-  val=(int) input->readULong(2); // small number between 0 and 255
+  val=static_cast<int>(input->readULong(2)); // small number between 0 and 255
   if (val)
     f << "f0=" << val << ",";
-  int formSz=(int) input->readULong(2);
+  auto formSz=static_cast<int>(input->readULong(2));
   if (formSz&0x8000) { // appear if now, rand, ... is used
     f << "formula[update],";
     formSz &= 0x7FFF;
@@ -1032,8 +1048,8 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
         content.m_formula=formula;
       }
       f << "[";
-      for (size_t l=0; l < formula.size(); ++l)
-        f << formula[l];
+      for (auto const &fo : formula)
+        f << fo;
       f << "]" << error << ",";
       if (formSz&1) formSz+=1;
       input->seek(formulaPos+formSz, librevenge::RVNG_SEEK_SET);
@@ -1041,7 +1057,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
   }
 
   if (ok && input->tell()+4<=endPos) {
-    int typeVal=(int) input->readLong(2);
+    auto typeVal=static_cast<int>(input->readLong(2));
     long valuePos=input->tell();
     switch (typeVal) {
     case 1: // no value
@@ -1073,7 +1089,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
     case 7: {
       if (content.m_contentType!=MWAWCellContent::C_FORMULA)
         content.m_contentType=MWAWCellContent::C_TEXT;
-      int sSz=(int) input->readULong(1);
+      auto sSz=static_cast<int>(input->readULong(1));
       if (valuePos+1+sSz+2>endPos) {
         MWAW_DEBUG_MSG(("GreatWksSSParser::readCell: can not read a string value\n"));
         f << "###";
@@ -1084,7 +1100,7 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
       content.m_textEntry.setLength(sSz);
       std::string text("");
       for (int i=0; i<sSz; ++i)
-        text += (char) input->readULong(1);
+        text += char(input->readULong(1));
       f << "\"" << text << "\",";
       if ((sSz%2)==0)
         input->seek(1, librevenge::RVNG_SEEK_CUR);
@@ -1133,15 +1149,17 @@ bool GreatWksSSParser::readCell(GreatWksSSParserInternal::Cell &cell)
     }
   }
   if (ok && input->tell()+2<=endPos) {
-    val =(int) input->readULong(2);
+    val =static_cast<int>(input->readULong(2));
     if (!input->checkPosition(input->tell()+4*val)) {
       MWAW_DEBUG_MSG(("GreatWksSSParser::readCell: can not read the number of childs\n"));
       f << "###";
     }
     else if (val) {
       f << "childs=[";
-      for (int i=0; i<val; ++i)
-        f << input->readULong(2) << "x" << input->readULong(2) << ",";
+      for (int i=0; i<val; ++i) {
+        f << input->readULong(2);
+        f << "x" << input->readULong(2) << ",";
+      }
       f << "],";
     }
   }
@@ -1166,7 +1184,7 @@ bool GreatWksSSParser::readStyles()
   libmwaw::DebugStream f;
 
   f << "Entries(Style):";
-  long sz=(long) input->readULong(4);
+  auto sz=long(input->readULong(4));
   long endPos=pos+4+sz;
   int expectedSize=vers==1 ? 18 : 40;
   if (!input->checkPosition(endPos) || (sz%expectedSize)) {
@@ -1178,20 +1196,20 @@ bool GreatWksSSParser::readStyles()
   }
   ascii().addPos(pos-2);
   ascii().addNote(f.str().c_str());
-  int numFonts=int(sz/expectedSize);
+  auto numFonts=int(sz/expectedSize);
   for (int i=0; i <numFonts; ++i) {
     pos=input->tell();
     f.str("");
     f << "Style-" << i << ":";
     GreatWksSSParserInternal::Style style;
     MWAWFont &font=style.m_font;
-    int val=(int) input->readLong(2); // always 0 ?
+    auto val=static_cast<int>(input->readLong(2)); // always 0 ?
     if (val) f << "#unkn=" << val << ",";
-    val=(int) input->readLong(2);
+    val=static_cast<int>(input->readLong(2));
     if (val!=1) f << "used?=" << val << ",";
 
-    font.setId(m_document->getTextParser()->getFontId((int) input->readULong(2)));
-    int flag =(int) input->readULong(2);
+    font.setId(m_document->getTextParser()->getFontId(static_cast<int>(input->readULong(2))));
+    auto flag =static_cast<int>(input->readULong(2));
     uint32_t flags=0;
     if (flag&0x1) flags |= MWAWFont::boldBit;
     if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -1210,10 +1228,9 @@ bool GreatWksSSParser::readStyles()
     flag &=0xD480;
     if (flag) f << "#fl=" << std::hex << flag << std::dec << ",";
     font.setFlags(flags);
-    font.setSize((float) input->readULong(2));
+    font.setSize(float(input->readULong(2)));
     unsigned char color[3];
-    for (int c=0; c<3; ++c)
-      color[c] = (unsigned char)(input->readULong(2)>>8);
+    for (auto &c : color) c= static_cast<unsigned char>(input->readULong(2)>>8);
     font.setColor(MWAWColor(color[0],color[1],color[2]));
     f << font.getDebugString(getParserState()->m_fontConverter) << ",";
     f << "h[line]?=" << input->readULong(2) << ",";
@@ -1225,15 +1242,14 @@ bool GreatWksSSParser::readStyles()
     }
     MWAWColor bfColors[2];
     for (int j=0; j<2; ++j) {  // front/back color?
-      for (int c=0; c<3; ++c)
-        color[c] = (unsigned char)(input->readULong(2)>>8);
+      for (auto &c : color) c= static_cast<unsigned char>(input->readULong(2)>>8);
       MWAWColor col(color[0],color[1],color[2]);
       bfColors[j]=col;
       if ((j==0 && col.isBlack()) || (j==1 && col.isWhite())) continue;
       if (j==0) f << "col[front]=" << MWAWColor(color[0],color[1],color[2]) << ",";
       else f << "col[back]=" << MWAWColor(color[0],color[1],color[2]) << ",";
     }
-    int patId=(int) input->readLong(2);
+    auto patId=static_cast<int>(input->readLong(2));
     if (!patId) {
       style.m_backgroundColor=bfColors[1];
       input->seek(8, librevenge::RVNG_SEEK_CUR);
@@ -1243,8 +1259,7 @@ bool GreatWksSSParser::readStyles()
       MWAWGraphicStyle::Pattern pattern;
       pattern.m_dim=MWAWVec2i(8,8);
       pattern.m_data.resize(8);
-      for (size_t j=0; j < 8; ++j)
-        pattern.m_data[j]=(unsigned char) input->readULong(1);
+      for (auto &data : pattern.m_data) data=static_cast<unsigned char>(input->readULong(1));
       pattern.m_colors[0]=bfColors[1];
       pattern.m_colors[1]=bfColors[0];
       pattern.getAverageColor(style.m_backgroundColor);
