@@ -87,7 +87,7 @@ bool MWAWCell::Format::getNumberingProperties(librevenge::RVNGPropertyList &prop
     propList.insert("librevenge:value-type", "boolean");
     break;
   case F_NUMBER:
-    if (m_digits>0)
+    if (m_digits>-1000)
       propList.insert("number:decimal-places", m_digits);
     if (m_thousandHasSeparator)
       propList.insert("number:grouping", true);
@@ -220,7 +220,7 @@ std::ostream &operator<<(std::ostream &o, MWAWCell::Format const &format)
   }
   o << ",";
 
-  if (format.m_digits != -1) o << "digits=" << format.m_digits << ",";
+  if (format.m_digits > -1000) o << "digits=" << format.m_digits << ",";
   if (format.m_integerDigits != -1) o << "digits[min]=" << format.m_integerDigits << ",";
   if (format.m_numeratorDigits != -1) o << "digits[num]=" << format.m_numeratorDigits << ",";
   if (format.m_denominatorDigits != -1) o << "digits[den]=" << format.m_denominatorDigits << ",";
@@ -252,7 +252,7 @@ int MWAWCell::Format::compare(MWAWCell::Format const &cell) const
 ////////////////////////////////////////////////////////////
 // MWAWCell
 ////////////////////////////////////////////////////////////
-void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, shared_ptr<MWAWFontConverter> fontConverter) const
+void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, std::shared_ptr<MWAWFontConverter> fontConverter) const
 {
   propList.insert("librevenge:column", position()[0]);
   propList.insert("librevenge:row", position()[1]);
@@ -327,6 +327,9 @@ void MWAWCell::addTo(librevenge::RVNGPropertyList &propList, shared_ptr<MWAWFont
 #endif
     MWAW_DEBUG_MSG(("MWAWCell::addTo: called with unknown valign=%d\n", vAlignment()));
   }
+  int rot=int(m_rotation); // int seems better than double
+  if (rot)
+    propList.insert("style:rotation-angle", rot);
 }
 
 std::string MWAWCell::getColumnName(int col)
@@ -478,6 +481,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCell const &cell)
     break;
 #endif
   }
+  if (cell.m_rotation<0 || cell.m_rotation>0)
+    o << "rot=" << cell.m_rotation << ",";
   if (cell.m_extraLine!=MWAWCell::E_None)
     o << cell.m_extraLineType << ",";
   return o;
@@ -505,7 +510,7 @@ bool MWAWCell::sendContent(MWAWListenerPtr, MWAWTable &)
 bool MWAWCellContent::double2Date(double val, int &Y, int &M, int &D)
 {
   /* first convert the date in long*/
-  long numDaysSinceOrigin=long(val+0.4);
+  auto numDaysSinceOrigin=long(val+0.4);
   // checkme: do we need to check before for isNan(val) ?
   if (numDaysSinceOrigin<-10000*365 || numDaysSinceOrigin>10000*365) {
     /* normally, we can expect documents to contain date between 1904
@@ -536,7 +541,7 @@ bool MWAWCellContent::double2Date(double val, int &Y, int &M, int &D)
     numDaysToEndY1=Y*365+(Y>0 ? (Y-1)/4+((century%4)?0:1) : 0);
   }
   // finish to compute the date
-  int numDaysFrom1Jan=int(numDaysSinceOrigin-numDaysToEndY1);
+  auto numDaysFrom1Jan=int(numDaysSinceOrigin-numDaysToEndY1);
   Y+=century*100;
   bool isLeap=(Y%4)==0 && ((Y%400)==0 || (Y%100)!=0);
 
@@ -603,7 +608,7 @@ bool MWAWCellContent::double2String(double val, MWAWCell::Format const &format,
     else s << "false";
     break;
   case MWAWCell::F_NUMBER:
-    if (format.m_digits>=0 && format.m_numberFormat!=MWAWCell::F_NUMBER_GENERIC)
+    if (format.m_digits>-1000 && format.m_numberFormat!=MWAWCell::F_NUMBER_GENERIC)
       s << std::setprecision(format.m_digits);
     switch (format.m_numberFormat) {
     case MWAWCell::F_NUMBER_CURRENCY:
@@ -638,9 +643,11 @@ bool MWAWCellContent::double2String(double val, MWAWCell::Format const &format,
     time.tm_year=Y;
     time.tm_wday=time.tm_yday=time.tm_isdst=-1;
 #if HAVE_STRUCT_TM_TM_ZONE
-    time.tm_zone=0;
+    time.tm_zone=nullptr;
 #endif
     char buf[256];
+#pragma GCC diagnostic ignored "-Wformat-y2k"
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
     if (mktime(&time)==-1 ||
         !strftime(buf, 256, format.m_DTFormat.empty() ? "%m/%d/%y" : format.m_DTFormat.c_str(), &time))
       return false;
@@ -660,7 +667,7 @@ bool MWAWCellContent::double2String(double val, MWAWCell::Format const &format,
     time.tm_year=100;
     time.tm_wday=time.tm_yday=time.tm_isdst=-1;
 #if HAVE_STRUCT_TM_TM_ZONE
-    time.tm_zone=0;
+    time.tm_zone=nullptr;
 #endif
     char buf[256];
     if (mktime(&time)==-1 ||
@@ -703,8 +710,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent const &content)
   break;
   case MWAWCellContent::C_FORMULA:
     o << ",formula=";
-    for (size_t l=0; l < content.m_formula.size(); ++l)
-      o << content.m_formula[l];
+    for (auto &form : content.m_formula)
+      o << form;
     if (content.isValueSet()) o << "[" << content.m_value << "]";
     break;
   case MWAWCellContent::C_UNKNOWN:
@@ -735,8 +742,7 @@ librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyLis
     // we must use the font converter here to get the final string
     pList.insert("librevenge:type","librevenge-text");
     librevenge::RVNGString finalStr("");
-    for (size_t i=0; i<m_content.size(); ++i) {
-      char c=m_content[i];
+    for (auto c : m_content) {
       int unicode=fontConverter.unicode(fontId,static_cast<unsigned char>(c));
       if (unicode==-1) {
         if (c < 0x20 && c!=9) {
@@ -751,6 +757,13 @@ librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyLis
     pList.insert("librevenge:text",finalStr);
     break;
   }
+  case F_Unicode:
+    pList.insert("librevenge:type","librevenge-text");
+    if (m_content.c_str())
+      pList.insert("librevenge:text",librevenge::RVNGString(m_content.c_str()));
+    else
+      pList.insert("librevenge:text","");
+    break;
   case F_Double:
     pList.insert("librevenge:type","librevenge-number");
     pList.insert("librevenge:number",m_doubleValue, librevenge::RVNG_GENERIC);
@@ -761,25 +774,43 @@ librevenge::RVNGPropertyList MWAWCellContent::FormulaInstruction::getPropertyLis
     break;
   case F_Cell:
     pList.insert("librevenge:type","librevenge-cell");
-    pList.insert("librevenge:column",m_position[0][0], librevenge::RVNG_GENERIC);
-    pList.insert("librevenge:row",m_position[0][1], librevenge::RVNG_GENERIC);
-    pList.insert("librevenge:column-absolute",!m_positionRelative[0][0]);
-    pList.insert("librevenge:row-absolute",!m_positionRelative[0][1]);
-    if (!m_sheet.empty())
-      pList.insert("librevenge:sheet-name",m_sheet.c_str());
+    if (m_position[0][0]>=0) {
+      pList.insert("librevenge:column",m_position[0][0]);
+      pList.insert("librevenge:column-absolute",!m_positionRelative[0][0]);
+    }
+    if (m_position[0][1]>=0) {
+      pList.insert("librevenge:row",m_position[0][1]);
+      pList.insert("librevenge:row-absolute",!m_positionRelative[0][1]);
+    }
+    if (!m_sheet[0].empty())
+      pList.insert("librevenge:sheet-name",m_sheet[0]);
+    if (!m_fileName.empty())
+      pList.insert("librevenge:file-name",m_fileName.cstr());
     break;
   case F_CellList:
     pList.insert("librevenge:type","librevenge-cells");
-    pList.insert("librevenge:start-column",m_position[0][0], librevenge::RVNG_GENERIC);
-    pList.insert("librevenge:start-row",m_position[0][1], librevenge::RVNG_GENERIC);
-    pList.insert("librevenge:start-column-absolute",!m_positionRelative[0][0]);
-    pList.insert("librevenge:start-row-absolute",!m_positionRelative[0][1]);
-    pList.insert("librevenge:end-column",m_position[1][0], librevenge::RVNG_GENERIC);
-    pList.insert("librevenge:end-row",m_position[1][1], librevenge::RVNG_GENERIC);
-    pList.insert("librevenge:end-column-absolute",!m_positionRelative[1][0]);
-    pList.insert("librevenge:end-row-absolute",!m_positionRelative[1][1]);
-    if (!m_sheet.empty())
-      pList.insert("librevenge:sheet-name",m_sheet.c_str());
+    if (m_position[0][0]>=0) {
+      pList.insert("librevenge:start-column",m_position[0][0]);
+      pList.insert("librevenge:start-column-absolute",!m_positionRelative[0][0]);
+    }
+    if (m_position[0][1]>=0) {
+      pList.insert("librevenge:start-row",m_position[0][1]);
+      pList.insert("librevenge:start-row-absolute",!m_positionRelative[0][1]);
+    }
+    if (m_position[1][0]>=0) {
+      pList.insert("librevenge:end-column",m_position[1][0]);
+      pList.insert("librevenge:end-column-absolute",!m_positionRelative[1][0]);
+    }
+    if (m_position[1][1]>=0) {
+      pList.insert("librevenge:end-row",m_position[1][1]);
+      pList.insert("librevenge:end-row-absolute",!m_positionRelative[1][1]);
+    }
+    if (!m_sheet[0].empty())
+      pList.insert("librevenge:sheet-name",m_sheet[0]);
+    if (!m_sheet[1].empty())
+      pList.insert("librevenge:end-sheet-name",m_sheet[1]);
+    if (!m_fileName.empty())
+      pList.insert("librevenge:file-name",m_fileName.cstr());
     break;
 #if !defined(__clang__)
   default:
@@ -796,7 +827,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent::FormulaInstruction co
   else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Long)
     o << inst.m_longValue;
   else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Cell) {
-    if (!inst.m_sheet.empty()) o << inst.m_sheet;
+    if (!inst.m_sheet[0].empty()) o << "\"" << inst.m_sheet[0].cstr() << "\"";
+    if (!inst.m_fileName.empty()) o << "[" << inst.m_fileName.cstr() << "]";
     if (!inst.m_positionRelative[0][0]) o << "$";
     if (inst.m_position[0][0]<0) o << "C" << inst.m_position[0][0];
     else {
@@ -808,8 +840,10 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent::FormulaInstruction co
     else o << inst.m_position[0][1];
   }
   else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_CellList) {
-    if (!inst.m_sheet.empty()) o << inst.m_sheet;
+    if (!inst.m_fileName.empty()) o << "[" << inst.m_fileName.cstr() << "]";
     for (int l=0; l<2; ++l) {
+      if (!inst.m_sheet[l].empty() && (l==0 || inst.m_sheet[0]!=inst.m_sheet[1]))
+        o << "\"" << inst.m_sheet[l].cstr() << "\"";
       if (!inst.m_positionRelative[l][0]) o << "$";
       if (inst.m_position[l][0]<0) o << "C" << inst.m_position[l][0];
       else {
@@ -822,7 +856,8 @@ std::ostream &operator<<(std::ostream &o, MWAWCellContent::FormulaInstruction co
       if (l==0) o << ":";
     }
   }
-  else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Text)
+  else if (inst.m_type==MWAWCellContent::FormulaInstruction::F_Text ||
+           inst.m_type==MWAWCellContent::FormulaInstruction::F_Unicode)
     o << "\"" << inst.m_content << "\"";
   else
     o << inst.m_content;
