@@ -35,12 +35,14 @@
  */
 #include <string.h>
 
+#include <algorithm>
 #include <cmath>
 #include <cstring>
 #include <iomanip>
 #include <iostream>
 #include <sstream>
 #include <string>
+#include <utility>
 
 #include <librevenge/librevenge.h>
 
@@ -81,6 +83,7 @@ std::ostream &operator<<(std::ostream &o, MWAWGraphicShape::PathData const &path
     if (path.m_largeAngle) o << ":largeAngle";
     if (path.m_sweep) o << ":sweep";
     if (path.m_rotate<0 || path.m_rotate>0) o << ":rot=" << path.m_rotate;
+    break;
   case 'Z':
     break;
   default:
@@ -136,6 +139,23 @@ void MWAWGraphicShape::PathData::rotate(float angle, MWAWVec2f const &decal)
                    std::sin(angl)*m_x2[0]+std::cos(angl)*m_x2[1])+decal;
 }
 
+void MWAWGraphicShape::PathData::transform(MWAWTransformation const &matrix, float rotation)
+{
+  if (m_type=='Z')
+    return;
+  m_x = matrix*m_x;
+  if (m_type=='A') {
+    m_rotate += rotation;
+    return;
+  }
+  if (m_type=='H' || m_type=='V' || m_type=='M' || m_type=='L' || m_type=='T')
+    return;
+  m_x1 = matrix*m_x1;
+  if (m_type=='Q' || m_type=='S')
+    return;
+  m_x2 = matrix*m_x2;
+}
+
 bool MWAWGraphicShape::PathData::get(librevenge::RVNGPropertyList &list, MWAWVec2f const &orig) const
 {
   list.clear();
@@ -145,31 +165,31 @@ bool MWAWGraphicShape::PathData::get(librevenge::RVNGPropertyList &list, MWAWVec
   if (m_type=='Z')
     return true;
   if (m_type=='H') {
-    list.insert("svg:x",m_x[0]-orig[0], librevenge::RVNG_POINT);
+    list.insert("svg:x",double(m_x[0]-orig[0]), librevenge::RVNG_POINT);
     return true;
   }
   if (m_type=='V') {
-    list.insert("svg:y",m_x[1]-orig[1], librevenge::RVNG_POINT);
+    list.insert("svg:y",double(m_x[1]-orig[1]), librevenge::RVNG_POINT);
     return true;
   }
-  list.insert("svg:x",m_x[0]-orig[0], librevenge::RVNG_POINT);
-  list.insert("svg:y",m_x[1]-orig[1], librevenge::RVNG_POINT);
+  list.insert("svg:x",double(m_x[0]-orig[0]), librevenge::RVNG_POINT);
+  list.insert("svg:y",double(m_x[1]-orig[1]), librevenge::RVNG_POINT);
   if (m_type=='M' || m_type=='L' || m_type=='T')
     return true;
   if (m_type=='A') {
-    list.insert("svg:rx",m_r[0], librevenge::RVNG_POINT);
-    list.insert("svg:ry",m_r[1], librevenge::RVNG_POINT);
+    list.insert("svg:rx",double(m_r[0]), librevenge::RVNG_POINT);
+    list.insert("svg:ry",double(m_r[1]), librevenge::RVNG_POINT);
     list.insert("librevenge:large-arc", m_largeAngle);
     list.insert("librevenge:sweep", m_sweep);
-    list.insert("librevenge:rotate", m_rotate, librevenge::RVNG_GENERIC);
+    list.insert("librevenge:rotate", double(m_rotate), librevenge::RVNG_GENERIC);
     return true;
   }
-  list.insert("svg:x1",m_x1[0]-orig[0], librevenge::RVNG_POINT);
-  list.insert("svg:y1",m_x1[1]-orig[1], librevenge::RVNG_POINT);
+  list.insert("svg:x1",double(m_x1[0]-orig[0]), librevenge::RVNG_POINT);
+  list.insert("svg:y1",double(m_x1[1]-orig[1]), librevenge::RVNG_POINT);
   if (m_type=='Q' || m_type=='S')
     return true;
-  list.insert("svg:x2",m_x2[0]-orig[0], librevenge::RVNG_POINT);
-  list.insert("svg:y2",m_x2[1]-orig[1], librevenge::RVNG_POINT);
+  list.insert("svg:x2",double(m_x2[0]-orig[0]), librevenge::RVNG_POINT);
+  list.insert("svg:y2",double(m_x2[1]-orig[1]), librevenge::RVNG_POINT);
   if (m_type=='C')
     return true;
   MWAW_DEBUG_MSG(("MWAWGraphicShape::PathData::get: unknown command %c\n", m_type));
@@ -201,6 +221,10 @@ int MWAWGraphicShape::PathData::cmp(MWAWGraphicShape::PathData const &a) const
 ////////////////////////////////////////////////////////////
 // MWAWGraphicShape
 ////////////////////////////////////////////////////////////
+MWAWGraphicShape::~MWAWGraphicShape()
+{
+}
+
 MWAWGraphicShape MWAWGraphicShape::line(MWAWVec2f const &orig, MWAWVec2f const &dest)
 {
   MWAWGraphicShape res;
@@ -219,6 +243,13 @@ MWAWGraphicShape MWAWGraphicShape::line(MWAWVec2f const &orig, MWAWVec2f const &
   return res;
 }
 
+MWAWGraphicShape MWAWGraphicShape::measure(MWAWVec2f const &orig, MWAWVec2f const &dest)
+{
+  MWAWGraphicShape res=line(orig,dest);
+  res.m_type= MWAWGraphicShape::Measure;
+  return res;
+}
+
 std::ostream &operator<<(std::ostream &o, MWAWGraphicShape const &sh)
 {
   o << "box=" << sh.m_bdBox << ",";
@@ -230,6 +261,13 @@ std::ostream &operator<<(std::ostream &o, MWAWGraphicShape const &sh)
     else
       o << "pts=" << sh.m_vertices[0] << "<->" << sh.m_vertices[1] << ",";
     break;
+  case MWAWGraphicShape::Measure:
+    o << "measure,";
+    if (sh.m_vertices.size()!=2)
+      o << "###pts,";
+    else
+      o << "pts=" << sh.m_vertices[0] << "<->" << sh.m_vertices[1] << ",";
+    break;
   case MWAWGraphicShape::Rectangle:
     o << "rect,";
     if (sh.m_formBox!=sh.m_bdBox)
@@ -247,19 +285,25 @@ std::ostream &operator<<(std::ostream &o, MWAWGraphicShape const &sh)
     o << "angle=" << sh.m_arcAngles << ",";
     break;
   case MWAWGraphicShape::Polygon:
-    o << "polygons,pts=[";
-    for (size_t pt=0; pt < sh.m_vertices.size(); ++pt)
-      o << sh.m_vertices[pt] << ",";
+  case MWAWGraphicShape::Polyline:
+    if (sh.m_type==MWAWGraphicShape::Polygon)
+      o << "polygon,pts=[";
+    else
+      o << "polyline,pts=[";
+    for (auto const &pt : sh.m_vertices)
+      o << pt << ",";
     o << "],";
     break;
   case MWAWGraphicShape::Path:
     o << "path,pts=[";
-    for (size_t pt=0; pt < sh.m_path.size(); ++pt)
-      o << sh.m_path[pt] << ",";
+    for (auto const &pt : sh.m_path)
+      o << pt << ",";
     o << "],";
     break;
   case MWAWGraphicShape::ShapeUnknown:
+#if !defined(__clang__)
   default:
+#endif
     o << "###unknown[shape],";
     break;
   }
@@ -271,21 +315,21 @@ int MWAWGraphicShape::cmp(MWAWGraphicShape const &a) const
 {
   if (m_type < a.m_type) return 1;
   if (m_type > a.m_type) return -1;
-  int diff = m_bdBox.cmp(a.m_bdBox);
-  if (diff) return diff;
-  diff = m_formBox.cmp(a.m_formBox);
-  if (diff) return diff;
-  diff = m_cornerWidth.cmp(a.m_cornerWidth);
+  if (m_bdBox < a.m_bdBox) return 1;
+  if (m_bdBox > a.m_bdBox) return -1;
+  if (m_formBox < a.m_formBox) return 1;
+  if (m_formBox > a.m_formBox) return -1;
+  int diff = m_cornerWidth.cmp(a.m_cornerWidth);
   if (diff) return diff;
   diff = m_arcAngles.cmp(a.m_arcAngles);
   if (diff) return diff;
-  if (m_vertices.size()<a.m_vertices.size()) return -1;
+  if (m_vertices.size()<a.m_vertices.size()) return 1;
   if (m_vertices.size()>a.m_vertices.size()) return -1;
   for (size_t pt=0; pt < m_vertices.size(); ++pt) {
     diff = m_vertices[pt].cmp(a.m_vertices[pt]);
     if (diff) return diff;
   }
-  if (m_path.size()<a.m_path.size()) return -1;
+  if (m_path.size()<a.m_path.size()) return 1;
   if (m_path.size()>a.m_path.size()) return -1;
   for (size_t pt=0; pt < m_path.size(); ++pt) {
     diff = m_path[pt].cmp(a.m_path[pt]);
@@ -303,7 +347,7 @@ MWAWBox2f MWAWGraphicShape::getBdBox(MWAWGraphicStyle const &style, bool moveToO
     bdBox.extend(style.m_lineWidth/2.f);
   if (m_type==Line) {
     // fixme: add 4pt for each arrows
-    int numArrows=(style.m_arrows[0] ? 1 : 0)+(style.m_arrows[1] ? 1 : 0);
+    int numArrows=(style.m_arrows[0].isEmpty() ? 0 : 1)+(style.m_arrows[1].isEmpty() ? 0 : 1);
     if (numArrows) bdBox.extend(float(2*numArrows));
   }
   return bdBox;
@@ -315,30 +359,30 @@ void MWAWGraphicShape::translate(MWAWVec2f const &decal)
     return;
   m_bdBox=MWAWBox2f(m_bdBox.min()+decal, m_bdBox.max()+decal);
   m_formBox=MWAWBox2f(m_formBox.min()+decal, m_formBox.max()+decal);
-  for (size_t pt=0; pt<m_vertices.size(); ++pt)
-    m_vertices[pt]+=decal;
-  for (size_t pt=0; pt<m_path.size(); ++pt)
-    m_path[pt].translate(decal);
+  for (auto &pt : m_vertices)
+    pt+=decal;
+  for (auto &pt : m_path)
+    pt.translate(decal);
 }
 
 void MWAWGraphicShape::scale(MWAWVec2f const &scaling)
 {
+  // checkme: does not work for symetry if shape is an arc...
   m_bdBox=MWAWBox2f(MWAWVec2f(scaling[0]*m_bdBox.min()[0],scaling[1]*m_bdBox.min()[1]),
                     MWAWVec2f(scaling[0]*m_bdBox.max()[0],scaling[1]*m_bdBox.max()[1]));
   m_formBox=MWAWBox2f(MWAWVec2f(scaling[0]*m_formBox.min()[0],scaling[1]*m_formBox.min()[1]),
                       MWAWVec2f(scaling[0]*m_formBox.max()[0],scaling[1]*m_formBox.max()[1]));
-  for (size_t pt=0; pt<m_vertices.size(); ++pt)
-    m_vertices[pt]=MWAWVec2f(scaling[0]*m_vertices[pt][0],
-                             scaling[1]*m_vertices[pt][1]);
-  for (size_t pt=0; pt<m_path.size(); ++pt)
-    m_path[pt].scale(scaling);
+  for (auto &pt : m_vertices)
+    pt=MWAWVec2f(scaling[0]*pt[0], scaling[1]*pt[1]);
+  for (auto &pt : m_path)
+    pt.scale(scaling);
 }
 
 MWAWGraphicShape MWAWGraphicShape::rotate(float angle, MWAWVec2f const &center) const
 {
   while (angle >= 360) angle -= 360;
   while (angle <= -360) angle += 360;
-  if (angle >= -1e-3 && angle <= 1e-3) return *this;
+  if (angle >= -1.e-3f && angle <= 1.e-3f) return *this;
   float angl=angle*float(M_PI/180.);
   MWAWVec2f decal=center-MWAWVec2f(std::cos(angl)*center[0]-std::sin(angl)*center[1],
                                    std::sin(angl)*center[0]+std::cos(angl)*center[1]);
@@ -351,12 +395,66 @@ MWAWGraphicShape MWAWGraphicShape::rotate(float angle, MWAWVec2f const &center)
     else fBox=fBox.getUnion(MWAWBox2f(pt,pt));
   }
   MWAWGraphicShape res = path(fBox);
-  res.m_path=getPath();
-  for (size_t p=0; p < res.m_path.size(); p++)
-    res.m_path[p].rotate(angle, decal);
+  res.m_path=getPath(false);
+  for (auto &pt : res.m_path)
+    pt.rotate(angle, decal);
   return res;
 }
 
+MWAWGraphicShape MWAWGraphicShape::transform(MWAWTransformation const &matrix) const
+{
+  if (matrix.isIdentity()) return *this;
+  if (matrix[0][1]<=0 && matrix[0][1]>=0 && matrix[1][0]<=0 && matrix[1][0]>=0) {
+    MWAWGraphicShape res=*this;
+    if (matrix[0][0]<1 || matrix[0][0]>1 || matrix[1][1]<1 || matrix[1][1]>1)
+      res.scale(MWAWVec2f(matrix[0][0], matrix[1][1]));
+    res.translate(MWAWVec2f(matrix[0][2],matrix[1][2]));
+    return res;
+  }
+
+  MWAWBox2f fBox;
+  for (int i=0; i < 4; ++i) {
+    MWAWVec2f pt = matrix*MWAWVec2f(m_bdBox[i%2][0],m_bdBox[i/2][1]);
+    if (i==0) fBox=MWAWBox2f(pt,pt);
+    else fBox=fBox.getUnion(MWAWBox2f(pt,pt));
+  }
+  MWAWGraphicShape res = path(fBox);
+  res.m_path=getPath(true);
+
+  MWAWTransformation transf;
+  float rotation=0;
+  MWAWVec2f shearing;
+  if (!matrix.decompose(rotation,shearing,transf,fBox.center()))
+    rotation=0;
+  for (auto &pt : res.m_path)
+    pt.transform(matrix, rotation);
+  return res;
+}
+
+bool MWAWGraphicShape::addPathTo(MWAWVec2f const &orig, librevenge::RVNGPropertyListVector &vect) const
+{
+  MWAWVec2f decal=orig-m_bdBox[0];
+  std::vector<MWAWGraphicShape::PathData> fPath=getPath(false);
+  size_t n=fPath.size();
+  if (!n) {
+    MWAW_DEBUG_MSG(("MWAWGraphicShape::addPathTo: can not find the path\n"));
+    return false;
+  }
+  librevenge::RVNGPropertyList list;
+  for (auto const &pt : fPath) {
+    list.clear();
+    if (pt.get(list, -1.0f*decal))
+      vect.append(list);
+  }
+  if (fPath[n-1].m_type != 'Z') {
+    // odg need a closed path to draw surface, so ...
+    list.clear();
+    list.insert("librevenge:path-action", "Z");
+    vect.append(list);
+  }
+  return true;
+}
+
 MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool asSurface, librevenge::RVNGPropertyList &propList) const
 {
   MWAWVec2f pt;
@@ -365,14 +463,17 @@ MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool as
   MWAWVec2f decal=orig-m_bdBox[0];
   switch (m_type) {
   case Line:
+  case Measure:
     if (m_vertices.size()!=2) break;
+    if (m_type==Measure)
+      propList.insert("draw:show-unit", true);
     pt=m_vertices[0]+decal;
-    list.insert("svg:x",pt.x(), librevenge::RVNG_POINT);
-    list.insert("svg:y",pt.y(), librevenge::RVNG_POINT);
+    list.insert("svg:x",double(pt.x()), librevenge::RVNG_POINT);
+    list.insert("svg:y",double(pt.y()), librevenge::RVNG_POINT);
     vect.append(list);
     pt=m_vertices[1]+decal;
-    list.insert("svg:x",pt.x(), librevenge::RVNG_POINT);
-    list.insert("svg:y",pt.y(), librevenge::RVNG_POINT);
+    list.insert("svg:x",double(pt.x()), librevenge::RVNG_POINT);
+    list.insert("svg:y",double(pt.y()), librevenge::RVNG_POINT);
     vect.append(list);
     propList.insert("svg:points", vect);
     return C_Polyline;
@@ -382,24 +483,24 @@ MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool as
       propList.insert("svg:ry",double(m_cornerWidth[1]), librevenge::RVNG_POINT);
     }
     pt=m_formBox[0]+decal;
-    propList.insert("svg:x",pt.x(), librevenge::RVNG_POINT);
-    propList.insert("svg:y",pt.y(), librevenge::RVNG_POINT);
+    propList.insert("svg:x",double(pt.x()), librevenge::RVNG_POINT);
+    propList.insert("svg:y",double(pt.y()), librevenge::RVNG_POINT);
     pt=m_formBox.size();
-    propList.insert("svg:width",pt.x(), librevenge::RVNG_POINT);
-    propList.insert("svg:height",pt.y(), librevenge::RVNG_POINT);
+    propList.insert("svg:width",double(pt.x()), librevenge::RVNG_POINT);
+    propList.insert("svg:height",double(pt.y()), librevenge::RVNG_POINT);
     return C_Rectangle;
   case Circle:
-    pt=0.5*(m_formBox[0]+m_formBox[1])+decal;
-    propList.insert("svg:cx",pt.x(), librevenge::RVNG_POINT);
-    propList.insert("svg:cy",pt.y(), librevenge::RVNG_POINT);
-    pt=0.5*(m_formBox[1]-m_formBox[0]);
-    propList.insert("svg:rx",pt.x(), librevenge::RVNG_POINT);
-    propList.insert("svg:ry",pt.y(), librevenge::RVNG_POINT);
+    pt=0.5f*(m_formBox[0]+m_formBox[1])+decal;
+    propList.insert("svg:cx",double(pt.x()), librevenge::RVNG_POINT);
+    propList.insert("svg:cy",double(pt.y()), librevenge::RVNG_POINT);
+    pt=0.5f*(m_formBox[1]-m_formBox[0]);
+    propList.insert("svg:rx",double(pt.x()), librevenge::RVNG_POINT);
+    propList.insert("svg:ry",double(pt.y()), librevenge::RVNG_POINT);
     return C_Ellipse;
   case Arc:
   case Pie: {
-    MWAWVec2f center=0.5*(m_formBox[0]+m_formBox[1])+decal;
-    MWAWVec2f rad=0.5*(m_formBox[1]-m_formBox[0]);
+    MWAWVec2f center=0.5f*(m_formBox[0]+m_formBox[1])+decal;
+    MWAWVec2f rad=0.5f*(m_formBox[1]-m_formBox[0]);
     float angl0=m_arcAngles[0];
     float angl1=m_arcAngles[1];
     if (rad[1]<0) {
@@ -421,15 +522,15 @@ MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool as
     if (addCenter) {
       pt=center;
       list.insert("librevenge:path-action", "M");
-      list.insert("svg:x",pt.x(), librevenge::RVNG_POINT);
-      list.insert("svg:y",pt.y(), librevenge::RVNG_POINT);
+      list.insert("svg:x",double(pt.x()), librevenge::RVNG_POINT);
+      list.insert("svg:y",double(pt.y()), librevenge::RVNG_POINT);
       vect.append(list);
     }
     list.clear();
     pt=center+MWAWVec2f(std::cos(angl)*rad[0],-std::sin(angl)*rad[1]);
     list.insert("librevenge:path-action", addCenter ? "L" : "M");
-    list.insert("svg:x",pt.x(), librevenge::RVNG_POINT);
-    list.insert("svg:y",pt.y(), librevenge::RVNG_POINT);
+    list.insert("svg:x",double(pt.x()), librevenge::RVNG_POINT);
+    list.insert("svg:y",double(pt.y()), librevenge::RVNG_POINT);
     vect.append(list);
 
     list.clear();
@@ -438,10 +539,10 @@ MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool as
     list.insert("librevenge:path-action", "A");
     list.insert("librevenge:large-arc", !(angl1-angl0<180.f));
     list.insert("librevenge:sweep", false);
-    list.insert("svg:rx",rad.x(), librevenge::RVNG_POINT);
-    list.insert("svg:ry",rad.y(), librevenge::RVNG_POINT);
-    list.insert("svg:x",pt.x(), librevenge::RVNG_POINT);
-    list.insert("svg:y",pt.y(), librevenge::RVNG_POINT);
+    list.insert("svg:rx",double(rad.x()), librevenge::RVNG_POINT);
+    list.insert("svg:ry",double(rad.y()), librevenge::RVNG_POINT);
+    list.insert("svg:x",double(pt.x()), librevenge::RVNG_POINT);
+    list.insert("svg:y",double(pt.y()), librevenge::RVNG_POINT);
     vect.append(list);
     if (asSurface) {
       list.clear();
@@ -452,25 +553,26 @@ MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool as
     propList.insert("svg:d", vect);
     return C_Path;
   }
-  case Polygon: {
+  case Polygon:
+  case Polyline: {
     size_t n=m_vertices.size();
     if (n<2) break;
-    for (size_t i = 0; i < n; ++i) {
+    for (auto point : m_vertices) {
       list.clear();
-      pt=m_vertices[i]+decal;
-      list.insert("svg:x", pt.x(), librevenge::RVNG_POINT);
-      list.insert("svg:y", pt.y(), librevenge::RVNG_POINT);
+      point += decal;
+      list.insert("svg:x", double(point.x()), librevenge::RVNG_POINT);
+      list.insert("svg:y", double(point.y()), librevenge::RVNG_POINT);
       vect.append(list);
     }
     propList.insert("svg:points", vect);
-    return asSurface ? C_Polygon : C_Polyline;
+    return (asSurface && m_type==Polygon) ? C_Polygon : C_Polyline;
   }
   case Path: {
     size_t n=m_path.size();
     if (!n) break;
-    for (size_t c=0; c < n; ++c) {
+    for (auto const &point : m_path) {
       list.clear();
-      if (m_path[c].get(list, -1.0f*decal))
+      if (point.get(list, -1.0f*decal))
         vect.append(list);
     }
     if (asSurface && m_path[n-1].m_type != 'Z') {
@@ -483,19 +585,26 @@ MWAWGraphicShape::Command MWAWGraphicShape::addTo(MWAWVec2f const &orig, bool as
     return C_Path;
   }
   case ShapeUnknown:
+#if !defined(__clang__)
   default:
+#endif
     break;
   }
   MWAW_DEBUG_MSG(("MWAWGraphicShape::addTo: can not send a shape with type=%d\n", int(m_type)));
   return C_Bad;
 }
 
-std::vector<MWAWGraphicShape::PathData> MWAWGraphicShape::getPath() const
+std::vector<MWAWGraphicShape::PathData> MWAWGraphicShape::getPath(bool forTransformation) const
 {
   std::vector<MWAWGraphicShape::PathData> res;
+  float const delta=0.55228f;
   switch (m_type) {
+  case Measure:
+    MWAW_DEBUG_MSG(("MWAWGraphicShape::getPath: called on a measure, transform it in line\n"));
+    MWAW_FALLTHROUGH;
   case Line:
-  case Polygon: {
+  case Polygon:
+  case Polyline: {
     size_t n=m_vertices.size();
     if (n<2) break;
     res.push_back(PathData('M',m_vertices[0]));
@@ -506,21 +615,45 @@ std::vector<MWAWGraphicShape::PathData> MWAWGraphicShape::getPath() const
   case Rectangle:
     if (m_cornerWidth[0] > 0 && m_cornerWidth[1] > 0) {
       MWAWBox2f box=m_formBox;
+      if (box.min()[0]>box.max()[0]) std::swap(box.min()[0],box.max()[0]);
+      if (box.min()[1]>box.max()[1]) std::swap(box.min()[1],box.max()[1]);
       MWAWVec2f c=m_cornerWidth;
-      res.push_back(PathData('M',MWAWVec2f(box[1][0]-c[0],box[0][1])));
-      PathData data('A',MWAWVec2f(box[1][0],box[0][1]+c[1]));
-      data.m_r=c;
-      data.m_sweep=true;
-      res.push_back(data);
-      res.push_back(PathData('L',MWAWVec2f(box[1][0],box[1][1]-c[1])));
-      data.m_x=MWAWVec2f(box[1][0]-c[0],box[1][1]);
-      res.push_back(data);
-      res.push_back(PathData('L',MWAWVec2f(box[0][0]+c[0],box[1][1])));
-      data.m_x=MWAWVec2f(box[0][0],box[1][1]-c[1]);
-      res.push_back(data);
-      res.push_back(PathData('L',MWAWVec2f(box[0][0],box[0][1]+c[1])));
-      data.m_x=MWAWVec2f(box[0][0]+c[0],box[0][1]);
-      res.push_back(data);
+      if (2*c[0]>box.size()[0]) c[0]=0.5f*box.size()[0];
+      if (2*c[1]>box.size()[1]) c[1]=0.5f*box.size()[1];
+      if (forTransformation) {
+        MWAWVec2f pt0(box[1][0]-c[0],box[0][1]);
+        res.push_back(PathData('M',pt0));
+        MWAWVec2f pt1(box[1][0],box[0][1]+c[1]);
+        res.push_back(PathData('C',pt1,pt0+MWAWVec2f(delta*c[0],0),pt1-MWAWVec2f(0,delta*c[1])));
+        pt0=MWAWVec2f(box[1][0],box[1][1]-c[1]);
+        res.push_back(PathData('L',pt0));
+        pt1=MWAWVec2f(box[1][0]-c[0],box[1][1]);
+        res.push_back(PathData('C',pt1,pt0+MWAWVec2f(0,delta*c[1]),pt1+MWAWVec2f(delta*c[0],0)));
+        pt0=MWAWVec2f(box[0][0]+c[0],box[1][1]);
+        res.push_back(PathData('L',pt0));
+        pt1=MWAWVec2f(box[0][0],box[1][1]-c[1]);
+        res.push_back(PathData('C',pt1,pt0-MWAWVec2f(delta*c[0],0),pt1+MWAWVec2f(0,delta*c[1])));
+        pt0=MWAWVec2f(box[0][0],box[0][1]+c[1]);
+        res.push_back(PathData('L',pt0));
+        pt1=MWAWVec2f(box[0][0]+c[0],box[0][1]);
+        res.push_back(PathData('C',pt1,pt0-MWAWVec2f(0,delta*c[1]),pt1-MWAWVec2f(delta*c[0],0)));
+      }
+      else {
+        res.push_back(PathData('M',MWAWVec2f(box[1][0]-c[0],box[0][1])));
+        PathData data('A',MWAWVec2f(box[1][0],box[0][1]+c[1]));
+        data.m_r=c;
+        data.m_sweep=true;
+        res.push_back(data);
+        res.push_back(PathData('L',MWAWVec2f(box[1][0],box[1][1]-c[1])));
+        data.m_x=MWAWVec2f(box[1][0]-c[0],box[1][1]);
+        res.push_back(data);
+        res.push_back(PathData('L',MWAWVec2f(box[0][0]+c[0],box[1][1])));
+        data.m_x=MWAWVec2f(box[0][0],box[1][1]-c[1]);
+        res.push_back(data);
+        res.push_back(PathData('L',MWAWVec2f(box[0][0],box[0][1]+c[1])));
+        data.m_x=MWAWVec2f(box[0][0]+c[0],box[0][1]);
+        res.push_back(data);
+      }
       res.push_back(PathData('Z'));
       break;
     }
@@ -531,21 +664,38 @@ std::vector<MWAWGraphicShape::PathData> MWAWGraphicShape::getPath() const
     res.push_back(PathData('Z'));
     break;
   case Circle: {
-    MWAWVec2f pt0 = MWAWVec2f(m_formBox[0][0],0.5f*(m_formBox[0][1]+m_formBox[1][1]));
-    MWAWVec2f pt1 = MWAWVec2f(m_formBox[1][0],pt0[1]);
-    res.push_back(PathData('M',pt0));
-    PathData data('A',pt1);
-    data.m_r=0.5*(m_formBox[1]-m_formBox[0]);
-    data.m_largeAngle=true;
-    res.push_back(data);
-    data.m_x=pt0;
-    res.push_back(data);
+    if (forTransformation) {
+      MWAWVec2f center=m_formBox.center();
+      MWAWVec2f dir=0.5f*delta*(m_formBox[1]-m_formBox[0]);
+      MWAWVec2f pt0(m_formBox[0][0],center[1]);
+      res.push_back(PathData('M',pt0));
+      MWAWVec2f pt1(center[0],m_formBox[0][1]);
+      res.push_back(PathData('C',pt1, pt0-MWAWVec2f(0,dir[1]), pt1-MWAWVec2f(dir[0],0)));
+      pt0=MWAWVec2f(m_formBox[1][0],center[1]);
+      res.push_back(PathData('C',pt0, pt1+MWAWVec2f(dir[0],0), pt0-MWAWVec2f(0,dir[1])));
+      pt1=MWAWVec2f(center[0],m_formBox[1][1]);
+      res.push_back(PathData('C',pt1, pt0+MWAWVec2f(0,dir[1]), pt1+MWAWVec2f(dir[0],0)));
+      pt0=MWAWVec2f(m_formBox[0][0],center[1]);
+      res.push_back(PathData('C',pt0, pt1-MWAWVec2f(dir[0],0), pt0+MWAWVec2f(0,dir[1])));
+      res.push_back(PathData('Z'));
+    }
+    else {
+      MWAWVec2f pt0 = MWAWVec2f(m_formBox[0][0],0.5f*(m_formBox[0][1]+m_formBox[1][1]));
+      MWAWVec2f pt1 = MWAWVec2f(m_formBox[1][0],pt0[1]);
+      res.push_back(PathData('M',pt0));
+      PathData data('A',pt1);
+      data.m_r=0.5f*(m_formBox[1]-m_formBox[0]);
+      data.m_largeAngle=true;
+      res.push_back(data);
+      data.m_x=pt0;
+      res.push_back(data);
+    }
     break;
   }
   case Arc:
   case Pie: {
-    MWAWVec2f center=0.5*(m_formBox[0]+m_formBox[1]);
-    MWAWVec2f rad=0.5*(m_formBox[1]-m_formBox[0]);
+    MWAWVec2f center=0.5f*(m_formBox[0]+m_formBox[1]);
+    MWAWVec2f rad=0.5f*(m_formBox[1]-m_formBox[0]);
     float angl0=m_arcAngles[0];
     float angl1=m_arcAngles[1];
     if (rad[1]<0) {
@@ -568,18 +718,38 @@ std::vector<MWAWGraphicShape::PathData> MWAWGraphicShape::getPath() const
       res.push_back(PathData('M', center));
     MWAWVec2f pt=center+MWAWVec2f(std::cos(angl)*rad[0],-std::sin(angl)*rad[1]);
     res.push_back(PathData(addCenter ? 'L' : 'M', pt));
-    angl=angl1*float(M_PI/180.);
-    pt=center+MWAWVec2f(std::cos(angl)*rad[0],-std::sin(angl)*rad[1]);
-    PathData data('A',pt);
-    data.m_largeAngle=(angl1-angl0>=180.f);
-    data.m_r=rad;
-    res.push_back(data);
+    if (!forTransformation) {
+      angl=angl1*float(M_PI/180.);
+      pt=center+MWAWVec2f(std::cos(angl)*rad[0],-std::sin(angl)*rad[1]);
+      PathData data('A',pt);
+      data.m_largeAngle=(angl1-angl0>=180.f);
+      data.m_r=rad;
+      res.push_back(data);
+    }
+    else {
+      int N=int(angl1-angl0)/90;
+      float dAngle=float(angl1-angl0)/float(N+1);
+      for (int i=0; i<=N; ++i) {
+        float newAngl= i==N ? angl1 : angl0+float(i+1)*dAngle;
+        newAngl*=float(M_PI/180.);
+        MWAWVec2f newPt=center+MWAWVec2f(std::cos(angl1)*rad[0],-std::sin(angl1)*rad[1]);
+        MWAWVec2f dir(-std::sin(angl)*rad[0],-std::cos(angl)*rad[1]);
+        MWAWVec2f newDir(-std::sin(newAngl)*rad[0],-std::cos(newAngl)*rad[1]);
+        float deltaDir=4/3.f*std::tan((newAngl-angl)/4);
+        res.push_back(PathData('C',newPt,pt+deltaDir*dir,newPt-deltaDir*newDir));
+        pt=newPt;
+        angl=newAngl;
+      }
+      if (m_type==Pie) res.push_back(PathData('Z'));
+    }
     break;
   }
   case Path:
     return m_path;
   case ShapeUnknown:
+#if !defined(__clang__)
   default:
+#endif
     MWAW_DEBUG_MSG(("MWAWGraphicShape::getPath: unexpected type\n"));
     break;
   }
