@@ -46,10 +46,10 @@
 
 #include "MWAWHeader.hxx"
 
-MWAWHeader::MWAWHeader(MWAWDocument::Type documentType, int vers, MWAWDocument::Kind kind) :
-  m_version(vers),
-  m_docType(documentType),
-  m_docKind(kind)
+MWAWHeader::MWAWHeader(MWAWDocument::Type documentType, int vers, MWAWDocument::Kind kind)
+  : m_version(vers)
+  , m_docType(documentType)
+  , m_docKind(kind)
 {
 }
 
@@ -61,7 +61,7 @@ MWAWHeader::~MWAWHeader()
  * So far, we have identified
  */
 std::vector<MWAWHeader> MWAWHeader::constructHeader
-(MWAWInputStreamPtr input, shared_ptr<MWAWRSRCParser> /*rsrcParser*/)
+(MWAWInputStreamPtr input, std::shared_ptr<MWAWRSRCParser> /*rsrcParser*/)
 {
   std::vector<MWAWHeader> res;
   if (!input) return res;
@@ -79,6 +79,18 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="APBP") {
+      if (type=="APBL") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_DRAWINGTABLE, 1, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
+    else if (creator=="ARTX") { // Painter X
+      if (type=="RIFF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 10, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
     else if (creator=="BOBO") {
       if (type=="CWDB" || type=="CWD2" || type=="sWDB") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CLARISWORKS, 1, MWAWDocument::MWAW_K_DATABASE));
@@ -145,6 +157,12 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="DAD2") {
+      if (type=="drw2") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, 2, MWAWDocument::MWAW_K_DRAW));
+        return res;
+      }
+    }
     else if (creator=="Dc@P" || creator=="Dk@P") {
       if (type=="APPL") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_DOCMAKER, 1));
@@ -163,6 +181,18 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="FSPS" || creator=="FSDA") { // Fractal Design Painter or Dabbler
+      if (type=="RIFF") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 1, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
+    else if (creator=="FSX3") {
+      if (type=="RIFF") { // also FSFS list of uncompressed picture data for movie
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 3, MWAWDocument::MWAW_K_PAINT));
+        return res;
+      }
+    }
     else if (creator=="FWRT") {
       if (type=="FWRM") { // 1.7 ?
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_FULLWRITE, 1));
@@ -201,6 +231,20 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="JAZZ") {
+      if (type=="JWPD") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_JAZZLOTUS,1));
+        return res;
+      }
+      else if (type=="JWKS") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_JAZZLOTUS, 1, MWAWDocument::MWAW_K_SPREADSHEET));
+        return res;
+      }
+      else if (type=="JDBS") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_JAZZLOTUS, 1, MWAWDocument::MWAW_K_DATABASE));
+        return res;
+      }
+    }
     else if (creator=="LWTE") {
       if (type=="TEXT" || type=="ttro") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_LIGHTWAYTEXT,1));
@@ -356,6 +400,12 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
+    else if (creator=="PLAN") {
+      if (type=="MPBN") {
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTMULTIPLAN, 1, MWAWDocument::MWAW_K_SPREADSHEET));
+        return res;
+      }
+    }
     else if (creator=="PIXR") {
       if (type=="PX01") {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_PIXELPAINT, 1, MWAWDocument::MWAW_K_DRAW));
@@ -430,8 +480,8 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         return res;
       }
     }
-    else if (creator==std::string("St")+char(0xd8)+"l") { //  argh, not standart character
-      std::string type1=std::string("TEd")+char(0xb6); // argh, not standart character
+    else if (creator==std::string("St")+'\xd8'+"l") { //  argh, not standart character
+      std::string type1=std::string("TEd")+'\xb6'; // argh, not standart character
       if (type==type1) {
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_STYLE, 1));
         return res;
@@ -550,14 +600,13 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
 
   input->seek(0, librevenge::RVNG_SEEK_SET);
   int val[5];
-  for (int i = 0; i < 5; i++)
-    val[i] = int(input->readULong(2));
+  for (auto &v : val) v = int(input->readULong(2));
 
   // ----------- clearly discriminant ------------------
   if (val[2] == 0x424F && val[3] == 0x424F && (val[0]>>8) < 7) {
     MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a Claris Works file\n"));
     int vers= (val[0] >> 8);
-    static int const(typePos[7])= {0, 242, 248, 248, 256, 268, 278};
+    static int const typePos[7] = {0, 242, 248, 248, 256, 268, 278};
     int typeFile=-1;
     if (vers >= 1 && vers <= 6 && input->checkPosition(typePos[vers])) {
       input->seek(typePos[vers], librevenge::RVNG_SEEK_SET);
@@ -588,7 +637,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
   }
   if (val[0]==0x5772 && val[1]==0x6974 && val[2]==0x654e && val[3]==0x6f77) {
     input->seek(8, librevenge::RVNG_SEEK_SET);
-    int version = int(input->readLong(2));
+    auto version = int(input->readLong(2));
 
 #ifdef DEBUG
     bool ok = (version >= 0 && version <= 3);
@@ -625,6 +674,11 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MARINERWRITE, 1));
     return res;
   }
+  if (val[0]==0x000c && val[1]==0x1357 && (val[2]==0x13 || val[2]==0x14) && val[3]==0) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a Drawing Table file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_DRAWINGTABLE, 1, MWAWDocument::MWAW_K_DRAW));
+    return res;
+  }
   if (val[0]==0x4257 && val[1]==0x6b73 && val[2]==0x4257) {
     if (val[3]==0x7770) {
       MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a BeagleWorks file\n"));
@@ -739,6 +793,10 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     }
     // maybe we can also add outline: if (val[1]==0x5a4f && val[2]==0x4c4e)
   }
+  if (val[0]==0x11ab && val[1]==0 && val[2]==0x13e8 && val[3]==0) {
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTMULTIPLAN, 1, MWAWDocument::MWAW_K_SPREADSHEET));
+    return res;
+  }
   if (val[3]==6 && val[4]<6) {
     if (val[0]==0x4d44 && val[1]==0x4443 && val[2]==0x3230) {
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAFT, 4, MWAWDocument::MWAW_K_DRAW));
@@ -755,7 +813,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTWORKS, 4));
     else if (mainOle && mainOle->size()>18) {
       mainOle->seek(16, librevenge::RVNG_SEEK_SET);
-      int value=static_cast<int>(mainOle->readULong(2));
+      auto value=static_cast<int>(mainOle->readULong(2));
       switch (value) {
       case 2:
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTWORKS, 4, MWAWDocument::MWAW_K_DATABASE));
@@ -770,6 +828,14 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
         break;
       }
     }
+    if (!mainOle && input->getSubStreamByName("PP40"))
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_POWERPOINT, 4, MWAWDocument::MWAW_K_PRESENTATION));
+    else if (!mainOle && input->getSubStreamByName("PowerPoint Document") && input->getSubStreamByName("PersistentStorage Directory"))
+      res.push_back(MWAWHeader(MWAWDocument::MWAW_T_POWERPOINT, 7, MWAWDocument::MWAW_K_PRESENTATION));
+  }
+  if (val[0]==0 && val[1]==2 && val[2]==11) {
+    MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Jazz spreadsheet file\n"));
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_JAZZLOTUS, 1, MWAWDocument::MWAW_K_SPREADSHEET));
   }
 
   if ((val[0]==0xfe32 && val[1]==0) || (val[0]==0xfe34 && val[1]==0) ||
@@ -836,7 +902,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
   // ----------- other ------------------
   if (val[0]==0 && val[1]==0 && val[2]==0 && val[3]==0) {
     input->seek(8, librevenge::RVNG_SEEK_SET);
-    int value=static_cast<int>(input->readULong(1));
+    auto value=static_cast<int>(input->readULong(1));
     if (value==0x4 || value==0x44) {
       MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a WriteNow 1.0 or 2.0 file\n"));
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_WRITENOW, 2));
@@ -869,7 +935,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     }
     if (vers > 0 && input->size()>16) {
       input->seek(16, librevenge::RVNG_SEEK_SET);
-      int value=static_cast<int>(input->readULong(2));
+      auto value=static_cast<int>(input->readULong(2));
       switch (value) {
       case 1:
         res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MICROSOFTWORKS, vers));
@@ -891,6 +957,9 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
   if (val[0]==0x4d44 && input->size()>=512) { // maybe a MacDraw 0 file, will be check later
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACDRAW, 0, MWAWDocument::MWAW_K_DRAW));
   }
+  if (val[0]==2 && (val[1]&0xff)==0 && input->size() > 300) {
+    res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CORELPAINTER, 1, MWAWDocument::MWAW_K_PAINT));
+  }
   if (val[0] == 3 || val[0] == 6) {
     // version will be print by MacWrtParser::check
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACWRITE, val[0]));
@@ -901,7 +970,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
   }
   if (val[0] == 0x1000) {
     input->seek(10, librevenge::RVNG_SEEK_SET);
-    int value=static_cast<int>(input->readULong(2));
+    auto value=static_cast<int>(input->readULong(2));
     // 1: bitmap, 2: vectorized graphic
     if (value==1)
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_SUPERPAINT, 1, MWAWDocument::MWAW_K_PAINT));
@@ -920,7 +989,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       bool ok=true;
       int numSlash=0;
       for (int i=0; i<sSz; ++i) {
-        char c=char(input->readULong(1));
+        auto c=char(input->readULong(1));
         if (c>='0' && c<='9')
           continue;
         if (c=='/')
@@ -942,7 +1011,7 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
     if (!input->checkPosition(512*st+13))
       break;
     input->seek(512*st+10, librevenge::RVNG_SEEK_SET);
-    int value=static_cast<int>(input->readULong(2));
+    auto value=static_cast<int>(input->readULong(2));
     if (value==0x1101) {
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_APPLEPICT, 1, MWAWDocument::MWAW_K_DRAW));
       break;
@@ -968,9 +1037,9 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
           ok=false;
           break;
         }
-        int wh=static_cast<int>(input->readULong(1));
+        auto wh=static_cast<int>(input->readULong(1));
         if (wh>=0x81) {
-          int color=static_cast<int>(input->readULong(1));
+          auto color=static_cast<int>(input->readULong(1));
           // consider that repeat color is anormal...
           if (col+(0x101-wh)>72 || (lastColor>=0 && color==lastColor)) {
             ok=false;
@@ -995,14 +1064,40 @@ std::vector<MWAWHeader> MWAWHeader::constructHeader
       res.push_back(MWAWHeader(MWAWDocument::MWAW_T_MACPAINT, 1, MWAWDocument::MWAW_K_PAINT));
     }
   }
+  if ((val[0]>0 || val[1]>=86) && input->size()>0x900) {
+    input->seek(0x34, librevenge::RVNG_SEEK_SET);
+    int littleEndian=int(input->readULong(1));
+    if (littleEndian==1)
+      input->setReadInverted(true);
+    input->seek(1, librevenge::RVNG_SEEK_CUR);
+    int vers=int(input->readULong(2));
+    if (littleEndian<=1 && (vers==1 || vers==2)) {
+      input->seek(0, librevenge::RVNG_SEEK_SET);
+      int numZero=0;
+      for (int i=0; i<13; ++i) {
+        int len=int(input->readLong(4));
+        if (len<0) {
+          numZero=1000;
+          break;
+        }
+        if (len==0)
+          ++numZero;
+      }
+      if (numZero<=2+littleEndian) {
+        MWAW_DEBUG_MSG(("MWAWHeader::constructHeader: find a potential Canvas 2/3 file\n"));
+        res.push_back(MWAWHeader(MWAWDocument::MWAW_T_CANVAS, vers+1, MWAWDocument::MWAW_K_DRAW));
+      }
+    }
+    if (littleEndian==1)
+      input->setReadInverted(false);
+  }
   //
   //ok now look at the end of file
   //
   if (input->seek(-4, librevenge::RVNG_SEEK_END))
     return res;
   int lVal[2];
-  for (int i = 0; i < 2; i++)
-    lVal[i]=int(input->readULong(2));
+  for (auto &v : lVal) v=int(input->readULong(2));
   if (lVal[0] == 0x4E4C && lVal[1]==0x544F) // NLTO
     res.push_back(MWAWHeader(MWAWDocument::MWAW_T_ACTA, 2));
   else if (lVal[1]==0 && val[0]==1 && (val[1]==1||val[1]==2))
