@@ -59,11 +59,15 @@ namespace MacWrtParserInternal
 
 //! Document header
 struct FileHeader {
-  FileHeader() : m_hideFirstPageHeaderFooter(false), m_startNumberPage(1),
-    m_freeListPos(0), m_freeListLength(0), m_freeListAllocated(0),
-    m_dataPos(0)
+  FileHeader()
+    : m_hideFirstPageHeaderFooter(false)
+    , m_startNumberPage(1)
+    , m_freeListPos(0)
+    , m_freeListLength(0)
+    , m_freeListAllocated(0)
+    , m_dataPos(0)
   {
-    for (int i=0; i < 3; i++) m_numParagraphs[i] = 0;
+    for (auto &num : m_numParagraphs) num = 0;
   }
 
   friend std::ostream &operator<<(std::ostream &o, FileHeader const &header);
@@ -116,11 +120,17 @@ struct Information {
   enum Type { TEXT, RULER, GRAPHIC, PAGEBREAK, UNKNOWN };
 
   //! constructor
-  Information() :
-    m_type(UNKNOWN),  m_compressed(false), m_pos(), m_height(0),
-    m_justify(MWAWParagraph::JustificationLeft), m_justifySet(false),
-    m_data(),m_font()
-  {}
+  Information()
+    : m_type(UNKNOWN)
+    , m_compressed(false)
+    , m_pos()
+    , m_height(0)
+    , m_justify(MWAWParagraph::JustificationLeft)
+    , m_justifySet(false)
+    , m_data()
+    , m_font()
+  {
+  }
 
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Information const &info);
@@ -208,11 +218,18 @@ std::ostream &operator<<(std::ostream &o, Information const &info)
 ////////////////////////////////////////
 //! the windows structure
 struct WindowsInfo {
-  WindowsInfo() : m_startSel(), m_endSel(), m_posTopY(0),
-    m_informations(),
-    m_firstParagLine(), m_linesHeight(),
-    m_pageNumber(), m_date(), m_time()
-  { }
+  WindowsInfo()
+    : m_startSel()
+    , m_endSel()
+    , m_posTopY(0)
+    , m_informations()
+    , m_firstParagLine()
+    , m_linesHeight()
+    , m_pageNumber()
+    , m_date()
+    , m_time()
+  {
+  }
 
   /** small function used to recognized empty header or footer */
   bool isEmpty() const
@@ -221,12 +238,12 @@ struct WindowsInfo {
     if (m_pageNumber.x() >= 0 || m_date.x() >= 0 || m_time.x() >= 0)
       return false;
     if (m_informations.size() > 2) return false;
-    for (size_t i = 0; i < m_informations.size(); i++) {
-      switch (m_informations[i].m_type) {
+    for (auto const &info : m_informations) {
+      switch (info.m_type) {
       case Information::GRAPHIC:
         return false;
       case Information::TEXT:
-        if (m_informations[i].m_data.length() != 10)
+        if (info.m_data.length() != 10)
           return false;
         // empty line : ok
         break;
@@ -265,8 +282,13 @@ std::ostream &operator<<(std::ostream &o, WindowsInfo const &w)
 //! Internal: the state of a MacWrtParser
 struct State {
   //! constructor
-  State() : m_compressCorr(" etnroaisdlhcfp"), m_actPage(0), m_numPages(0), m_fileHeader(),
-    m_headerHeight(0), m_footerHeight(0)
+  State()
+    : m_compressCorr(" etnroaisdlhcfp")
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_fileHeader()
+    , m_headerHeight(0)
+    , m_footerHeight(0)
   {
   }
 
@@ -287,25 +309,23 @@ struct State {
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a MacWrtParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(MacWrtParser &pars, MWAWInputStreamPtr input, int zoneId) :
-    MWAWSubDocument(&pars, input, MWAWEntry()), m_id(zoneId) {}
-
-  //! destructor
-  virtual ~SubDocument() {}
-
-  //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const;
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
+  SubDocument(MacWrtParser &pars, MWAWInputStreamPtr const &input, int zoneId)
+    : MWAWSubDocument(&pars, input, MWAWEntry())
+    , m_id(zoneId)
   {
-    return !operator!=(doc);
   }
 
+  //! destructor
+  ~SubDocument() final {}
+
+  //! operator!=
+  bool operator!=(MWAWSubDocument const &doc) const final;
+
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
   //! the subdocument id
@@ -322,7 +342,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     MWAW_DEBUG_MSG(("MacWrtParserInternal::SubDocument::parse: unknown zone\n"));
     return;
   }
-  MacWrtParser *parser=dynamic_cast<MacWrtParser *>(m_parser);
+  auto *parser=dynamic_cast<MacWrtParser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("MacWrtParserInternal::SubDocument::parse: no parser\n"));
     return;
@@ -336,7 +356,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
 bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 {
   if (MWAWSubDocument::operator!=(doc)) return true;
-  SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+  auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
   if (!sDoc) return true;
   if (m_id != sDoc->m_id) return true;
   return false;
@@ -346,8 +366,9 @@ bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-MacWrtParser::MacWrtParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWTextParser(input, rsrcParser, header), m_state()
+MacWrtParser::MacWrtParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWTextParser(input, rsrcParser, header)
+  , m_state()
 {
   init();
 }
@@ -388,13 +409,13 @@ void MacWrtParser::newPage(int number)
 ////////////////////////////////////////////////////////////
 void MacWrtParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
-    checkHeader(0L);
+    checkHeader(nullptr);
     if (getRSRCParser()) {
       MWAWEntry corrEntry = getRSRCParser()->getEntry("STR ", 700);
       std::string corrString("");
@@ -504,20 +525,20 @@ bool MacWrtParser::createZones()
 
   // ok, we can find calculate the number of pages and the header and the footer height
   for (int i = 1; i < 3; i++) {
-    MacWrtParserInternal::WindowsInfo const &info = m_state->m_windows[i];
+    auto const &info = m_state->m_windows[i];
     if (info.isEmpty()) // avoid reserving space for empty header/footer
       continue;
     int height = 0;
-    for (size_t j=0; j < info.m_informations.size(); j++)
-      height+=info.m_informations[j].m_height;
+    for (auto const &inf : info.m_informations)
+      height+=inf.m_height;
     if (i == 1) m_state->m_headerHeight = height;
     else m_state->m_footerHeight = height;
   }
   int numPages = 0;
-  MacWrtParserInternal::WindowsInfo const &mainInfo = m_state->m_windows[0];
-  for (size_t i=0; i < mainInfo.m_informations.size(); i++) {
-    if (mainInfo.m_informations[i].m_pos.page() > numPages)
-      numPages = mainInfo.m_informations[i].m_pos.page();
+  auto const &mainInfo = m_state->m_windows[0];
+  for (auto &info : mainInfo.m_informations) {
+    if (info.m_pos.page() > numPages)
+      numPages = info.m_pos.page();
   }
   m_state->m_numPages = numPages+1;
 
@@ -550,7 +571,7 @@ bool MacWrtParser::createZonesV3()
     input->seek(pos+(i+1)*windowsSize, librevenge::RVNG_SEEK_SET);
   }
 
-  MacWrtParserInternal::FileHeader const &header = m_state->m_fileHeader;
+  auto const &header = m_state->m_fileHeader;
 
   for (int i = 0; i < 3; i++) {
     if (!readInformationsV3
@@ -573,11 +594,11 @@ bool MacWrtParser::createZonesV3()
   }
   for (int z = 0; z < 3; z++) {
     int numParag = header.m_numParagraphs[z];
-    MacWrtParserInternal::WindowsInfo &wInfo = m_state->m_windows[z];
+    auto &wInfo = m_state->m_windows[z];
     for (int p = 0; p < numParag; p++) {
       pos = input->tell();
-      int type = static_cast<int>(input->readLong(2));
-      int sz = static_cast<int>(input->readLong(2));
+      auto type = static_cast<int>(input->readLong(2));
+      auto sz = static_cast<int>(input->readLong(2));
       input->seek(pos+4+sz, librevenge::RVNG_SEEK_SET);
       if (sz < 0 || long(input->tell()) !=  pos+4+sz) {
         MWAW_DEBUG_MSG(("MacWrtParser::createZonesV3: pb with dataZone\n"));
@@ -589,8 +610,7 @@ bool MacWrtParser::createZonesV3()
       if (int(wInfo.m_informations.size()) <= p)
         continue;
       wInfo.m_informations[size_t(p)].m_data = entry;
-      MacWrtParserInternal::Information::Type newType =
-        MacWrtParserInternal::Information::UNKNOWN;
+      auto newType = MacWrtParserInternal::Information::UNKNOWN;
 
       switch ((type & 0x7)) {
       case 0:
@@ -618,10 +638,10 @@ bool MacWrtParser::createZonesV3()
   }
 
   int numPages = 0;
-  MacWrtParserInternal::WindowsInfo const &mainInfo = m_state->m_windows[0];
-  for (size_t i=0; i < mainInfo.m_informations.size(); i++) {
-    if (mainInfo.m_informations[i].m_pos.page() > numPages)
-      numPages = mainInfo.m_informations[i].m_pos.page();
+  auto const &mainInfo = m_state->m_windows[0];
+  for (auto const &info : mainInfo.m_informations) {
+    if (info.m_pos.page() > numPages)
+      numPages = info.m_pos.page();
   }
   m_state->m_numPages = numPages+1;
   return true;
@@ -634,9 +654,9 @@ bool MacWrtParser::sendWindow(int zone)
     return false;
   }
 
-  MacWrtParserInternal::WindowsInfo const &info = m_state->m_windows[zone];
+  auto const &info = m_state->m_windows[zone];
   size_t numInfo = info.m_informations.size();
-  int numPara = int(info.m_firstParagLine.size());
+  auto numPara = int(info.m_firstParagLine.size());
 
   if (version() <= 3 && zone == 0)
     newPage(1);
@@ -712,7 +732,7 @@ bool MacWrtParser::checkHeader(MWAWHeader *header, bool /*strict*/)
   }
   input->seek(0,librevenge::RVNG_SEEK_SET);
 
-  int vers = static_cast<int>(input->readULong(2));
+  auto vers = static_cast<int>(input->readULong(2));
   setVersion(vers);
 
   std::string vName("");
@@ -728,7 +748,7 @@ bool MacWrtParser::checkHeader(MWAWHeader *header, bool /*strict*/)
     MWAW_DEBUG_MSG(("MacWrtParser::checkHeader: unknown version\n"));
     return false;
   }
-  if (!vName.length()) {
+  if (vName.empty()) {
     MWAW_DEBUG_MSG(("Maybe a MacWrite file unknown version(%d)\n", vers));
   }
   else {
@@ -739,9 +759,9 @@ bool MacWrtParser::checkHeader(MWAWHeader *header, bool /*strict*/)
 
   if (vers <= 3) fHeader.m_dataPos = static_cast<int>(input->readULong(2));
 
-  for (int i = 0; i < 3; i++) {
-    int numParag = static_cast<int>(input->readLong(2));
-    fHeader.m_numParagraphs[i] = numParag;
+  for (int &numParagraph : fHeader.m_numParagraphs) {
+    auto numParag = static_cast<int>(input->readLong(2));
+    numParagraph = numParag;
     if (numParag < 0) {
       MWAW_DEBUG_MSG(("MacWrtParser::checkHeader: numParagraphs is negative : %d\n",
                       numParag));
@@ -881,22 +901,22 @@ bool MacWrtParser::readWindowsInfo(int wh)
   }
 
   int which = 2-wh;
-  MacWrtParserInternal::WindowsInfo &info = m_state->m_windows[which];
+  auto &info = m_state->m_windows[which];
   f << ": ";
 
   MWAWEntry informations;
   MWAWEntry lineHeightEntry;
 
   for (int i = 0; i < 2; i++) {
-    int x = static_cast<int>(input->readLong(2));
-    int y = static_cast<int>(input->readLong(2));
+    auto x = static_cast<int>(input->readLong(2));
+    auto y = static_cast<int>(input->readLong(2));
     if (i == 0) info.m_startSel = MWAWVec2i(x,y);
     else info.m_endSel = MWAWVec2i(x,y);
   }
 
   if (version() <= 3) {
     for (int i = 0; i < 2; i++) {
-      int val = static_cast<int>(input->readLong(2));
+      auto val = static_cast<int>(input->readLong(2));
       if (val) f << "unkn" << i << "=" << val << ",";
     }
   }
@@ -917,8 +937,8 @@ bool MacWrtParser::readWindowsInfo(int wh)
       << std::dec;
   }
   for (int i = 0; i < 3; i++) {
-    int x = static_cast<int>(input->readLong(2));
-    int y = static_cast<int>(input->readLong(2));
+    auto x = static_cast<int>(input->readLong(2));
+    auto y = static_cast<int>(input->readLong(2));
     if (i == 0) info.m_pageNumber = MWAWVec2i(x,y);
     else if (i == 1) info.m_date = MWAWVec2i(x,y);
     else info.m_time = MWAWVec2i(x,y);
@@ -929,7 +949,7 @@ bool MacWrtParser::readWindowsInfo(int wh)
     input->seek(6,librevenge::RVNG_SEEK_CUR); // unknown flags: ff ff ff ff ff 00
     f << "actFont=" << input->readLong(1) << ",";
     for (int i= 0; i < 2; i++) {
-      int val = static_cast<int>(input->readULong(1));
+      auto val = static_cast<int>(input->readULong(1));
       if (val==255) f << "f" << i << "=true,";
     }
     f << "flg=" << input->readLong(1);
@@ -983,9 +1003,11 @@ bool MacWrtParser::readLinesHeight(MWAWEntry const &entry, std::vector<int> &fir
   int numParag=0;
   while (input->tell() != endPos) {
     pos = input->tell();
-    int sz = static_cast<int>(input->readULong(2));
+    auto sz = static_cast<int>(input->readULong(2));
     if (pos+sz+2 > endPos) {
       MWAW_DEBUG_MSG(("MacWrtParser::readLinesHeight: find odd line\n"));
+      ascii().addPos(pos);
+      ascii().addNote("Entries(LineHeight):###");
       return false;
     }
 
@@ -995,7 +1017,7 @@ bool MacWrtParser::readLinesHeight(MWAWEntry const &entry, std::vector<int> &fir
     f.str("");
     f << "Entries(LineHeight)[" << entry.id() << "-" << ++numParag << "]:";
     for (int c = 0; c < sz; c++) {
-      int val = static_cast<int>(input->readULong(1));
+      auto val = static_cast<int>(input->readULong(1));
       if (val & 0x80) {
         val &= 0x7f;
         if (!heightOk || val==0) {
@@ -1047,7 +1069,7 @@ bool MacWrtParser::readInformationsV3(int numEntries, std::vector<MacWrtParserIn
     MacWrtParserInternal::Information info;
     f.str("");
     f << "Entries(Information)[" << i+1 << "]:";
-    int height = static_cast<int>(input->readLong(2));
+    auto height = static_cast<int>(input->readLong(2));
     info.m_height = height;
     if (info.m_height < 0) {
       info.m_height = 0;
@@ -1058,7 +1080,7 @@ bool MacWrtParser::readInformationsV3(int numEntries, std::vector<MacWrtParserIn
     else
       info.m_type = MacWrtParserInternal::Information::RULER;
 
-    int y = static_cast<int>(input->readLong(2));
+    auto y = static_cast<int>(input->readLong(2));
     info.m_pos=MWAWPosition(MWAWVec2f(0,float(y)), MWAWVec2f(0, float(height)), librevenge::RVNG_POINT);
     info.m_pos.setPage(static_cast<int>(input->readLong(1)));
     f << info;
@@ -1097,7 +1119,7 @@ bool MacWrtParser::readInformations(MWAWEntry const &entry, std::vector<MacWrtPa
     MWAW_DEBUG_MSG(("MacWrtParser::readInformations: entry size is odd\n"));
     return false;
   }
-  int numEntries = int((endPos-pos)/16);
+  auto numEntries = int((endPos-pos)/16);
   libmwaw::DebugStream f;
 
   input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1107,7 +1129,7 @@ bool MacWrtParser::readInformations(MWAWEntry const &entry, std::vector<MacWrtPa
     f.str("");
     f << "Entries(Information)[" << entry.id() << "-" << i+1 << "]:";
     MacWrtParserInternal::Information info;
-    int height = static_cast<int>(input->readLong(2));
+    auto height = static_cast<int>(input->readLong(2));
     if (height < 0) {
       info.m_type = MacWrtParserInternal::Information::GRAPHIC;
       height *= -1;
@@ -1118,13 +1140,13 @@ bool MacWrtParser::readInformations(MWAWEntry const &entry, std::vector<MacWrtPa
       info.m_type = MacWrtParserInternal::Information::TEXT;
     info.m_height = height;
 
-    int y = static_cast<int>(input->readLong(2));
-    int page = static_cast<int>(input->readULong(1));
+    auto y = static_cast<int>(input->readLong(2));
+    auto page = static_cast<int>(input->readULong(1));
     input->seek(3, librevenge::RVNG_SEEK_CUR); // unused
     info.m_pos = MWAWPosition(MWAWVec2f(0,float(y)), MWAWVec2f(0, float(height)), librevenge::RVNG_POINT);
     info.m_pos.setPage(page);
 
-    int paragStatus = static_cast<int>(input->readULong(1));
+    auto paragStatus = static_cast<int>(input->readULong(1));
     switch (paragStatus & 0x3) {
     case 0:
       info.m_justify = MWAWParagraph::JustificationLeft;
@@ -1145,11 +1167,11 @@ bool MacWrtParser::readInformations(MWAWEntry const &entry, std::vector<MacWrtPa
     info.m_justifySet = (paragStatus & 0x20);
 
     // other bits used internally
-    unsigned int highPos = static_cast<unsigned int>(input->readULong(1));
+    auto highPos = static_cast<unsigned int>(input->readULong(1));
     info.m_data.setBegin(long(highPos<<16)+long(input->readULong(2)));
     info.m_data.setLength(long(input->readULong(2)));
 
-    int paragFormat = static_cast<int>(input->readULong(2));
+    auto paragFormat = static_cast<int>(input->readULong(2));
     uint32_t flags = 0;
     // bit 1 = plain
     if (paragFormat&0x2) flags |= MWAWFont::boldBit;
@@ -1185,8 +1207,9 @@ bool MacWrtParser::readInformations(MWAWEntry const &entry, std::vector<MacWrtPa
     case 6:
       fontSize=14;
       break;
-    default:
-      MWAW_DEBUG_MSG(("MacWrtParser::readInformations: unknown size=7\n"));
+    default: // rare, but can appears on some empty lines
+      MWAW_DEBUG_MSG(("MacWrtParser::readInformations: unknown font size=7\n"));
+      f << "##fSize=7,";
     }
     if (fontSize) info.m_font.setSize(float(fontSize));
     if ((paragFormat >> 11)&0x1F) info.m_font.setId((paragFormat >> 11)&0x1F);
@@ -1233,7 +1256,7 @@ bool MacWrtParser::readText(MacWrtParserInternal::Information const &info,
   libmwaw::DebugStream f;
   f << "Entries(Text):";
 
-  int numChar = static_cast<int>(input->readULong(2));
+  auto numChar = static_cast<int>(input->readULong(2));
   std::string text("");
   if (!info.m_compressed) {
     if (numChar+2 >= entry.length()) {
@@ -1244,7 +1267,7 @@ bool MacWrtParser::readText(MacWrtParserInternal::Information const &info,
       text += char(input->readULong(1));
   }
   else {
-    std::string compressCorr = m_state->m_compressCorr;
+    std::string const &compressCorr = m_state->m_compressCorr;
 
     int actualChar = 0;
     bool actualCharSet = false;
@@ -1285,7 +1308,7 @@ bool MacWrtParser::readText(MacWrtParserInternal::Information const &info,
     actPos++;
   }
 
-  int formatSize = static_cast<int>(input->readULong(2));
+  auto formatSize = static_cast<int>(input->readULong(2));
   if ((formatSize%6)!=0 || actPos+2+formatSize > entry.end()) {
     MWAW_DEBUG_MSG(("MacWrtParser::readText: format is too long\n"));
     return false;
@@ -1296,11 +1319,11 @@ bool MacWrtParser::readText(MacWrtParserInternal::Information const &info,
   std::vector<MWAWFont> listFonts;
 
   for (int i = 0; i < numFormat; i++) {
-    int tPos = static_cast<int>(input->readULong(2));
+    auto tPos = static_cast<int>(input->readULong(2));
 
     MWAWFont font;
     font.setSize(float(input->readULong(1)));
-    int flag = static_cast<int>(input->readULong(1));
+    auto flag = static_cast<int>(input->readULong(1));
     uint32_t flags = 0;
     // bit 1 = plain
     if (flag&0x1) flags |= MWAWFont::boldBit;
@@ -1334,8 +1357,8 @@ bool MacWrtParser::readText(MacWrtParserInternal::Information const &info,
     if (readLinesHeight(hEntry, fParagLines, textLineHeight)) {
       lHeight = &textLineHeight;
       totalHeight = 0;
-      for (size_t i = 0; i < textLineHeight.size(); i++)
-        totalHeight+=textLineHeight[i];
+      for (auto height : textLineHeight)
+        totalHeight+=height;
     }
     else
       input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1365,11 +1388,22 @@ bool MacWrtParser::readText(MacWrtParserInternal::Information const &info,
         getTextListener()->setFont(listFonts[size_t(actFormat)]);
         actFormat++;
       }
-      unsigned char c = static_cast<unsigned char>(text[size_t(i)]);
+      auto c = static_cast<unsigned char>(text[size_t(i)]);
       if (c == 0x9)
         getTextListener()->insertTab();
       else if (c == 0xd)
         getTextListener()->insertEOL();
+      else if (c==0x11) // command key (found in some files)
+        getTextListener()->insertUnicode(0x2318);
+      else if (c==0x14) // apple logo: check me
+        getTextListener()->insertUnicode(0xf8ff);
+      else if (c<0x1f) {
+        // MacWrite allows to add "invalid" characters in the text
+        // (and do not display them), this does not imply that the
+        // file is invalid...
+        MWAW_DEBUG_MSG(("MacWrtParser::readText: find bad character %d at pos=0x%lx\n", int(c), version()<=3 ? entry.begin()-4 : entry.begin()));
+        f << "###[" << int(c) << "]";
+      }
       else
         getTextListener()->insertCharacter(c);
     }
@@ -1410,7 +1444,7 @@ bool MacWrtParser::readParagraph(MacWrtParserInternal::Information const &info)
 
   parag.m_margins[1] = double(input->readLong(2))/80.;
   parag.m_margins[2] = double(input->readLong(2))/80.;
-  int justify = static_cast<int>(input->readLong(1));
+  auto justify = static_cast<int>(input->readLong(1));
   switch (justify) {
   case 0:
     parag.m_justify = MWAWParagraph::JustificationLeft;
@@ -1428,19 +1462,19 @@ bool MacWrtParser::readParagraph(MacWrtParserInternal::Information const &info)
     f << "##justify=" << justify << ",";
     break;
   }
-  int numTabs = static_cast<int>(input->readLong(1));
+  auto numTabs = static_cast<int>(input->readLong(1));
   if (numTabs < 0 || numTabs > 10) {
     f << "##numTabs=" << numTabs << ",";
     numTabs = 0;
   }
-  int highspacing = static_cast<int>(input->readULong(1));
+  auto highspacing = static_cast<int>(input->readULong(1));
   if (highspacing==0x80) // 6 by line
     parag.setInterline(12, librevenge::RVNG_POINT);
   else if (highspacing) {
     f << "##highSpacing=" << std::hex << highspacing << std::dec << ",";
     MWAW_DEBUG_MSG(("MacWrtParser::readParagraph: high spacing bit set=%d\n", highspacing));
   }
-  int spacing = static_cast<int>(input->readLong(1));
+  auto spacing = static_cast<int>(input->readLong(1));
   if (spacing < 0)
     f << "#interline=" << 1.+spacing/2.0 << ",";
   else if (spacing)
@@ -1449,8 +1483,8 @@ bool MacWrtParser::readParagraph(MacWrtParserInternal::Information const &info)
 
   parag.m_tabs->resize(size_t(numTabs));
   for (size_t i = 0; i < size_t(numTabs); i++) {
-    int numPixel = static_cast<int>(input->readLong(2));
-    MWAWTabStop::Alignment align = MWAWTabStop::LEFT;
+    auto numPixel = static_cast<int>(input->readLong(2));
+    auto align = MWAWTabStop::LEFT;
     if (numPixel < 0) {
       align = MWAWTabStop::DECIMAL;
       numPixel *= -1;
@@ -1500,11 +1534,11 @@ bool MacWrtParser::readPageBreak(MacWrtParserInternal::Information const &info)
 
   f << "Entries(PageBreak):";
   for (int i = 0; i < 2; i++) {
-    int val = static_cast<int>(input->readLong(2));
+    auto val = static_cast<int>(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
   int dim[2]= {0,0};
-  for (int i = 0; i < 2; i++) dim[i] = static_cast<int>(input->readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input->readLong(2));
   f << "pageSize(?)=" << dim[0] << "x" << dim[1] << ",";
   f << "unk=" << input->readLong(2) << ","; // find 0xd
 
@@ -1544,8 +1578,7 @@ bool MacWrtParser::readGraphic(MacWrtParserInternal::Information const &info)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
   int dim[4];
-  for (int i = 0; i < 4; i++)
-    dim[i] = static_cast<int>(input->readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input->readLong(2));
   if (dim[2] < dim[0] || dim[3] < dim[1]) {
     MWAW_DEBUG_MSG(("MacWrtParser::readGraphic: bdbox is bad\n"));
     return false;
@@ -1554,7 +1587,7 @@ bool MacWrtParser::readGraphic(MacWrtParserInternal::Information const &info)
   f << "Entries(Graphic):";
 
   MWAWBox2f box;
-  MWAWPict::ReadResult res = MWAWPictData::check(input, int(entry.length()-8), box);
+  auto res = MWAWPictData::check(input, int(entry.length()-8), box);
   if (res == MWAWPict::MWAW_R_BAD) {
     MWAW_DEBUG_MSG(("MacWrtParser::readGraphic: can not find the picture\n"));
     return false;
@@ -1563,7 +1596,7 @@ bool MacWrtParser::readGraphic(MacWrtParserInternal::Information const &info)
 
   MWAWVec2f actualSize(float(dim[3]-dim[1]), float(dim[2]-dim[0])), naturalSize(actualSize);
   if (box.size().x() > 0 && box.size().y()  > 0) naturalSize = box.size();
-  MWAWPosition pictPos=MWAWPosition(MWAWVec2i(0,0),actualSize, librevenge::RVNG_POINT);
+  MWAWPosition pictPos=MWAWPosition(MWAWVec2f(0,0),actualSize, librevenge::RVNG_POINT);
   pictPos.setRelativePosition(MWAWPosition::Char);
   pictPos.setNaturalSize(naturalSize);
   f << pictPos;
@@ -1571,7 +1604,7 @@ bool MacWrtParser::readGraphic(MacWrtParserInternal::Information const &info)
   // get the picture
   input->seek(pos+8, librevenge::RVNG_SEEK_SET);
 
-  shared_ptr<MWAWPict> pict(MWAWPictData::get(input, int(entry.length()-8)));
+  std::shared_ptr<MWAWPict> pict(MWAWPictData::get(input, int(entry.length()-8)));
   if (pict) {
     if (getTextListener()) {
       MWAWParagraph para=getTextListener()->getParagraph();
@@ -1621,47 +1654,41 @@ bool MacWrtParser::checkFreeList()
     return true;
   MWAWInputStreamPtr input = getInput();
   long pos = m_state->m_fileHeader.m_freeListPos;
-  input->seek(pos+8, librevenge::RVNG_SEEK_SET);
-  if (long(input->tell()) != pos+8) {
-    MWAW_DEBUG_MSG(("MacWrtParser::checkFreeList: list is too short\n"));
+  if (!input->checkPosition(pos+m_state->m_fileHeader.m_freeListLength)) {
+    MWAW_DEBUG_MSG(("MacWrtParser::checkFreeList: zone is too short\n"));
     return false;
   }
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
   libmwaw::DebugStream f;
-  int num = 0;
-  while (!input->isEnd()) {
-    pos = input->tell();
-    long freePos = long(input->readULong(4));
-    long sz = long(input->readULong(4));
-
-    if (long(input->tell()) != pos+8) {
-      MWAW_DEBUG_MSG(("MacWrtParser::checkFreeList: list is too short\n"));
-      return false;
-    }
+  int N=int(m_state->m_fileHeader.m_freeListLength/8);
+  for (int n=0; n<N; ++n) {
+    pos=input->tell();
+    auto freePos = long(input->readULong(4));
+    auto sz = long(input->readULong(4));
 
     f.str("");
-    f << "Entries(FreeList)[" << ++num << "]:" << std::hex << freePos << "-" << sz;
+    f << "Entries(FreeList)[" << n << "]:" << std::hex << freePos << "-" << sz << std::dec;
     ascii().addPos(pos);
     ascii().addNote(f.str().c_str());
 
-    if (input->isEnd()) break;
-
-    input->seek(freePos+sz, librevenge::RVNG_SEEK_SET);
-    if (long(input->tell()) != freePos+sz) {
-      MWAW_DEBUG_MSG(("MacWrtParser::checkFreeList: bad free block\n"));
-      return false;
+    // if the file end by a free zone, pos+sz can be greater than the file size
+    if (!input->checkPosition(freePos+1)) {
+      if (!input->checkPosition(freePos)) {
+        MWAW_DEBUG_MSG(("MacWrtParser::checkFreeList: bad free block: \n"));
+        return false;
+      }
+      continue;
     }
-
     f.str("");
-    f << "Entries(FreeBlock)[" << num << "]:";
-
+    f << "Entries(FreeBlock)[" << n << "]:";
     ascii().addPos(freePos);
     ascii().addNote(f.str().c_str());
-
-    input->seek(pos+8, librevenge::RVNG_SEEK_SET);
   }
-
+  if (m_state->m_fileHeader.m_freeListLength!=m_state->m_fileHeader.m_freeListAllocated) {
+    ascii().addPos(input->tell());
+    ascii().addNote("Entries(FreeList)[end]:");
+  }
   return true;
 }
 
