@@ -70,8 +70,19 @@ struct Zone {
   //! a enum to define the diffent zone type
   enum Type { Z_Main, Z_Footnote, Z_Header, Z_Footer, Z_Unknown };
   //! constructor
-  Zone() : m_id(-1), m_fileId(0), m_type(Z_Unknown), m_endNote(false), m_height(0), m_RBpos(0,0), m_dim(),
-    m_pageDim(), m_pageTextDim(), m_section(), m_backgroundColor(MWAWColor::white()), m_extra("")
+  Zone()
+    : m_id(-1)
+    , m_fileId(0)
+    , m_type(Z_Unknown)
+    , m_endNote(false)
+    , m_height(0)
+    , m_RBpos(0,0)
+    , m_dim()
+    , m_pageDim()
+    , m_pageTextDim()
+    , m_section()
+    , m_backgroundColor(MWAWColor::white())
+    , m_extra("")
   {
   }
   //! operator<<
@@ -161,8 +172,13 @@ std::ostream &operator<<(std::ostream &o, Zone const &zone)
 //! Internal: the state of a MarinerWrtParser
 struct State {
   //! constructor
-  State() : m_zonesList(), m_fileToZoneMap(),
-    m_actPage(0), m_numPages(0), m_firstPageFooter(false), m_hasOddEvenHeaderFooter(false), m_headerHeight(0), m_footerHeight(0)
+  State()
+    : m_zonesList()
+    , m_fileToZoneMap()
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_firstPageFooter(false)
+    , m_hasOddEvenHeaderFooter(false)
   {
   }
 
@@ -175,39 +191,34 @@ struct State {
   bool m_firstPageFooter;
   /** a flag to know if we have odd/even header footer */
   bool m_hasOddEvenHeaderFooter;
-  int m_headerHeight /** the header height if known */,
-      m_footerHeight /** the footer height if known */;
 };
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a MarinerWrtParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(MarinerWrtParser &pars, MWAWInputStreamPtr input, int zoneId) :
-    MWAWSubDocument(&pars, input, MWAWEntry()), m_id(zoneId) {}
+  SubDocument(MarinerWrtParser &pars, MWAWInputStreamPtr const &input, int zoneId)
+    : MWAWSubDocument(&pars, input, MWAWEntry())
+    , m_id(zoneId)
+  {
+  }
 
   //! destructor
-  virtual ~SubDocument() {}
+  ~SubDocument() final {}
 
   //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const
+  bool operator!=(MWAWSubDocument const &doc) const final
   {
     if (MWAWSubDocument::operator!=(doc)) return true;
-    SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+    auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
     if (!sDoc) return true;
     if (m_id != sDoc->m_id) return true;
     return false;
   }
 
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
-  {
-    return !operator!=(doc);
-  }
-
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
   //! the subdocument id
@@ -220,7 +231,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
     MWAW_DEBUG_MSG(("MarinerWrtParserInternal::SubDocument::parse: no listener\n"));
     return;
   }
-  MarinerWrtParser *parser=dynamic_cast<MarinerWrtParser *>(m_parser);
+  auto *parser=dynamic_cast<MarinerWrtParser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("MarinerWrtParserInternal::SubDocument::parse: no parser\n"));
     return;
@@ -234,8 +245,12 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
 ////////////////////////////////////////////////////////////
 // constructor/destructor + basic interface ...
 ////////////////////////////////////////////////////////////
-MarinerWrtParser::MarinerWrtParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWTextParser(input, rsrcParser, header), m_state(), m_pageMarginsSpanSet(false), m_graphParser(), m_textParser()
+MarinerWrtParser::MarinerWrtParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWTextParser(input, rsrcParser, header)
+  , m_state()
+  , m_pageMarginsSpanSet(false)
+  , m_graphParser()
+  , m_textParser()
 {
   init();
 }
@@ -264,7 +279,7 @@ void MarinerWrtParser::init()
 MWAWVec2f MarinerWrtParser::getPageLeftTop() const
 {
   return MWAWVec2f(float(getPageSpan().getMarginLeft()),
-                   float(getPageSpan().getMarginTop()+m_state->m_headerHeight/72.0));
+                   float(getPageSpan().getMarginTop()));
 }
 
 MWAWSection MarinerWrtParser::getSection(int zId) const
@@ -332,14 +347,14 @@ void MarinerWrtParser::sendToken(int zoneId, long tokenId)
 ////////////////////////////////////////////////////////////
 void MarinerWrtParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L)) throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr)) throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
 
-    checkHeader(0L);
+    checkHeader(nullptr);
     ok = createZones();
     if (ok) {
       createDocument(docInterface);
@@ -389,7 +404,7 @@ void MarinerWrtParser::createDocument(librevenge::RVNGTextInterface *documentInt
   // look for an header/footer
   int headerId[4] = { -1, -1, -1, -1}, footerId[4] = { -1, -1, -1, -1};
   for (size_t z = 0; z < m_state->m_zonesList.size(); z++) {
-    MarinerWrtParserInternal::Zone const &zone=m_state->m_zonesList[z];
+    auto const &zone=m_state->m_zonesList[z];
     if (zone.m_type==MarinerWrtParserInternal::Zone::Z_Header) {
       if (zone.m_fileId < 4)
         headerId[zone.m_fileId]=int(z);
@@ -683,7 +698,7 @@ bool MarinerWrtParser::readZoneHeader(MarinerWrtEntry const &entry, int actId, b
       break;
     }
     case 3: { // 80000[01] or 800000[01]
-      uint32_t value=uint32_t(data.value(0));
+      auto value=uint32_t(data.value(0));
       if (value&0x2000000) {
         zone.m_endNote=true;
         value &= ~uint32_t(0x2000000);
@@ -694,7 +709,7 @@ bool MarinerWrtParser::readZoneHeader(MarinerWrtEntry const &entry, int actId, b
       break;
     }
     case 4: {
-      uint32_t v = uint32_t(data.value(0));
+      auto v = uint32_t(data.value(0));
       switch (v>>28) {
       case 0: // main
         zone.m_type = MarinerWrtParserInternal::Zone::Z_Main;
@@ -899,7 +914,7 @@ bool MarinerWrtParser::readZoneDim(MarinerWrtEntry const &entry, int zoneId)
   if (entry.m_fileType == 0xa && zoneId >= 0 &&
       zoneId < int(m_state->m_zonesList.size()) &&
       colPos.size() > 2 && int(colPos.size())==2*(entry.m_N-1)) {
-    size_t numCols=size_t(entry.m_N-1);
+    auto numCols=size_t(entry.m_N-1);
     MWAWSection &sec=m_state->m_zonesList[size_t(zoneId)].m_section;
     sec.m_columns.resize(numCols);
     for (size_t c=0; c < numCols; c++) {
@@ -931,7 +946,7 @@ bool MarinerWrtParser::readDocInfo(MarinerWrtEntry const &entry, int zoneId)
   decodeZone(dataList);
   input->popLimit();
 
-  int numDatas = int(dataList.size());
+  auto numDatas = int(dataList.size());
   if (numDatas < 60) {
     MWAW_DEBUG_MSG(("MarinerWrtParser::readDocInfo: find unexpected number of data\n"));
     return false;
@@ -956,7 +971,7 @@ bool MarinerWrtParser::readDocInfo(MarinerWrtEntry const &entry, int zoneId)
     }
     switch (j) {
     case 0: { // 15|40
-      uint32_t val=uint32_t(dt.value(0));
+      auto val=uint32_t(dt.value(0));
       if (val&0x4000) {
         if (zoneId==0)
           m_state->m_hasOddEvenHeaderFooter=true;
@@ -1205,7 +1220,7 @@ bool MarinerWrtParser::readZone13(MarinerWrtEntry const &entry, int)
         */
         f << "bl=[";
         input->seek(data.m_pos.begin(), librevenge::RVNG_SEEK_SET);
-        int N = int(data.m_pos.length()/2);
+        auto N = int(data.m_pos.length()/2);
         for (int k = 0; k < N; k++) {
           val = static_cast<int>(input->readLong(2));
           if (val) f << val << ",";
@@ -1349,7 +1364,7 @@ bool MarinerWrtParser::readEntryHeader(MarinerWrtEntry &entry)
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     return false;
   }
-  long length = (dataList[1]<<16)+dataList[2];
+  long length = dataList[1] >= 0 ? (dataList[1]<<16)+dataList[2] : -1;
   if (length < 0 || !input->checkPosition(input->tell()+length)) {
     MWAW_DEBUG_MSG(("MarinerWrtParser::readEntryHeader: the header data seems to short\n"));
     input->seek(pos, librevenge::RVNG_SEEK_SET);
@@ -1371,7 +1386,7 @@ bool MarinerWrtParser::readNumbersString(int num, std::vector<long> &res)
   MWAWInputStreamPtr input = getInput();
   std::string str("");
   while (!input->isEnd()) {
-    int ch = int(input->readULong(1));
+    auto ch = int(input->readULong(1));
     if (ch=='-' || (ch >= 'A' && ch <= 'F') || (ch >= '0' && ch <= '9')) {
       str += char(ch);
       continue;
@@ -1434,7 +1449,7 @@ bool MarinerWrtParser::decodeZone(std::vector<MarinerWrtStruct> &dataList, long
       break;
     MarinerWrtStruct data;
     data.m_filePos = pos;
-    int type = int(input->readULong(1));
+    auto type = int(input->readULong(1));
     data.m_type = (type&3);
     if (type == 3)
       return true;
@@ -1442,7 +1457,7 @@ bool MarinerWrtParser::decodeZone(std::vector<MarinerWrtStruct> &dataList, long
       break;
     if ((type>>4)==0xc) {
       if (input->isEnd()) break;
-      int num = int(input->readULong(1));
+      auto num = int(input->readULong(1));
       if (!num) break;
       if (numVal==0) {
         MWAW_DEBUG_MSG(("MarinerWrtParser::decodeZone: no previous data to copy\n"));
