@@ -61,10 +61,11 @@ namespace WriteNowTextInternal
 //! Internal: the fonts
 struct Font {
   //! the constructor
-  Font(): m_font()
+  Font()
+    : m_font()
   {
-    for (int i = 0; i < 3; i++) m_flags[i] = 0;
-    for (int i = 0; i < 2; i++) m_styleId[i] = -1;
+    for (auto &fl : m_flags) fl = 0;
+    for (auto &id : m_styleId) id = -1;
   }
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Font const &font)
@@ -89,16 +90,19 @@ struct Font {
 };
 
 /** Internal: class to store the paragraph properties */
-struct Paragraph : public MWAWParagraph {
+struct Paragraph final : public MWAWParagraph {
   //! constructor
-  Paragraph() : MWAWParagraph()
+  Paragraph()
+    : MWAWParagraph()
   {
     m_marginsUnit = librevenge::RVNG_POINT;
-    for (int i = 0; i < 8; i++)
-      m_values[i] = 0;
+    for (auto &val : m_values) val=0;
   }
+  Paragraph(Paragraph const &)=default;
+  Paragraph &operator=(Paragraph const &)=default;
+  Paragraph &operator=(Paragraph &&)=default;
   //! destructor
-  virtual ~Paragraph();
+  ~Paragraph() final;
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Paragraph const &ind)
   {
@@ -121,8 +125,8 @@ Paragraph::~Paragraph()
 struct Style {
   Style() : m_name(""), m_nextId(-1), m_font(), m_paragraph()
   {
-    for (int i = 0; i < 13; i++) m_values[i] = 0;
-    for (int i = 0; i < 6; i++) m_flags[i] = 0;
+    for (auto &val : m_values) val=0;
+    for (auto &fl : m_flags) fl=0;
   }
 
   //! operator<<
@@ -130,14 +134,14 @@ struct Style {
   {
     if (st.m_name.length()) o << st.m_name << ",";
     if (st.m_nextId >= 0) o << "nextId?=" << st.m_nextId << ",";
-    for (int i = 0; i < 13; i++) {
+    for (int i = 0; i < 12; i++) {
       if (st.m_values[i])
         o << "f" << i << "=" << st.m_values[i] << ",";
     }
     o << "flags=[" << std::hex;
-    for (int i = 0; i < 6; i++) {
-      if (st.m_flags[i])
-        o << st.m_flags[i] << ",";
+    for (auto fl : st.m_flags) {
+      if (fl)
+        o << fl << ",";
       else
         o << "_,";
     }
@@ -154,7 +158,7 @@ struct Style {
   /** the paragraph properties */
   Paragraph m_paragraph;
   /** some unknown value */
-  int m_values[13];
+  int m_values[12];
   /** some unknown flags */
   int m_flags[6];
 };
@@ -162,9 +166,12 @@ struct Style {
 ////////////////////////////////////////
 //! Internal: the token of a WriteNowText
 struct Token {
-  Token() : m_graphicZone(-1), m_box(), m_error("")
+  Token()
+    : m_graphicZone(-1)
+    , m_box()
+    , m_error("")
   {
-    for (int i = 0; i < 19; i++) m_values[i] = 0;
+    for (auto &val : m_values) val=0;
   }
 
   //! operator<<
@@ -208,17 +215,21 @@ struct Token {
 //! Internal: the table of a WriteNowText
 struct TableData {
   //! constructor
-  TableData() : m_type(-1), m_box(), m_color(MWAWColor::white()), m_error("")
+  TableData()
+    : m_type(-1)
+    , m_box()
+    , m_color(MWAWColor::white())
+    , m_error("")
   {
-    for (int i = 0; i < 4; i++) m_flags[i] = 1;
-    for (int i = 0; i < 10; i++) m_values[i] = 0;
+    for (auto &val : m_values) val=0;
+    for (auto &fl : m_flags) fl=0;
   }
 
   //! update a cell
   void updateCell(MWAWCell &cell) const
   {
     // as the cells can overlap a little, we build a new box
-    cell.setBdBox(MWAWBox2f(m_box.min(), m_box.max()-MWAWVec2i(1,1)));
+    cell.setBdBox(MWAWBox2f(MWAWVec2f(m_box.min()), MWAWVec2f(m_box.max()-MWAWVec2i(1,1))));
     cell.setBackgroundColor(m_color);
     for (int i=0; i<4; i++) {
       MWAWBorder border;
@@ -274,7 +285,7 @@ struct TableData {
     if (!table.m_color.isWhite())
       o << "color=" << table.m_color << ",";
     for (int i = 0; i < 4; i++) {
-      static char const *(wh[4])= {"T", "R", "B", "L"};
+      static char const *wh[4]= {"T", "R", "B", "L"};
       if (table.m_flags[i]&0xFF00)
         o << "#bFlags" << wh[i] << "[high]=" << (table.m_flags[i]>>8) << ",";
       // flags&0x80 : duplicated?
@@ -334,9 +345,11 @@ struct TableData {
 ////////////////////////////////////////
 //! Internal: structure used to store the content structure
 struct ContentZone {
-  ContentZone() : m_type(-1), m_value(0)
+  ContentZone()
+    : m_type(-1)
+    , m_value(0)
   {
-    for (int i = 0; i < 2; i++) m_pos[i] = -1;
+    for (auto &pos : m_pos) pos=-1;
   }
 
   //! operator<<
@@ -440,8 +453,14 @@ std::ostream &operator<<(std::ostream &o, ContentZone const &z)
 ////////////////////////////////////////
 //! Internal: structure used to store the content structure
 struct ContentZones {
-  ContentZones() : m_entry(), m_id(-1), m_type(0),
-    m_zonesList(), m_textCalledTypesList(), m_footnoteList(), m_sent(false)
+  ContentZones()
+    : m_entry()
+    , m_id(-1)
+    , m_type(0)
+    , m_zonesList()
+    , m_textCalledTypesList()
+    , m_footnoteList()
+    , m_sent(false)
   {
   }
   /** returns true if the entry corresponds to a page/column break */
@@ -452,8 +471,8 @@ struct ContentZones {
   int getNumberOfZonesWithType(int type) const
   {
     int res = 0;
-    for (size_t i = 0; i < m_zonesList.size(); i++)
-      if (m_zonesList[i].m_type == type) res++;
+    for (auto const &zone : m_zonesList)
+      if (zone.m_type == type) res++;
     return res;
   }
   //! the general zone
@@ -467,7 +486,7 @@ struct ContentZones {
   //! the list of type of text zone called by this zone
   std::vector<int> m_textCalledTypesList;
   //! a list to retrieve the footnote content
-  std::vector<shared_ptr<ContentZones> > m_footnoteList;
+  std::vector<std::shared_ptr<ContentZones> > m_footnoteList;
   //! true if this zone was sent to the listener
   mutable bool m_sent;
 };
@@ -475,36 +494,42 @@ struct ContentZones {
 
 ////////////////////////////////////////
 //! Internal: the cell of a WriteNowText
-struct Cell : public MWAWCell {
+struct Cell final : public MWAWCell {
   //! constructor
-  explicit Cell(WriteNowText &parser) : MWAWCell(), m_parser(parser),
-    m_zonesList(), m_footnoteList() {}
+  explicit Cell(WriteNowText &parser)
+    : MWAWCell()
+    , m_parser(parser)
+    , m_zonesList()
+    , m_footnoteList()
+  {
+  }
 
   //! send the content
-  bool sendContent(MWAWListenerPtr, MWAWTable &);
+  bool sendContent(MWAWListenerPtr, MWAWTable &) final;
   //! the text parser
   WriteNowText &m_parser;
   //! the list of zone
   std::vector<ContentZone> m_zonesList;
   //! a list to retrieve the footnote content
-  std::vector<shared_ptr<ContentZones> > m_footnoteList;
+  std::vector<std::shared_ptr<ContentZones> > m_footnoteList;
 };
 
 ////////////////////////////////////////
 ////////////////////////////////////////
-struct Table : public MWAWTable {
+struct Table final : public MWAWTable {
   //! constructor
-  Table() : MWAWTable()
+  Table()
+    : MWAWTable()
   {
   }
   //! destructor
-  virtual ~Table();
+  ~Table() final;
   //! return a cell corresponding to id
   Cell *get(int id)
   {
     if (id < 0 || id >= numCells()) {
       MWAW_DEBUG_MSG(("WriteNowTextInternal::Table::get: cell %d does not exists\n",id));
-      return 0;
+      return nullptr;
     }
     return static_cast<Cell *>(MWAWTable::get(id).get());
   }
@@ -518,19 +543,31 @@ Table::~Table()
 //! Internal: structure used to store the content structure
 struct Zone {
   //! constructor
-  Zone() : m_zones() {}
+  Zone()
+    : m_zones()
+  {
+  }
 
   //! a zones
-  std::vector<shared_ptr<ContentZones> > m_zones;
+  std::vector<std::shared_ptr<ContentZones> > m_zones;
 };
 
 ////////////////////////////////////////
 //! Internal: the state of a WriteNowText
 struct State {
   //! constructor
-  State() : m_version(-1), m_numColumns(1), m_numPages(1), m_actualPage(1),
-    m_paragraph(), m_header(), m_footer(),
-    m_localFIdMap(), m_styleMap(), m_styleList(), m_contentMap()
+  State()
+    : m_version(-1)
+    , m_numColumns(1)
+    , m_numPages(1)
+    , m_actualPage(1)
+    , m_paragraph()
+    , m_header()
+    , m_footer()
+    , m_localFIdMap()
+    , m_styleMap()
+    , m_styleList()
+    , m_contentMap()
   {
   }
 
@@ -544,10 +581,10 @@ struct State {
     if (m_version>=0 && m_version <= 2 && type==0) {
       res.m_margins[1] = 90.0;
       res.m_margins[2] = 60.0-28.0;
-      static double const(defPos[2])= {1.5,5.5};
-      for (int i=0; i < 2; i++) {
+      static double const defPos[2] = {1.5,5.5};
+      for (double i : defPos) {
         MWAWTabStop defTab;
-        defTab.m_position=defPos[i];
+        defTab.m_position=i;
         res.m_tabs->push_back(defTab);
       }
     }
@@ -566,11 +603,11 @@ struct State {
     return m_localFIdMap.find(localId)->second;
   }
   //! return the content corresponding to a pos
-  shared_ptr<ContentZones> getContentZone(long pos) const
+  std::shared_ptr<ContentZones> getContentZone(long pos) const
   {
-    std::map<long, shared_ptr<ContentZones> >::const_iterator it = m_contentMap.find(pos);
+    auto it = m_contentMap.find(pos);
     if (it == m_contentMap.end())
-      return shared_ptr<ContentZones>();
+      return std::shared_ptr<ContentZones>();
     return it->second;
   }
 
@@ -583,7 +620,7 @@ struct State {
   /** the paragraph properties */
   Paragraph m_paragraph;
   //! the header and the footer
-  shared_ptr<ContentZones> m_header, m_footer;
+  std::shared_ptr<ContentZones> m_header, m_footer;
   //! a map local fontId->fontId
   std::map<int, int> m_localFIdMap;
   //! the style indirection table
@@ -595,7 +632,7 @@ struct State {
   Zone m_mainZones[3];
 
   //! the list of contentZones
-  std::map<long, shared_ptr<ContentZones> > m_contentMap;
+  std::map<long, std::shared_ptr<ContentZones> > m_contentMap;
 };
 
 bool Cell::sendContent(MWAWListenerPtr, MWAWTable &)
@@ -612,9 +649,11 @@ bool Cell::sendContent(MWAWListenerPtr, MWAWTable &)
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-WriteNowText::WriteNowText(WriteNowParser &parser) :
-  m_parserState(parser.getParserState()), m_state(new WriteNowTextInternal::State),
-  m_entryManager(parser.m_entryManager), m_mainParser(&parser)
+WriteNowText::WriteNowText(WriteNowParser &parser)
+  : m_parserState(parser.getParserState())
+  , m_state(new WriteNowTextInternal::State)
+  , m_entryManager(parser.m_entryManager)
+  , m_mainParser(&parser)
 {
 }
 
@@ -641,7 +680,7 @@ int WriteNowText::numPages() const
     m_state->m_numPages = 1;
     return 1;
   }
-  shared_ptr<WriteNowTextInternal::ContentZones> mainContent = m_state->m_mainZones[0].m_zones[0];
+  auto mainContent = m_state->m_mainZones[0].m_zones[0];
   int nPages = 1+mainContent->getNumberOfZonesWithType(0x10);
   m_state->m_numPages = nPages;
   return nPages;
@@ -669,9 +708,7 @@ WriteNowEntry WriteNowText::getFooter() const
 ////////////////////////////////////////////////////////////
 bool WriteNowText::createZones()
 {
-  std::multimap<std::string, WriteNowEntry const *>::const_iterator iter;
-
-  iter = m_entryManager->m_typeMap.find("FontZone");
+  auto iter = m_entryManager->m_typeMap.find("FontZone");
   if (iter != m_entryManager->m_typeMap.end())
     readFontNames(*iter->second);
 
@@ -701,13 +738,11 @@ bool WriteNowText::createZones()
   m_mainParser->getColumnInfo(nCol, width);
   m_state->m_numColumns = nCol;
 
-  std::map<long, shared_ptr<WriteNowTextInternal::ContentZones> > listDone;
-  std::map<long, shared_ptr<WriteNowTextInternal::ContentZones> >::iterator it;
+  std::map<long, std::shared_ptr<WriteNowTextInternal::ContentZones> > listDone;
   /** we can now create the content zone and type them */
   for (int z = 2; z >= 0; z--) {
-    for (size_t i = 0; i < listData[z].size(); i++) {
-      WriteNowEntry data = listData[z][i];
-      it = listDone.find(data.begin());
+    for (auto data : listData[z]) {
+      auto it = listDone.find(data.begin());
       if (it != listDone.end()) {
         m_state->m_mainZones[z].m_zones.push_back(it->second);
         continue;
@@ -715,7 +750,7 @@ bool WriteNowText::createZones()
 
       // ok we must create the data
       data.setId(numData++);
-      shared_ptr<WriteNowTextInternal::ContentZones> zone;
+      std::shared_ptr<WriteNowTextInternal::ContentZones> zone;
       if (m_entryManager->add(data))
         zone = parseContent(m_entryManager->m_posMap.find(data.begin())->second);
       if (zone) {
@@ -726,16 +761,14 @@ bool WriteNowText::createZones()
     }
   }
   /* we can now recreate a zone 0 which contains all the main content zone*/
-  WriteNowTextInternal::Zone &mainZone = m_state->m_mainZones[0];
-  size_t numZones = mainZone.m_zones.size();
-  std::vector<shared_ptr<WriteNowTextInternal::ContentZones> > headerFooterList;
+  auto &mainZone = m_state->m_mainZones[0];
+  std::vector<std::shared_ptr<WriteNowTextInternal::ContentZones> > headerFooterList;
   std::vector<int> calledTypesList;
 
-  shared_ptr<WriteNowTextInternal::ContentZones> zone0(new WriteNowTextInternal::ContentZones);
+  std::shared_ptr<WriteNowTextInternal::ContentZones> zone0(new WriteNowTextInternal::ContentZones);
   zone0->m_id = -1;
   zone0->m_type = 0;
-  for (size_t z = 0; z < numZones; z++) {
-    shared_ptr<WriteNowTextInternal::ContentZones> content =  mainZone.m_zones[z];
+  for (auto content : mainZone.m_zones) {
     switch (content->m_type) {
     case 0:
       break;
@@ -768,12 +801,10 @@ bool WriteNowText::createZones()
   /* try to create the footnote and the header link */
   int numHeaderFooter = 0; // header+footer
 
-  shared_ptr<WriteNowTextInternal::ContentZones> content =  mainZone.m_zones[0];
+  auto content =  mainZone.m_zones[0];
   if (content->m_type != 0) return false;
 
-  size_t numCalledZones = calledTypesList.size();
-  for (size_t c = 0; c < numCalledZones; c++) {
-    int called = calledTypesList[c];
+  for (auto called : calledTypesList) {
     if (called < 5 || called > 6) {
       MWAW_DEBUG_MSG(("WriteNowText::createZones: unknown content %d\n", called));
       continue;
@@ -803,7 +834,7 @@ bool WriteNowText::createZones()
 ////////////////////////////////////////////////////////////
 // try to parse a text data zone
 ////////////////////////////////////////////////////////////
-shared_ptr<WriteNowTextInternal::ContentZones> WriteNowText::parseContent(WriteNowEntry const &entry)
+std::shared_ptr<WriteNowTextInternal::ContentZones> WriteNowText::parseContent(WriteNowEntry const &entry)
 {
   int vers = version();
 
@@ -814,7 +845,7 @@ shared_ptr<WriteNowTextInternal::ContentZones> WriteNowText::parseContent(WriteN
 
   if (!entry.valid()) {
     MWAW_DEBUG_MSG(("WriteNowText::parseContent: text zone size is invalid\n"));
-    return shared_ptr<WriteNowTextInternal::ContentZones>();
+    return std::shared_ptr<WriteNowTextInternal::ContentZones>();
   }
 
   MWAWInputStreamPtr &input= m_parserState->m_input;
@@ -824,7 +855,7 @@ shared_ptr<WriteNowTextInternal::ContentZones> WriteNowText::parseContent(WriteN
   // print the text zone data
   f << std::hex << "txtZone=[" << std::hex << entry.m_val[0] << std::dec
     <<",h=" << entry.m_val[1] << "],";
-  shared_ptr<WriteNowTextInternal::ContentZones> text;
+  std::shared_ptr<WriteNowTextInternal::ContentZones> text;
   if (vers >= 3) {
     if (entry.length() < 16) {
       MWAW_DEBUG_MSG(("WriteNowText::parseContent: text zone size is too short\n"));
@@ -877,7 +908,7 @@ shared_ptr<WriteNowTextInternal::ContentZones> WriteNowText::parseContent(WriteN
   while (long(input->tell()) < entry.end()) {
     long pos = input->tell();
     WriteNowTextInternal::ContentZone zone;
-    int c = static_cast<int>(input->readULong(1));
+    auto c = static_cast<int>(input->readULong(1));
     if (c == 0xf0) {
       MWAW_DEBUG_MSG(("WriteNowText::parseContent: find 0xf0 entry\n"));
       ascFile.addPos(pos);
@@ -979,7 +1010,7 @@ bool WriteNowText::parseZone(WriteNowEntry const &entry, std::vector<WriteNowEnt
   long endPos = entry.end();
   input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
   libmwaw::DebugFile &ascFile = m_parserState->m_asciiFile;
-  long sz = long(input->readULong(lengthSz));
+  auto sz = long(input->readULong(lengthSz));
   if (vers>2 && sz != entry.length()) {
     MWAW_DEBUG_MSG(("WriteNowText::parseZone: bad begin of last zone\n"));
     return false;
@@ -1013,10 +1044,10 @@ bool WriteNowText::parseZone(WriteNowEntry const &entry, std::vector<WriteNowEnt
       f << "f" << i << "=" << val << ",";
     }
   }
-  int numElts = int((entry.length()-headerSz)/dataSz);
+  auto numElts = int((entry.length()-headerSz)/dataSz);
   for (int elt=0; elt<numElts; elt++) {
     f << "entry" << elt << "=[";
-    int flags = static_cast<int>(input->readULong(1));
+    auto flags = static_cast<int>(input->readULong(1));
     /*  find
         40 44 45 : header ?
         60 61 64 65 : text ?
@@ -1107,7 +1138,7 @@ bool WriteNowText::readFontNames(WriteNowEntry const &entry)
     val = input->readLong(2);
     if (val) f << "f" << i << "=" << std::hex << val << std::dec << ",";
   }
-  int N=static_cast<int>(input->readULong(2));
+  auto N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   for (int i = 0; i < 2; i++) { // 0
     val = input->readLong(2);
@@ -1124,7 +1155,7 @@ bool WriteNowText::readFontNames(WriteNowEntry const &entry)
     pos = input->tell();
     f.str("");
     f << "Fonts[" << n << "]:";
-    int type = static_cast<int>(input->readULong(1));
+    auto type = static_cast<int>(input->readULong(1));
     switch (type) {
     case 0:
       f << "def,";
@@ -1166,7 +1197,7 @@ bool WriteNowText::readFontNames(WriteNowEntry const &entry)
       val = input->readLong(2);
       if (val) f << "f" << i << "=" << val << ",";
     }
-    int sz = static_cast<int>(input->readULong(1));
+    auto sz = static_cast<int>(input->readULong(1));
     if (pos+13+sz > entry.end()) {
       MWAW_DEBUG_MSG(("WriteNowText::readFontNames: can not read entry name : %d\n", n));
       return false;
@@ -1175,7 +1206,7 @@ bool WriteNowText::readFontNames(WriteNowEntry const &entry)
     bool ok = true;
     std::string name("");
     for (int i = 0; i < sz; i++) {
-      char ch = char(input->readULong(1));
+      auto ch = char(input->readULong(1));
       if (ch == '\0') {
         MWAW_DEBUG_MSG(("WriteNowText::readFontNames: pb with name field %d\n", n));
         ok = false;
@@ -1224,7 +1255,7 @@ bool WriteNowText::readFont(MWAWInputStream &input, bool inStyle, WriteNowTextIn
 
   font.m_font.setId(m_state->getFontId(static_cast<int>(input.readULong(2))));
   font.m_font.setSize(float(input.readULong(vers <= 2 ? 1 : 2)));
-  int flag = int(input.readULong(1));
+  auto flag = int(input.readULong(1));
   uint32_t flags=0;
   if (flag&0x1) flags |= MWAWFont::boldBit;
   if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -1265,7 +1296,7 @@ bool WriteNowText::readFont(MWAWInputStream &input, bool inStyle, WriteNowTextIn
   if (flag&0x40) font.m_font.setUnderlineStyle(MWAWFont::Line::Dot);
   if (flag&0x81) f << "#flag2=" << std::hex << (flag&0x81) << std::dec << ",";
 
-  int color = static_cast<int>(input.readULong(1)); // fixme find color map
+  auto color = static_cast<int>(input.readULong(1)); // fixme find color map
   if (color) {
     MWAWColor col;
     if (m_mainParser->getColor(color,col))
@@ -1273,7 +1304,7 @@ bool WriteNowText::readFont(MWAWInputStream &input, bool inStyle, WriteNowTextIn
     else
       f << "#colorId=" << color << ",";
   }
-  int heightDecal = static_cast<int>(input.readLong(2));
+  auto heightDecal = static_cast<int>(input.readLong(2));
   if (heightDecal)
     font.m_font.set(MWAWFont::Script(float(heightDecal), librevenge::RVNG_POINT));
 
@@ -1327,7 +1358,7 @@ bool WriteNowText::readParagraph(MWAWInputStream &input, WriteNowTextInternal::P
     for (int i = 0; i < 3; i++)
       ruler.m_values[actVal++] = static_cast<int>(input.readULong(2));
   }
-  int flag = static_cast<int>(input.readULong(1));
+  auto flag = static_cast<int>(input.readULong(1));
   switch (flag & 3) {
   case 0:
     break; // left
@@ -1355,7 +1386,7 @@ bool WriteNowText::readParagraph(MWAWInputStream &input, WriteNowTextInternal::P
     int tab = 0;
     while (!input.isEnd()) {
       MWAWTabStop newTab;
-      int newVal = static_cast<int>(input.readULong(2));
+      auto newVal = static_cast<int>(input.readULong(2));
       if (tab && newVal < previousVal) {
         MWAW_DEBUG_MSG(("WriteNowText::readParagraph: find bad tab pos\n"));
         f << "#tab[" << tab << ",";
@@ -1433,7 +1464,7 @@ bool WriteNowText::readStyles(WriteNowEntry const &entry)
     val = input->readLong(2);
     if (val) f << "f" << i << "=" << std::hex << val << std::dec << ",";
   }
-  int N=static_cast<int>(input->readULong(2));
+  auto N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   for (int i = 0; i < 2; i++) { // 0
     val = input->readLong(2);
@@ -1447,12 +1478,11 @@ bool WriteNowText::readStyles(WriteNowEntry const &entry)
   ascFile.addNote(f.str().c_str());
   std::vector<long> defPos;
   std::map<long, int> seenMap;
-  std::map<long, int>::iterator seenIt;
   for (int n = 0; n < N; n++) {
     pos = input->tell();
     f.str("");
     f << "Styles[" << n << "]:";
-    int type = static_cast<int>(input->readULong(1));
+    auto type = static_cast<int>(input->readULong(1));
     switch (type) {
     case 0:
       f << "def[named],";
@@ -1477,7 +1507,7 @@ bool WriteNowText::readStyles(WriteNowEntry const &entry)
     }
     val = long(input->readULong(4));
     if (type != 0x80 && pos+val != entry.end()) {
-      seenIt = seenMap.find(pos+val);
+      auto seenIt = seenMap.find(pos+val);
       int id = 0;
       if (seenIt != seenMap.end())
         id = seenIt->second;
@@ -1503,39 +1533,32 @@ bool WriteNowText::readStyles(WriteNowEntry const &entry)
     WriteNowTextInternal::Style style;
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     f.str("");
-    int type = static_cast<int>(input->readULong(1));
+    auto type = static_cast<int>(input->readULong(1));
     style.m_values[0] = int(type>>4);// find 1, 2 or 3
     style.m_values[1] = int(type&0xF); // always 0 ?
 
     /* f2: 0 or 0x30, 35, 36, 37, 38, 4d, 53
        f3: small number : maybe a previous id?
      */
-    for (int i = 2; i < 4; i++) {
+    for (int i = 2; i < 4; i++)
       style.m_values[i] = static_cast<int>(input->readULong(1));
-    }
     style.m_nextId = static_cast<int>(input->readLong(1))-1; // almost always n+1, so...
     /* f4: almost always 0, but one time f4=nextId
-       f5: always=0
+       f5-f9: always=0
      */
-    for (int i = 4; i < 6; i++) {
+    for (int i = 4; i < 6; i++)
       style.m_values[i] = static_cast<int>(input->readULong(1));
-    }
-    for (int i = 7; i < 10; i++) { // always 0
+    for (int i = 6; i < 9; i++) // always 0
       style.m_values[i] = static_cast<int>(input->readLong(2));
-    }
-    for (int i = 0; i < 6; i++) {
-      style.m_flags[i] = static_cast<int>(input->readULong(2));
-    }
-    for (int i = 10; i < 12; i++) { // always 0
+    for (auto &fl : style.m_flags) fl=static_cast<int>(input->readULong(2));
+    for (int i = 9; i < 11; i++) // always 0
       style.m_values[i] = static_cast<int>(input->readLong(2));
-    }
     int relPos[3];
-    for (int i = 0; i < 3; i++)
-      relPos[i] = static_cast<int>(input->readULong(2));
-    style.m_values[12] = static_cast<int>(input->readULong(2)); // another pos ?
+    for (auto &rPos : relPos) rPos = static_cast<int>(input->readULong(2));
+    style.m_values[11] = static_cast<int>(input->readULong(2)); // another pos ?
     if (relPos[0]) { // style name
       input->seek(pos+relPos[0], librevenge::RVNG_SEEK_SET);
-      int sz = static_cast<int>(input->readULong(1));
+      auto sz = static_cast<int>(input->readULong(1));
       if (!sz || pos+relPos[0]+1+sz > entry.end()) {
         MWAW_DEBUG_MSG(("WriteNowText::readStyles: can not read name for entry : %ld\n", long(n)));
         f << "name[length],";
@@ -1575,7 +1598,7 @@ bool WriteNowText::readStyles(WriteNowEntry const &entry)
       f.str("");
       f << "StylesData[" << n << ":ruler]:";
       input->seek(pos+relPos[2], librevenge::RVNG_SEEK_SET);
-      int sz = static_cast<int>(input->readULong(2));
+      auto sz = static_cast<int>(input->readULong(2));
       input->pushLimit(pos+relPos[2]+sz);
       WriteNowTextInternal::Paragraph ruler;
       if (readParagraph(*input, ruler)) {
@@ -1612,31 +1635,24 @@ bool WriteNowText::readToken(MWAWInputStream &input, WriteNowTextInternal::Token
   }
   input.seek(pos, librevenge::RVNG_SEEK_SET);
   int dim[4];
-  for (int i=0; i < 4; i++)
-    dim[i] = static_cast<int>(input.readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input.readLong(2));
   token.m_box = MWAWBox2i(MWAWVec2i(dim[1], dim[0]), MWAWVec2i(dim[3], dim[2]));
   int actVal = 0;
-  for (int st = 0; st < 2; st++) {
-    int dim0 = static_cast<int>(input.readLong(2));
+  for (auto &st : token.m_pos) {
+    auto dim0 = static_cast<int>(input.readLong(2));
     token.m_values[actVal++] = static_cast<int>(input.readLong(2)); // always 0
     token.m_values[actVal++] = static_cast<int>(input.readLong(2)); // always 0
-    int dim1 = static_cast<int>(input.readLong(2));
-    token.m_pos[st] = MWAWVec2i(dim1, -dim0);
+    auto dim1 = static_cast<int>(input.readLong(2));
+    st = MWAWVec2i(dim1, -dim0);
   }
   /*
     f0/f2:  0 or 1 ( very similar)
     f1/f3 : 0 or big number
-   */
-  for (int i = 0; i < 4; i++) {
-    token.m_values[actVal++] = static_cast<int>(input.readULong(2));
-  }
-  /*
     f8=1, f9=14
     other 0
-  */
-  for (int i = 0; i < 10; i++) {
-    token.m_values[actVal++] = static_cast<int>(input.readLong(2));
-  }
+   */
+  for (int i = 0; i < 14; i++)
+    token.m_values[actVal++] = static_cast<int>(input.readULong(2));
   token.m_graphicZone = static_cast<int>(input.readLong(2));
   return true;
 }
@@ -1647,8 +1663,7 @@ bool WriteNowText::readTokenV2(MWAWInputStream &input, WriteNowTextInternal::Tok
 
   long actPos = input.tell();
   int dim[2];
-  for (int i=0; i < 2; i++)
-    dim[i] = static_cast<int>(input.readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input.readLong(2));
   MWAWVec2i box(dim[1], dim[0]);
   token.m_box=MWAWBox2i(MWAWVec2i(0,0), box);
   // we need to get the size, so...
@@ -1659,7 +1674,7 @@ bool WriteNowText::readTokenV2(MWAWInputStream &input, WriteNowTextInternal::Tok
   if (sz <= 0) return false;
   input.seek(actPos+4, librevenge::RVNG_SEEK_SET);
   MWAWInputStreamPtr ip(&input,MWAW_shared_ptr_noop_deleter<MWAWInputStream>());
-  shared_ptr<MWAWPict> pict(MWAWPictData::get(ip, static_cast<int>(sz)));
+  std::shared_ptr<MWAWPict> pict(MWAWPictData::get(ip, static_cast<int>(sz)));
   if (!pict) {
     MWAW_DEBUG_MSG(("WriteNowParser::readTokenV2: can not read the picture\n"));
     return false;
@@ -1668,7 +1683,7 @@ bool WriteNowText::readTokenV2(MWAWInputStream &input, WriteNowTextInternal::Tok
 
   MWAWPosition pictPos;
   if (box.x() > 0 && box.y() > 0) {
-    pictPos=MWAWPosition(MWAWVec2f(0,0),box, librevenge::RVNG_POINT);
+    pictPos=MWAWPosition(MWAWVec2f(0,0),MWAWVec2f(box), librevenge::RVNG_POINT);
     pictPos.setNaturalSize(pict->getBdBox().size());
   }
   else
@@ -1707,23 +1722,21 @@ bool WriteNowText::readTable(MWAWInputStream &input, WriteNowTextInternal::Table
   int actVal = 0;
   table.m_values[actVal++] = static_cast<int>(input.readLong(1));
   table.m_values[actVal++] = static_cast<int>(input.readLong(1));
-  int backColor = static_cast<int>(input.readULong(1));
+  auto backColor = static_cast<int>(input.readULong(1));
   MWAWColor col;
   if (m_mainParser->getColor(backColor,col))
     table.m_color = col;
   else {
     MWAW_DEBUG_MSG(("WriteNowText::readTable: can not read backgroundColor: %d\n", backColor));
   }
-  for (int i = 0; i < 4; i++) {
-    table.m_flags[i] = static_cast<int>(input.readULong(1)); // 0x80, 0x81, 5, 0 : border flag ?
+  for (int &flag : table.m_flags) {
+    flag = static_cast<int>(input.readULong(1)); // 0x80, 0x81, 5, 0 : border flag ?
     table.m_values[actVal++] = static_cast<int>(input.readLong(1)); //  always 0 ?
   }
   for (int i = 0; i < 3; i++)
     table.m_values[actVal++] = static_cast<int>(input.readLong(2)); // alway 0
-
   int dim[4];
-  for (int i = 0; i < 4; i++)
-    dim[i] = static_cast<int>(input.readLong(2));
+  for (auto &d : dim) d = static_cast<int>(input.readLong(2));
   table.m_box = MWAWBox2i(MWAWVec2i(dim[1], dim[0]), MWAWVec2i(dim[3], dim[2]));
   table.m_values[actVal++] = static_cast<int>(input.readLong(2));
 
@@ -1734,19 +1747,18 @@ bool WriteNowText::readTable(MWAWInputStream &input, WriteNowTextInternal::Table
 // send data to the listener
 bool WriteNowText::send(WriteNowEntry const &entry)
 {
-  shared_ptr<WriteNowTextInternal::ContentZones> text
-  (m_state->getContentZone(entry.begin()));
+  std::shared_ptr<WriteNowTextInternal::ContentZones> text(m_state->getContentZone(entry.begin()));
   if (!text) {
     MWAW_DEBUG_MSG(("WriteNowText::send: can not find entry\n"));
     return false;
   }
-  WriteNowTextInternal::Paragraph ruler=m_state->getDefaultParagraph(text->m_type);
+  auto ruler=m_state->getDefaultParagraph(text->m_type);
   text->m_sent = true;
   return send(text->m_zonesList, text->m_footnoteList, ruler);
 }
 
 bool WriteNowText::send(std::vector<WriteNowTextInternal::ContentZone> &listZones,
-                        std::vector<shared_ptr<WriteNowTextInternal::ContentZones> > &footnoteList,
+                        std::vector<std::shared_ptr<WriteNowTextInternal::ContentZones> > &footnoteList,
                         WriteNowTextInternal::Paragraph &ruler)
 {
   MWAWTextListenerPtr listener=m_parserState->m_textListener;
@@ -1763,12 +1775,11 @@ bool WriteNowText::send(std::vector<WriteNowTextInternal::ContentZone> &listZone
   bool rulerSet = false;
   int numLineTabs = 0, actTabs = 0, numFootnote = 0;
 
-  shared_ptr<WriteNowTextInternal::Table> table;
-  shared_ptr<WriteNowTextInternal::Cell> cell;
+  std::shared_ptr<WriteNowTextInternal::Table> table;
+  std::shared_ptr<WriteNowTextInternal::Cell> cell;
 
   MWAWInputStreamPtr &input= m_parserState->m_input;
-  for (size_t z = 0; z < listZones.size(); z++) {
-    WriteNowTextInternal::ContentZone const &zone = listZones[z];
+  for (auto const &zone : listZones) {
     if (table && cell) {
       bool done = true;
       switch (zone.m_type) {
@@ -1847,7 +1858,7 @@ bool WriteNowText::send(std::vector<WriteNowTextInternal::ContentZone> &listZone
     librevenge::RVNGBinaryData data;
     input->seek(zone.m_pos[0],librevenge::RVNG_SEEK_SET); //000a2f
     while (int(input->tell()) < zone.m_pos[1]) {
-      int ch = static_cast<int>(input->readULong(1));
+      auto ch = static_cast<int>(input->readULong(1));
       if (ch == 0xf0) {
         ch = static_cast<int>(input->readULong(1));
         if (ch & 0xf0) {
@@ -1862,7 +1873,7 @@ bool WriteNowText::send(std::vector<WriteNowTextInternal::ContentZone> &listZone
 
     f.str("");
     if (zone.m_type == 0) {
-      long sz = long(data.size());
+      auto sz = long(data.size());
       const unsigned char *buffer = data.getDataBuffer();
       if (!buffer && sz) {
         MWAW_DEBUG_MSG(("WriteNowText::send: can find data buffer\n"));
@@ -1873,7 +1884,7 @@ bool WriteNowText::send(std::vector<WriteNowTextInternal::ContentZone> &listZone
         numLineTabs = int(ruler.m_tabs->size());
         rulerSet = true;
       }
-      for (int i = 0; i < sz; i++) {
+      for (long i = 0; i < sz; i++) {
         unsigned char c = *(buffer++);
         f << c;
         switch (c) {
@@ -2061,16 +2072,16 @@ void WriteNowText::sendZone(int id)
       listener->openSection(sec);
     }
   }
-  WriteNowTextInternal::Zone &mZone = m_state->m_mainZones[id];
-  WriteNowTextInternal::Paragraph ruler=m_state->getDefaultParagraph(id);
+  auto &mZone = m_state->m_mainZones[id];
+  auto ruler=m_state->getDefaultParagraph(id);
   if (id==0)
     listener->setParagraph(ruler);
-  for (size_t i = 0; i < mZone.m_zones.size(); i++) {
-    if (mZone.m_zones[i]->m_sent) continue;
-    if (id == 0 && mZone.m_zones[i]->m_type) continue;
+  for (auto &zone : mZone.m_zones) {
+    if (zone->m_sent) continue;
+    if (id == 0 && zone->m_type) continue;
     if (id) ruler=m_state->getDefaultParagraph(id);
-    send(mZone.m_zones[i]->m_zonesList, mZone.m_zones[i]->m_footnoteList, ruler);
-    mZone.m_zones[i]->m_sent = true;
+    send(zone->m_zonesList, zone->m_footnoteList, ruler);
+    zone->m_sent = true;
   }
 }
 
